module margo

use mkinds
use mdates
use mncio
use mobs

implicit none

integer(i4)                :: npr_argo

contains

subroutine get_argo_files(cdate,mxnpr,mxlvl,prtyp,prlvls,prlon,prlat,prtime,prdpth,prtemp,prsaln,nstart,nend)

implicit none

character(len=*)           :: cdate
integer(i4)                :: mxnpr
integer(i4)                :: mxlvl
integer(i4)                :: nstart
integer(i4)                :: nend

integer(i4)                :: prtyp(mxnpr)
integer(i4)                :: prlvls(mxnpr)
real(r4)                   :: prlon(mxnpr)
real(r4)                   :: prlat(mxnpr)
real(r4)                   :: prtime(mxnpr)
real(r4)                   :: prdpth(mxnpr,mxlvl)
real(r4)                   :: prtemp(mxnpr,mxlvl)
real(r4)                   :: prsaln(mxnpr,mxlvl)

character(:),allocatable  :: argoFile
character(:),allocatable  :: atlFile
character(:),allocatable  :: pacFile
character(:),allocatable  :: indFile
character(:),allocatable  :: rfile

integer(i4)               :: nprof
integer(i4)               :: nlvl
integer(i4)               :: nvalid
integer(i4)               :: fcount
integer(i4)               :: vcount


real(r4),allocatable      :: lon(:)
real(r4),allocatable      :: lat(:)

character(:),allocatable  :: pres_qc
character(:),allocatable  :: temp_qc
character(:),allocatable  :: saln_qc
character(:),allocatable  :: juld_qc
character(:),allocatable  :: pos_qc
character(:),allocatable  :: data_mode

real(r4),allocatable      :: pres(:,:)
real(r4),allocatable      :: temp(:,:)
real(r4),allocatable      :: saln(:,:)
integer(i4),allocatable   :: valid(:)

real(r4),allocatable      :: pres1d(:)
real(r4),allocatable      :: temp1d(:)
real(r4),allocatable      :: saln1d(:)
real(r4),allocatable      :: pres1d_qc(:)
real(r4),allocatable      :: temp1d_qc(:)
real(r4),allocatable      :: saln1d_qc(:)
real(r8),allocatable      :: juldr8(:)
real(r4)                  :: date_offset
real(r4)                  :: t_fill_value
real(r4)                  :: s_fill_value
real(r4)                  :: p_fill_value

logical(bl)               :: lexist
logical(bl)               :: lrexist
logical(bl)               :: laexist
logical(bl)               :: lpexist
logical(bl)               :: liexist
logical(bl)               :: dt_argo
logical(bl)               :: rt_argo
integer(i4)               :: spresent



integer(i4),parameter     :: nprofmx=10000
character(cl)             :: argoFileNames(3)
integer(i4)               :: nargoFiles
integer(i4)               :: total_files_processed
integer(i4)               :: nqc_fail
integer(i4)               :: nqc_shortfail
integer(i4)               :: nqc_gapsfail

! reject counts
integer(i4)               :: nuniq
real(r4)                  :: ipr,jpr
real(r4)                  :: hday
integer(i4)               :: i,j,k,l,ii
integer(i4)               :: kamx(2)
integer(i4)               :: kmx

date_offset=ndays(01,01,1950,12,31,1900)
total_files_processed=0

! analysix date

call cymd2hday(cdate,hday)

nuniq=0
fcount=0


atlFile=trim(argo_profiles_location)//"/atl/"//cdate(1:8)//"_prof.nc"
pacFile=trim(argo_profiles_location)//"/pac/"//cdate(1:8)//"_prof.nc"
indFile=trim(argo_profiles_location)//"/ind/"//cdate(1:8)//"_prof.nc"
rFile=trim(argo_profiles_location)//"/nrt/"//"R"//cdate(1:8)//"_prof.nc"

nargoFiles=0
nqc_fail=0
nqc_gapsfail=0
nqc_shortfail=0

! firxt try delayed time files

INQUIRE(FILE=atlFile, EXIST=laexist)
INQUIRE(FILE=pacFile, EXIST=lpexist)
INQUIRE(FILE=indFile, EXIST=liexist)
INQUIRE(FILE=rFile, EXIST=lrexist)

if(laexist) then
        nargoFiles=nargoFiles+1
        argoFileNames(nargoFiles)=atlFile
endif

if(lpexist) then
        nargoFiles=nargoFiles+1
        argoFileNames(nargoFiles)=pacFile
endif 

if(liexist) then
         nargoFiles=nargoFiles+1
         argoFileNames(nargoFiles)=indFile
endif   

dt_argo = laexist .or. lpexist .or. liexist
rt_argo = lrexist .and. (.not. dt_argo)

if (dt_argo) then
        write(*,*) "....delayed time files available...."
        write(*,*) "....will use the foll files.... "
        
        do i=1,nargoFiles
        write(*,*) "...."//trim(argoFileNames(i))
        enddo 

else if (rt_argo) then
        write(*,*) "....delayed time files not available but real-time file present...."
        write(*,*) "....will use the foll...."
        nargofiles=1
        argoFileNames(1)=rFile
        write(*,*) "...."//trim(argoFileNames(1))
else
        write(*,*) "....no argo files...."
        return
endif

fcount=nstart

do ii=1,nargoFiles
 argoFile=argoFilenames(ii)

call nciopn(argoFile,ncid)
call ncioin(argoFile,ncid,"N_PROF",nprof)
call ncioin(argoFile,ncid,"N_LEVELS",nlvl)

call ncioiv(argoFile,ncid,"PSAL",spresent)
if(spresent==0) then


if(allocated(lon)) deallocate(lon,lat,juldr8,juld_qc,pos_qc,data_mode)
allocate(lon(nprof),lat(nprof),juldr8(nprof))
allocate(character(len=nprof) :: juld_qc)
allocate(character(len=nprof) :: pos_qc)
allocate(character(len=nprof) :: data_mode)

call nciorv(argoFile,ncid,"LONGITUDE",lon)
call nciorv(argoFile,ncid,"LATITUDE",lat)
call nciorv(argoFile,ncid,"JULD",juldr8)
call nciorv(argoFile,ncid,"JULD_QC",juld_qc)
call nciorv(argoFile,ncid,"POSITION_QC",pos_qc)
call nciorv(argoFile,ncid,"DATA_MODE",data_mode)

! fcount ix the final number of profiles retained
! it ix updated and used for file names
do i=1,nprof
        total_files_processed=total_files_processed+1
        if(allocated(pres)) deallocate(pres,temp,saln,pres_qc,temp_qc,saln_qc,valid)
        allocate(pres(nlvl,1),temp(nlvl,1),saln(nlvl,1),valid(nlvl))
        
        allocate(character(len=nlvl) :: pres_qc)
        allocate(character(len=nlvl) :: temp_qc)
        allocate(character(len=nlvl) :: saln_qc)

        valid=0

        !if(rt_argo) then
        if(data_mode(i:i) .eq. 'R') then
          call nciorv(argoFile,ncid,"PRES",pres,[1,i],[nlvl,1])
          call nciorv(argoFile,ncid,"PRES_QC",pres_qc,[1,i],[nlvl,1])
          call nciora(argoFile,ncid,"PRES","_FillValue",p_fill_value)
          call nciorv(argoFile,ncid,"TEMP",temp,[1,i],[nlvl,1])
          call nciora(argoFile,ncid,"TEMP","_FillValue",t_fill_value)
          call nciorv(argoFile,ncid,"TEMP_QC",temp_qc,[1,i],[nlvl,1])

          call nciorv(argoFile,ncid,"PSAL",saln,[1,i],[nlvl,1])
          call nciora(argoFile,ncid,"PSAL","_FillValue",s_fill_value)

          call nciorv(argoFile,ncid,"PSAL_QC",saln_qc,[1,i],[nlvl,1])

        !else if(dt_argo) then 

        else if(data_mode(i:i) .eq. 'A' .or. data_mode(i:i) .eq. 'D' ) then

          call nciorv(argoFile,ncid,"PRES_ADJUSTED",pres,[1,i],[nlvl,1])
          call nciora(argoFile,ncid,"PRES_ADJUSTED","_FillValue",p_fill_value)
          call nciorv(argoFile,ncid,"PRES_ADJUSTED_QC",pres_qc,[1,i],[nlvl,1])
          call nciorv(argoFile,ncid,"TEMP_ADJUSTED",temp,[1,i],[nlvl,1])
          call nciora(argoFile,ncid,"TEMP_ADJUSTED","_FillValue",t_fill_value)
          call nciorv(argoFile,ncid,"TEMP_ADJUSTED_QC",temp_qc,[1,i],[nlvl,1])

          call nciorv(argoFile,ncid,"PSAL_ADJUSTED",saln,[1,i],[nlvl,1])
          call nciora(argoFile,ncid,"PSAL_ADJUSTED","_FillValue",s_fill_value)

          call nciorv(argoFile,ncid,"PSAL_ADJUSTED_QC",saln_qc,[1,i],[nlvl,1])

        endif

        kamx=maxloc(PRES,mask=PRES<p_fill_value)
        kmx=kamx(1)

        ! check the QC flags and flag invalid profiles
        do k=1,kmx
!        print *, k, pres_qc(k:k),temp_qc(k:k),saln_qc(k:k),i
          if((pres_qc(k:k)=="1" .or. pres_qc(k:k)=="2" )  .and. &
                  (temp_qc(k:k)=="1" .or. temp_qc(k:k)=="2") .and. &
                  (saln_qc(k:k)=="1" .or. saln_qc(k:k)=="2") ) then
            valid(k)=1
          endif
        enddo

        nvalid=sum(valid)

        !print *,i,minval(PRES,mask=PRES<p_fill_value),maxval(PRES,mask=PRES<p_fill_value),nvalid

        if(kmx<2) nqc_shortfail=nqc_shortfail+1
        if(nvalid==0 .or.  juld_qc(i:i) .ne. "1" .or. pos_qc(i:i) .ne."1" ) then
                nqc_fail=nqc_fail+1
        endif

        ! select valid profiles with position qc and time qc with flags 1 

        if(nvalid>1 .and. juld_qc(i:i) .eq. "1" .and. pos_qc(i:i) .eq."1") then
        allocate(pres1d(nvalid),temp1d(nvalid),saln1d(nvalid),pres1d_qc(nvalid),temp1d_qc(nvalid),saln1d_qc(nvalid))
        vcount=0
        do k=1,kmx
         if(valid(k)==1) then
          vcount=vcount+1       
          pres1d(vcount)=pres(k,1)
          temp1d(vcount)=temp(k,1)
          saln1d(vcount)=saln(k,1)
          pres1d_qc(vcount)=0.
          temp1d_qc(vcount)=0.
          saln1d_qc(vcount)=0.
         endif
        enddo
        
        ! select profiles without gaps in the vertical, even one gap in either salinity or temperature
        ! disqualifies the profile 
        
        if(all(temp1d /=t_fill_value) .and. all(saln1d /=s_fill_value)) then
               fcount=fcount+1
               prtyp(fcount)=3
               prlvls(fcount)=vcount
               prlon(fcount)=lon(i)
               prlat(fcount)=lat(i)
               prdpth(fcount,1:vcount)=pres1d(1:vcount)
               prtemp(fcount,1:vcount)=temp1d(1:vcount)
               prsaln(fcount,1:vcount)=saln1d(1:vcount)
               prtime(fcount)=real(juldr8(i),r4)+date_offset
       else
               nqc_gapsfail=nqc_gapsfail+1
       endif
      
         deallocate(pres1d,temp1d,saln1d,pres1d_qc,temp1d_qc,saln1d_qc)
         endif
        enddo ! profiles in a file

        else
                Write(*,*) ".... skipping file - no salinity"

        endif

         call nciocl(argoFile,ncid)

       enddo ! no of files

! Report some stats after basic  QC.

nend=fcount
if(nstart==0) then
        npr_argo=fcount
else
        npr_argo=fcount-nstart+1
endif

Write(*,*) "....Total no of ARGO profiles processed:.... ", total_files_processed
Write(*,*) "....ARGO QC flagged: .... ", nqc_fail
Write(*,*) "....Very short profiles: .... ", nqc_shortfail
Write(*,*) "....ARGO profiles with gaps:.... ", nqc_gapsfail  
Write(*,*) "....Total no of ARGO profiles selected:.... ", npr_argo

end subroutine




subroutine get_cargo_files(cdate,mxnpr,mxlvl,prtyp,prlvls,prlon,prlat,prtime,prdpth,prtemp,prsaln,nstart,nend)

implicit none

character(len=*)           :: cdate
integer(i4)                :: mxnpr
integer(i4)                :: mxlvl
integer(i4)                :: nstart
integer(i4)                :: nend

integer(i4)                :: prtyp(mxnpr)
integer(i4)                :: prlvls(mxnpr)
real(r4)                   :: prlon(mxnpr)
real(r4)                   :: prlat(mxnpr)
real(r4)                   :: prtime(mxnpr)
real(r4)                   :: prdpth(mxnpr,mxlvl)
real(r4)                   :: prtemp(mxnpr,mxlvl)
real(r4)                   :: prsaln(mxnpr,mxlvl)

character(:),allocatable  :: argoFile
character(:),allocatable  :: atlFile
character(:),allocatable  :: pacFile
character(:),allocatable  :: indFile
character(:),allocatable  :: rfile

integer(i4)               :: nprof
integer(i4)               :: nlvl
integer(i4)               :: nvalid
integer(i4)               :: fcount
integer(i4)               :: vcount

real(r4),allocatable      :: lon(:)
real(r4),allocatable      :: lat(:)

integer(i4),allocatable  :: pres_qc(:,:)
integer(i4),allocatable  :: temp_qc(:,:)
integer(i4),allocatable  :: saln_qc(:,:)
integer(i4),allocatable  :: juld_qc(:)
integer(i4),allocatable  :: pos_qc(:)
integer(i4),allocatable  :: data_mode(:)

real(r4),allocatable      :: pres(:,:)
real(r4),allocatable      :: temp(:,:)
real(r4),allocatable      :: saln(:,:)
integer(i4),allocatable   :: valid(:)
real(r8),allocatable      :: juld(:)
real(r4)                  :: date_offset
real(r4)                  :: t_fill_value
real(r4)                  :: s_fill_value
real(r4)                  :: p_fill_value

real(r4),allocatable      :: pres1d(:)
real(r4),allocatable      :: temp1d(:)
real(r4),allocatable      :: saln1d(:)
real(r4),allocatable      :: pres1d_qc(:)
real(r4),allocatable      :: temp1d_qc(:)
real(r4),allocatable      :: saln1d_qc(:)


integer(i4)               :: nqc_fail
integer(i4)               :: nqc_shortfail
integer(i4)               :: nqc_gapsfail

! reject counts
integer(i4)               :: nuniq
real(r4)                  :: ipr,jpr
real(r4)                  :: hday
integer(i4)               :: i,j,k,l,ii
integer(i4)               :: kamx(2,1)
integer(i4)               :: kmx
logical(bl)               :: lexist

date_offset=ndays(01,01,1950,12,31,1900)

! analysix date

call cymd2hday(cdate,hday)
!call read_obs_location()

nuniq=0
fcount=0
nqc_fail=0
nqc_gapsfail=0
nqc_shortfail=0


fcount=nstart
argoFile=trim(argo_profiles_location)//"/cargo/cargo_l2_"//cdate(1:8)//".nc"

INQUIRE(FILE=argoFile, EXIST=lexist)

if(lexist) then

call nciopn(argoFile,ncid)
call ncioin(argoFile,ncid,"N_PROF",nprof)
call ncioin(argoFile,ncid,"N_LEVELS",nlvl)


allocate(lon(nprof))
allocate(lat(nprof))
allocate(juld(nprof))
allocate(juld_qc(nprof))
allocate(pos_qc(nprof))
allocate(data_mode(nprof))
allocate(pres(nlvl,nprof))
allocate(pres_qc(nlvl,nprof))
allocate(temp(nlvl,nprof))
allocate(temp_qc(nlvl,nprof))
allocate(saln(nlvl,nprof))
allocate(saln_qc(nlvl,nprof))

call nciorv(argoFile,ncid,"LONGITUDE",lon)
call nciorv(argoFile,ncid,"LATITUDE",lat)
call nciorv(argoFile,ncid,"JULD",juld)
call nciorv(argoFile,ncid,"JULD_QC",juld_qc)
call nciorv(argoFile,ncid,"POSITION_QC",pos_qc)
call nciorv(argoFile,ncid,"DATA_MODE",data_mode)
          call nciorv(argoFile,ncid,"PRES",pres)
          call nciorv(argoFile,ncid,"PRES_QC",pres_qc)
          call nciora(argoFile,ncid,"PRES","_FillValue",p_fill_value)
          call nciorv(argoFile,ncid,"TEMP",temp)
          call nciora(argoFile,ncid,"TEMP","_FillValue",t_fill_value)
          call nciorv(argoFile,ncid,"TEMP_QC",temp_qc)
          call nciorv(argoFile,ncid,"PSAL",saln)
          call nciora(argoFile,ncid,"PSAL","_FillValue",s_fill_value)
          call nciorv(argoFile,ncid,"PSAL_QC",saln_qc)
          call nciocl(argoFile,ncid)

          ! fcount ix the final number of profiles retained
! it ix updated and used for file names
allocate(valid(nlvl))
do i=1,nprof
valid=0

        kmx=nlvl
        do k=1,nlvl
         if(PRES(k,i)>=p_fill_value) then
           kmx=k-1
           exit
         endif
        enddo

        ! check the QC flags and flag invalid profiles
        do k=1,kmx
!        print *, k, pres_qc(k:k),temp_qc(k:k),saln_qc(k:k),i
          if((pres_qc(k,i)==1 .or. pres_qc(k,i)==2 )  .and. &
                  (temp_qc(k,i)==1 .or. temp_qc(k,i)==2) .and. &
                  (saln_qc(k,i)==1 .or. saln_qc(k,i)==2) ) then
            valid(k)=1
          endif
        enddo

                nvalid=sum(valid)

        !print *,i,minval(PRES,mask=PRES<p_fill_value),maxval(PRES,mask=PRES<p_fill_value),nvalid

        if(kmx<2) nqc_shortfail=nqc_shortfail+1
        if(nvalid==0 .or.  juld_qc(i) /= 1 .or. pos_qc(i) /= 1 ) then
                nqc_fail=nqc_fail+1
        endif

        ! select valid profiles with position qc and time qc with flags 1

        if(nvalid>1 .and. juld_qc(i) == 1 .and. pos_qc(i) == 1) then
        allocate(pres1d(nvalid),temp1d(nvalid),saln1d(nvalid))
        vcount=0
        do k=1,kmx
         if(valid(k)==1) then
          vcount=vcount+1
          pres1d(vcount)=pres(k,i)
          temp1d(vcount)=temp(k,i)
          saln1d(vcount)=saln(k,i)
          !pres1d_qc(vcount)=0.
          !temp1d_qc(vcount)=0.
          !saln1d_qc(vcount)=0.
         endif
        enddo

        ! select profiles without gaps in the vertical, even one gap in either salinity or temperature
        ! disqualifies the profile

        if(all(temp1d /=t_fill_value) .and. all(saln1d /=s_fill_value)) then
               fcount=fcount+1
               prtyp(fcount)=3
               prlvls(fcount)=vcount
               prlon(fcount)=lon(i)
               prlat(fcount)=lat(i)
               prdpth(fcount,1:vcount)=pres1d(1:vcount)
               prtemp(fcount,1:vcount)=temp1d(1:vcount)
               prsaln(fcount,1:vcount)=saln1d(1:vcount)
               prtime(fcount)=real(juld(i),r4)+date_offset
       else
               nqc_gapsfail=nqc_gapsfail+1
       endif

         deallocate(pres1d,temp1d,saln1d)
         endif
        enddo ! profiles in a file


! Report some stats after basic  QC.

nend=fcount
if(nstart==0) then
        npr_argo=fcount
else
        npr_argo=fcount-nstart+1
endif

Write(*,*) "....Total no of ARGO profiles processed:.... ", nprof
Write(*,*) "....ARGO QC flagged: .... ", nqc_fail
Write(*,*) "....Very short profiles: .... ", nqc_shortfail
Write(*,*) "....ARGO profiles with gaps:.... ", nqc_gapsfail
Write(*,*) "....Total no of ARGO profiles selected:.... ", npr_argo

else
npr_argo=0

Write(*,*) "....no profiles .... "        
endif


end subroutine



















end module
