module marmor

use mkinds
use mdates
use mncio
use msearch
use msplint
use mseawater
use mobs

implicit none

integer(i4)                :: ar_nx
integer(i4)                :: ar_ny
integer(i4)                :: ar_nz
integer(i4)                :: ar_nt


real(r4),allocatable       :: ar_lon(:)
real(r4),allocatable       :: ar_lat(:)
real(r4),allocatable       :: ar_dpth(:)
real(r4),allocatable       :: ar_time(:)

real(r4)                   :: ar_scl_t,ar_off_t
real(r4)                   :: ar_scl_s,ar_off_s
real(r4)                   :: ar_FillValue
real(r4)                   :: d_spval
real(r4)                   :: vtime
real(r4)                   :: ar_date_offset

character(cl),allocatable  :: dclim_files(:)
real(r4),allocatable       :: dclim_Ftime(:)
integer(i4)                :: n_dclim_files
character(:),allocatable    :: F_pfx

contains

  subroutine armor_build_lookup(data_loc,cdate)


  character(*)                :: data_loc
  character(*)                :: cdate

  character(:),allocatable    :: F_in
  character(:),allocatable    :: F_out
  character(:),allocatable    :: cfname
  character(cl)               :: buffer
  real(r4)                    :: ar_time(1)
  integer(i4)                 :: n

  ar_date_offset=ndays(01,01,1950,12,31,1900)
  f_pfx=trim(data_loc)//"/"//cdate(1:4)
  F_in=f_pfx//"/dclim.txt"
  f_out=f_pfx//"/dclim_lookup.txt"

  open(unit=10,file=F_in,status="unknown")
  n=0
     DO
     READ(10,"(A)",END=100) buffer
     n=n+1
    ! cfname=f_pfx//"/"//trim(buffer)
    ! call nciopn(cfname,fid)
    ! call nciorv(cfname,fid,"time",ar_time)
    ! call nciocl(cfname,fid)
    ! ar_time=ar_time/24.0+ar_date_offset
    ! write(20,'(a120,5x,F10.2)') cfname,ar_time
     ENDDO
100  CONTINUE
     CLOSE(10)
    ! CLOSE(20)
    ! WRITE(*,*) "Total No of Members: ", n

  open(unit=10,file=F_in,status="unknown")
  open(unit=20,file=F_out,status="unknown")
  write(20,*) n

    n=0
     DO
     READ(10,"(A)",END=120) buffer
     n=n+1
     cfname=f_pfx//"/"//trim(buffer)
     call nciopn(cfname,fid)
     call nciorv(cfname,fid,"time",ar_time)
     call nciocl(cfname,fid)
     ar_time=ar_time/24.0+ar_date_offset
     write(20,'(F10.2,5X,a120)') ar_time,trim(cfname)
     ENDDO
120  CONTINUE
     CLOSE(10)
     CLOSE(20)
     WRITE(*,*) "Total No of dclim Files: ", n

  end subroutine


  subroutine init_armor(dclim_location,cdate,iflag)

        implicit none
        character(*)              :: dclim_location
        character(*)              :: cdate
        integer(i4)               :: iflag        
        logical(bl)               :: lexist
        character(:),allocatable  :: dclim_lookup_table
        character(:),allocatable  :: cfname
        integer(i4)               :: i

        f_pfx=trim(dclim_location)//"/"//cdate(1:4)
        dclim_lookup_table=f_pfx//"/dclim_lookup.txt"
       

        inquire(file=dclim_lookup_table,exist=lexist)


        if(lexist) then

 
        open(unit=10,file=dclim_lookup_table,status="unknown")
        read(10,*) n_dclim_files

        allocate(dclim_files(n_dclim_files),dclim_Ftime(n_dclim_files))

        do i=1,n_dclim_files
         read(10,'(F10.2,5X,a120)') dclim_Ftime(i),dclim_files(i) 
        !read(10,'(a95,5x,F10.2)') dclim_files(i),dclim_Ftime(i)  
         !print *, armor_files(i)(len_trim(armor_location)+7:len_trim(armor_files(i))),armor_Ftime(i) 
        !print *,i, adjustL(dclim_files(i)),dclim_Ftime(i)
        enddo


        cfname=adjustL(trim(dclim_files(1)))

      
        ar_date_offset=ndays(01,01,1950,12,31,1900)
        call nciopn(cfname,fid)
        call ncioin(cfname,fid,"longitude",ar_nx)
        call ncioin(cfname,fid,"latitude",ar_ny)
        call ncioin(cfname,fid,"depth",ar_nz)
        call ncioin(cfname,fid,"time",ar_nt)

        allocate(ar_lon(ar_nx))
        allocate(ar_lat(ar_ny))
        allocate(ar_dpth(ar_nz))
        allocate(ar_time(ar_nt))

        call nciorv(cfname,fid,"longitude",ar_lon)
        call nciorv(cfname,fid,"latitude",ar_lat)
        call nciorv(cfname,fid,"depth",ar_dpth)
        call nciorv(cfname,fid,"time",ar_time)

               ar_lon(1)=0.0
               ar_lon(ar_nx)=360.0

        call nciora(cfname,fid,"to","scale_factor",ar_scl_t)
        call nciora(cfname,fid,"to","add_offset",ar_off_t)

        call nciora(cfname,fid,"so","scale_factor",ar_scl_s)
        call nciora(cfname,fid,"so","add_offset",ar_off_s)
        call nciora(cfname,fid,"so","_FillValue",ar_FillValue)

        ar_time=ar_time/24.0+ar_date_offset
        call nciocl(cfname,fid)
        iflag=0
        else

        Write(*,*) "....No reference state loookup file found...."
        iflag=1
        return


        endif         

end subroutine

subroutine get_dclim_profiles(cdtg,n,lon,lat,cnz,cdpth,ct,cs)

        character(*)  :: cdtg
        integer(i4)   :: n,cnz
        real(r4)      :: lon(n)
        real(r4)      :: lat(n)

        real(r4)      :: cdpth(cnz)
        real(r4)      :: c_t(ar_nz)
        real(r4)      :: c_s(ar_nz)

        real(r4)      :: ct(n,cnz)
        real(r4)      :: cs(n,cnz)

        real(r4)      :: ptlon
        real(r4)      :: vtime
        real(r4)      :: qr
        real(r4)      :: lr
        integer(i4)   :: li
        character(:),allocatable  :: cfname0,cfname1

        real(r4),allocatable      :: ar_t0(:,:,:),ar_s0(:,:,:)
        real(r4),allocatable      :: ar_t1(:,:,:),ar_s1(:,:,:)
        real(r4),allocatable      :: ar_t(:,:,:),ar_s(:,:,:)


        integer(i4)    :: i,j,k,ii,jj

        d_spval=2.00**100
        call cymd2hday(cdtg,vtime)
!       vtime is in the future but less then 2 weeks
        if(vtime>maxval(dclim_Ftime) .and. vtime<maxval(dclim_Ftime)+14.0) then
        
        allocate(ar_t(ar_nx,ar_ny,ar_nz))
        allocate(ar_s(ar_nx,ar_ny,ar_nz))
        cfname0=adjustL(trim(dclim_files(n_dclim_files)))       
        call nciopn(cfname0,fid)
        call nciorv(cfname0,fid,"to",ar_t)
        call nciorv(cfname0,fid,"so",ar_s)
        call nciocl(cfname0,fid)
        
        where(ar_t>=ar_fillValue) ar_t=d_spval
        where(ar_s>=ar_fillValue) ar_s=d_spval
        where(ar_t<d_spval) ar_t=ar_t*ar_scl_t+ar_off_t
        where(ar_s<d_spval) ar_s=ar_s*ar_scl_s+ar_off_s
        
                
        do k=1,ar_nz
         do j=1,ar_ny
          do i=1,ar_nx

          if(ar_t(i,j,k)<d_spval .and. ar_s(i,j,k)<d_spval) then
                  ar_t(i,j,k)=theta (ar_s(i,j,k), ar_t(i,j,k),ar_dpth(k),0.0)
          else
                  ar_t(i,j,k)=d_spval
                  ar_s(i,j,k)=d_spval
          endif
          enddo
          enddo
          enddo

          do i=1,n

           ptlon=lon(i)
           if(ptlon<0.0) ptlon=ptlon+360.
           ii=int(ridx(ar_lon,ptlon))
           jj=int(ridx(ar_lat,lat(i)))

           c_t(:)=ar_t(ii,jj,:)
           c_s(:)=ar_s(ii,jj,:)


           do k=2,ar_nz
           if(c_t(k)>=d_spval) then
                   c_t(k)=c_t(k-1)
                   c_s(k)=c_s(k-1)
           endif

           enddo

           

           call akspline(ar_nz,ar_dpth,c_t,cnz,cdpth,ct(i,:))
           call akspline(ar_nz,ar_dpth,c_s,cnz,cdpth,cs(i,:))

           enddo
!       vtime is in between the first and last times

        else if (vtime>=minval(dclim_Ftime) .and. vtime<maxval(dclim_Ftime)) then
        allocate(ar_t0(ar_nx,ar_ny,ar_nz))
        allocate(ar_s0(ar_nx,ar_ny,ar_nz))

        allocate(ar_t1(ar_nx,ar_ny,ar_nz))
        allocate(ar_s1(ar_nx,ar_ny,ar_nz))

        allocate(ar_t(ar_nx,ar_ny,ar_nz))
        allocate(ar_s(ar_nx,ar_ny,ar_nz))
        lr=ridx(dclim_Ftime,vtime)
        li=int(lr)
        qr=lr-li

        !print *, lr,li,qr
        cfname0=adjustL(trim(dclim_files(li)))
        cfname1=adjustL(trim(dclim_files(li+1)))


        call nciopn(cfname0,fid)
        call nciorv(cfname0,fid,"to",ar_t0)
        call nciorv(cfname0,fid,"so",ar_s0)
        call nciocl(cfname0,fid)

        call nciopn(cfname1,fid)
        call nciorv(cfname1,fid,"to",ar_t1)
        call nciorv(cfname1,fid,"so",ar_s1)
        call nciocl(cfname1,fid)

        !unpack variables

        where(ar_t0>=ar_fillValue) ar_t0=d_spval
        where(ar_s0>=ar_fillValue) ar_s0=d_spval

        where(ar_t1>=ar_fillValue) ar_t1=d_spval
        where(ar_s1>=ar_fillValue) ar_s1=d_spval


        ar_t=(1.0-qr)*ar_t0+qr*ar_t1
        ar_s=(1.0-qr)*ar_s0+qr*ar_s1


        where(ar_t<d_spval) ar_t=ar_t*ar_scl_t+ar_off_t
        where(ar_s<d_spval) ar_s=ar_s*ar_scl_s+ar_off_s

        do k=1,ar_nz
         do j=1,ar_ny
          do i=1,ar_nx

          if(ar_t(i,j,k)<d_spval .and. ar_s(i,j,k)<d_spval) then
                  ar_t(i,j,k)=theta (ar_s(i,j,k), ar_t(i,j,k),ar_dpth(k),0.0)
          else
                  ar_t(i,j,k)=d_spval
                  ar_s(i,j,k)=d_spval
          endif
          enddo
          enddo
          enddo

          do i=1,n

           ptlon=lon(i)
           if(ptlon<0.0) ptlon=ptlon+360.
           ii=int(ridx(ar_lon,ptlon))
           jj=int(ridx(ar_lat,lat(i)))

           c_t(:)=ar_t(ii,jj,:)
           c_s(:)=ar_s(ii,jj,:)


           do k=2,ar_nz
           if(c_t(k)>=d_spval) then
                   c_t(k)=c_t(k-1)
                   c_s(k)=c_s(k-1)
           endif

           enddo

           

           call akspline(ar_nz,ar_dpth,c_t,cnz,cdpth,ct(i,:))
           call akspline(ar_nz,ar_dpth,c_s,cnz,cdpth,cs(i,:))

           enddo


           else
                   print *, "...no ref file...."
           endif



end subroutine        
        








end module     
