module maviso
use mkinds
 use mconstants
 use mdates
 use mncio
 use mobs
 use msearch


real(r4),allocatable      :: l3sla_lon(:)
real(r4),allocatable      :: l3sla_lat(:)
real(r4),allocatable      :: l3sla_val(:)
real(r8),allocatable      :: l3sla_time(:)
real(r4),allocatable      :: l3sla_err(:)
real(r4),allocatable      :: l3sla_grid_val(:,:)
real(r4),allocatable      :: l3sla_grid_err(:,:)
real(r4),allocatable      :: l3sla_grid_inov(:,:)
integer(i4),parameter     :: nmx=5000000
integer(i4)               :: nl3sla
real(r4),allocatable      :: l4sla_val(:,:)
real(r4),allocatable      :: l4sla_err(:,:)
real(r4),allocatable      :: l4sla_inov(:,:)

contains

subroutine aviso_l4sla_2cobs(cdate,iflag)
implicit none


character(*)              :: cdate
integer(i4)               :: gnx,lrgnx
integer(i4)               :: gny,lrgny
real(r4)                  :: scl_sla,off_sla,fill_value_sla
real(r4)                  :: scl_err,off_err,fill_value_err
real(r4)                  :: grdi,grdj,ptlon
integer(i4)               :: i,j,k
character(:),allocatable  :: inFile
logical(bl)               :: lexist
real(r4),allocatable      :: tmp_lon(:)
real(r4),allocatable      :: tmp_lat(:)
real(r4),allocatable      :: tmp_sla(:,:)
real(r4),allocatable      :: tmp_sla_err(:,:)
real(r4),allocatable      :: lon(:)
real(r4),allocatable      :: lat(:)
character(3),allocatable  :: altim(:)
integer(i4)               :: naltim
integer(i4)               :: twin_start,twin_end
integer(i4)               :: navisoFiles
integer(i4)               :: cstat
integer(i4)               :: iflag

inFile=trim(l4sla_data_location)//"/"//cdate(1:4)//"/"//trim(l4_sla_file_pfx)//"_"//cdate(1:8)//".nc"
INQUIRE(FILE=inFile, EXIST=lexist)
 if(lexist) then
   write(*,*) "....L4 SLA File....",inFile


         call nciopn(inFile,fid)
         call ncioin(inFile,fid,"latitude",gny)
         call ncioin(inFile,fid,"longitude",gnx)
         allocate(tmp_lon(gnx))
         allocate(tmp_lat(gny))
         allocate(tmp_sla(gnx,gny))
         allocate(tmp_sla_err(gnx,gny))
         allocate(l4sla_val(gnx,gny))
         allocate(l4sla_err(gnx,gny))

         call nciorv(inFile,fid,"longitude",tmp_lon)
         call nciorv(inFile,fid,"latitude",tmp_lat)
         call nciora(inFile,fid,"sla","_FillValue",fill_value_sla)
         call nciora(inFile,fid,"sla","scale_factor",scl_sla)
         call nciorv(inFile,fid,"sla",tmp_sla)
         call ncioiv(inFile,fid,"err",cstat)
         if(cstat==0) then
            call nciorv(inFile,fid,"err",tmp_sla_err)
            call nciora(inFile,fid,"err","scale_factor",scl_err)
            call nciora(inFile,fid,"err","_FillValue",fill_value_sla)
         endif

         call ncioiv(inFile,fid,"err_sla",cstat)
         if(cstat==0) then
            call nciorv(inFile,fid,"err_sla",tmp_sla_err)
            call nciora(inFile,fid,"err_sla","scale_factor",scl_err)
            call nciora(inFile,fid,"err_sla","_FillValue",fill_value_sla)
         endif

         call nciocl(inFile,fid)



         off_err=0.0
         off_sla=0.0

         where(tmp_sla>fill_value_sla)
                 tmp_sla=tmp_sla*scl_sla+off_sla
                 tmp_sla_err=tmp_sla_err*scl_err+off_err
         else where
                 tmp_sla=spval
                 tmp_sla_err=spval
         end where

         do j=1,gny
          do i=1,gnx
           if(l4obs_lon(i) <0.0 .and. tmp_lon(1) >0.0) then
                   ptlon=l4obs_lon(i)+360.0
           else
                  ptlon=l4obs_lon(i)
          endif
          
           grdi=ridx(tmp_lon,ptlon)
           grdj=ridx(tmp_lat,l4obs_lat(j))
           if(grdi>-99.0 .and. grdj>-99.0) then
              l4sla_val(i,j)=tmp_sla(nint(grdi),nint(grdj))
              l4sla_err(i,j)=tmp_sla_err(nint(grdi),nint(grdj))
           else
              l4sla_val(i,j)=spval
              l4sla_err(i,j)=spval
           endif 
          enddo
         enddo

         !print *, minval(l4sla_val,mask=l4sla_val<spval)
         !print *, maxval(l4sla_val,mask=l4sla_val<spval)

         iflag=0
         else
         iflag=-1
  Write(*,*) ".... L4 SLA file not present...."
 endif

end subroutine

subroutine aviso_l3sla_2cobs(cdate,iflag)

implicit none


character(10)             :: cdate

character(:),allocatable  :: dt_avisoFile
character(:),allocatable  :: rt_avisoFile

character(8)              :: cdatel
logical(bl)               :: lexist
logical(bl)               :: lrexist
logical(bl)               :: ldexist
logical(bl)               :: dt_aviso
logical(bl)               :: rt_aviso
logical(bl)               :: lmexist
integer(i4)               :: navisoFiles
integer(i4)               :: nob_grp
integer(i4)               :: ntwin
integer(i4)               :: twin_start
integer(i4)               :: twin_end
integer(i4)               :: ncount
integer(i4)               :: ix
integer(i4)               :: jy
real(r4)                  :: wsday,wday
character(cl),allocatable :: avisoFiles(:)
logical(bl)               :: lobs
integer(i4)               :: i,j,ii,jj

character(:),allocatable  :: inFile
character(:),allocatable  :: fname
character(5)              :: calt
character(:),allocatable  :: alt
integer(i4)               :: iflag


allocate(l3sla_val(nmx))
allocate(l3sla_lon(nmx))
allocate(l3sla_lat(nmx))
allocate(l3sla_time(nmx))
allocate(l3sla_err(nmx))


ntwin=1
navisoFiles=naltim*ntwin
allocate(avisoFiles(navisoFiles))
navisoFiles=0
  do ii=1,naltim
   calt=trim(altim(ii))
   alt=calt(1:len_trim(altim(ii)))
   
   dt_avisoFile=trim(l3sla_data_location)//"/"//trim(alt)//"/"//"aviso_tsla_"//trim(alt)//"_l3_"//cdate(1:8)//".nc"
   
   !dt_avisoFile=trim(atsla_data_location)//trim(alt)//"/dt_global_"//trim(alt)//"_phy_l3_"//cdtgl(1:8)//".nc"
   !rt_avisoFile=trim(atsla_data_location)//trim(alt)//"/nrt_global_"//trim(alt)//"_phy_l3_"//cdtgl(1:8)//".nc"

   INQUIRE(FILE=dt_avisoFile, EXIST=ldexist)
   !INQUIRE(FILE=rt_avisoFile, EXIST=lrexist)

   if(ldexist) then
!        if (navisoFiles==1) print *, "delayed time files available"    
        navisoFiles=navisoFiles+1   
        avisoFiles(navisoFiles)=dt_avisoFile
   endif

   !dt_aviso = ldexist 
   !rt_aviso = lrexist .and. (.not. dt_aviso)

   !if (dt_aviso) then
   !     print *, "delayed time files available"
   !     print *, " will use the foll files: "
   !  do i=1,navisoFiles
   !  print *, avisoFiles(navisoFiles)
   !  enddo
   !endif
   !if (rt_aviso) then
   !    if (navisoFiles==1) print *, "delayed time files not available but real-time file present"
   !     navisoFiles=navisoFiles+1
   !     avisoFiles(navisoFiles)=rt_avisoFile 
  
   !endif
 enddo

    ncount=0
    do ii=1,navisoFiles
     write(*,'(a6,a,a5)') " .... ", trim(avisoFiles(ii)), " ...."
     !call system('gunzip '//avisoFiles(ii)(1:len_trim(avisoFiles(ii))))
     ncfile=avisoFiles(ii)(1:len_trim(avisoFiles(ii)))
     call readin_aviso_l3sla(ncfile,glnmn,glnmx,gltmn,gltmx,ncount)
     !call system('gzip '//avisoFiles(ii)(1:len_trim(avisoFiles(ii))-3))
     !write(*,'(a,5x,a,5x,a,5x,I7)') "Filename: ",trim(ncfile), "No of Observations: ",ncount
    enddo

        nl3sla=ncount         

        if(ncount>0) then
                write(*,*) "....Total no of L3 SLA observations points: ",ncount
                iflag=0
        else
                iflag=-1
                write(*,*) "....no l3sla...."
        endif

return

end subroutine




SUBROUTINE readin_aviso_l3sla(obsdb,lnmn,lnmx,ltmn,ltmx,ncount)
IMPLICIT NONE

! Arguments

CHARACTER(Len=*),INTENT(IN)      :: obsdb
INTEGER(i4),INTENT(INOUT)        :: ncount

! Local Variables

INTEGER(i4)                      :: nvals
INTEGER(i4)                      :: index_begin
INTEGER(i4)                      :: index_end
INTEGER(i4)                      :: k
REAL(r8),allocatable             :: time(:)
REAL(r4),allocatable             :: longitude(:)
REAL(r4),allocatable             :: latitude(:)
REAL(r4),allocatable             :: sla(:)
LOGICAL(bl)                      :: obsdb_exists
real(r4)                         :: lon_scl,lon_off
real(r4)                         :: lat_scl,lat_off
real(r4)                         :: time_scl,time_off
real(r4)                         :: sla_scl,sla_off

real(r4)                         :: lnmn
real(r4)                         :: lnmx
real(r4)                         :: ltmn
real(r4)                         :: ltmx

INQUIRE(FILE=obsdb, EXIST=obsdb_exists)
IF (obsdb_exists) then

call nciopn(trim(obsdb),fid)
call ncioin(trim(obsdb),fid,"time",nvals)

call nciora(trim(obsdb),fid,"longitude","scale_factor",lon_scl)
call nciora(trim(obsdb),fid,"longitude","add_offset",lon_off)

call nciora(trim(obsdb),fid,"latitude","scale_factor",lat_scl)
call nciora(trim(obsdb),fid,"latitude","add_offset",lat_off)

call nciora(trim(obsdb),fid,"sla_filtered","scale_factor",sla_scl)
call nciora(trim(obsdb),fid,"sla_filtered","add_offset",sla_off)

allocate(time(nvals))
allocate(longitude(nvals))
allocate(latitude(nvals))
allocate(sla(nvals))

!sla=0.0


call nciorv(trim(obsdb),fid,'time',time)
call nciorv(trim(obsdb),fid,'longitude',longitude)
call nciorv(trim(obsdb),fid,'latitude',latitude)
call nciorv(trim(obsdb),fid,'sla_filtered',sla)
call nciocl(trim(obsdb),fid)

longitude=longitude*lon_scl + lon_off
latitude=latitude*lat_scl   + lat_off
sla=sla*sla_scl+sla_off

time=time+ndays(01,01,1950,12,31,1900)
where(longitude>180.0) longitude=longitude-360.0

        
do k=1,nvals
if(latitude(k)>ltmn .and. latitude(k)<=ltmx) then
  if (longitude(k)>=lnmn .and. longitude(k)<=lnmx .and. abs(sla(k))<1.5) then
!                   ncount=ncount+1
!                   lobs=.true.
!                   tdiff=abs(analysis_time-time(k))/taucorr_sla


  ncount=ncount+1
  l3sla_val(ncount)=sla(k)
  l3sla_lat(ncount)=latitude(k)
  l3sla_lon(ncount)=longitude(k)
  l3sla_time(ncount)=time(k)
  l3sla_err(ncount)=0.05
  !work_obs(ncount)%err=err*(1.0 + fact_sla*(1.0-exp(-tdiff)))

  endif
  endif

  enddo  



ENDIF
END SUBROUTINE readin_aviso_l3sla


end module maviso      
