module mcobsio

use mkinds
use mncio
use mglobals
use mobs
use mdates

!==========================================================================================================
!                                             Subroutine List                                             !
!==========================================================================================================
!1. subroutine write_cobs_file_skel(filename)
!2. subroutine cobs_add_profiles(filename,ncount,nzl,prof_num,zlevels,plon,plat,ptime,
!                                 temp,saln,ctemp,csaln,ctstd,csstd,it,is,pvalid,ityp)
!3. subroutine cobs_add_l3sla(filename,n,lon,lat,time,val,err) 
!4. subroutine cobs_add_l2isst(filename,n,lon,lat,time,val,err) 
!5. subroutine cobs_add_l3sst(filename,n,lon,lat,time,val,err) 
!6. subroutine cobs_add_drifter(filename,n,lon,lat,time,uc,vc)
!7. subroutine cobs_add_l4sst(filename,l4obs_nx,l4obs_ny,l4obs_lon,l4obs_lat,l4sst_val,l4sst_err,l4ice_cov)
!8. subroutine cobs_add_l4sla(filename,l4obs_nx,l4obs_ny,l4obs_lon,l4obs_lat,l4sla_val,l4sla_err)
!9. subroutine write_tsis_obsfile(cdtg,nprofiles,depths,nlevels,ntyp,glon,glat,nx,ny,pfx)
!10.subroutine write_tsis_obsfilez(cdtg,nprofiles,depths,nlevels,ntyp,glon,glat,nx,ny,pfx)
!===========================================================================================================

contains

subroutine write_cobs_file_skel(filename,nx,ny)

  implicit none

  character(*)             :: filename
  integer(i4)              :: nx
  integer(i4)              :: ny
  
  integer(i4)              :: obsgrp(nobsgrp)
  integer(i4)              :: i



  ! set l4obs_nx and l4obs_ny

  l4obs_nx=nx
  l4obs_ny=ny

  !profiles,atsla,msla,atsst,msst,drift
  do i=1,nobsgrp
      obsgrp(i)=0
  enddo

  call nciocf(filename,fid)
  call nciodd(filename,fid,"nobsgrp",nobsgrp)
  
  vname="obsgrp"
  vtype=NF90_INT
  vdim1d(1)=1
  call nciocv(filename,fid,vname,vtype,vdim1d)
  call nciowa(filename,fid,vname,'long_name','obsgroup')
  
  vname="l4obs_lon"
  vtype=NF90_float
  vdim1d(1)=2
  call nciodd(filename,fid,vname,l4obs_nx)
  call nciocv(filename,fid,vname,vtype,vdim1d)
  call nciowa(filename,fid,vname,'long_name','l4longitude')

  vname="l4obs_lat"
  vtype=NF90_float
  vdim1d(1)=3
  call nciodd(filename,fid,vname,l4obs_ny)
  call nciocv(filename,fid,vname,vtype,vdim1d)
  call nciowa(filename,fid,vname,'long_name','l4Latitude')

  ! end define mode
  call ncioed(filename,fid)

  ! write data into variables

  call nciowv(filename,fid,"obsgrp",obsgrp)
  call nciocl(filename,fid)
  
  end subroutine        


  subroutine cobs_add_profiles(filename,ncount,nzl,prof_num,zlevels,plon,plat,ptime,&
                            temp,saln,sctemp,scsaln,dctemp,dcsaln,ctstd,csstd,it,is,pvalid,ityp)


  implicit none
  character(*)              :: filename
  integer(i4)               :: ncount,nsla,nzl,nft
  integer(i4)               :: prof_num(ncount)
  real(r4)                  :: zlevels(nzl)
  real(r4)                  :: plon(ncount),plat(ncount),ptime(ncount)
  real(r4)                  :: temp(ncount,nzl),saln(ncount,nzl)
  real(r4)                  :: sctemp(ncount,nzl),scsaln(ncount,nzl)
  real(r4)                  :: dctemp(ncount,nzl),dcsaln(ncount,nzl)
  real(r4)                  :: ctstd(ncount,nzl),csstd(ncount,nzl)
  integer(i4)               :: it(ncount,nzl),is(ncount,nzl),ityp(ncount)
  integer(i4)               :: qctype(3)
  integer(i4)               :: pvalid(ncount,3)
  real(r4),parameter        :: pr_spval=-99.0
  integer(i4)               :: i
  integer(i4)               :: obsgrp(nobsgrp)

  qctype=[1,2,3]  

  call nciopn(filename,fid,'w')
  call ncioiq(filename,fid,ndim,nvar,natt,nulim)
  call nciorc(filename,fid)
  vname="nprof"
  vtype=NF90_INT
  vdim1d(1)=ndim+1
  call nciodd(filename,fid,vname,ncount)
  call nciocv(filename,fid,vname,vtype,vdim1d)
  call nciowa(filename,fid,vname,'long_name','profile_number')
  call nciowa(filename,fid,vname,'standard_name','profile_number')

  vname="depth"
  vtype=NF90_Float
  vdim1d(1)=ndim+2
  call nciodd(filename,fid,vname,nzl)
  call nciocv(filename,fid,vname,vtype,vdim1d)
  call nciowa(filename,fid,vname,'long_name','pressure')
  call nciowa(filename,fid,vname,'standard_name','pressure')
  call nciowa(filename,fid,vname,'units','decibars')

  !create data variable 1
  vname="plon"
  vtype=NF90_Float
  vdim1d=(/ndim+1/)
  call nciocv(filename,fid,vname,vtype,vdim1d)
  call nciowa(filename,fid,vname,'long_name','profile_longitude')

  vname="plat"
  vtype=NF90_Float
  vdim1d=(/ndim+1/)
  call nciocv(filename,fid,vname,vtype,vdim1d)
  call nciowa(filename,fid,vname,'long_name','profile_latitude')

  vname="ptime"
  vtype=NF90_Float
  vdim1d=(/ndim+1/)
  call nciocv(filename,fid,vname,vtype,vdim1d)
  call nciowa(filename,fid,vname,'long_name','time')
  call nciowa(filename,fid,vname,'standard_name','time')
  call nciowa(filename,fid,vname,'units','days since 1900-12-31')
  call nciowa(filename,fid,vname,'_FillValue',pr_spval)

!create data variable 2
  vname="temp"
  vtype=NF90_Float
  vdim2d=(/ndim+1,ndim+2/)
  call nciocv(filename,fid,vname,vtype,vdim2d)
  call nciowa(filename,fid,vname,'long_name','Potential Temperature')
  call nciowa(filename,fid,vname,'standard_name','Potential Temperature')
  call nciowa(filename,fid,vname,'units','degC')
  call nciowa(filename,fid,vname,'_FillValue',pr_spval)

!create data variable 3
  vname="saln"
  vtype=NF90_Float
  vdim2d=(/ndim+1,ndim+2/)
  call nciocv(filename,fid,vname,vtype,vdim2d)
  call nciowa(filename,fid,vname,'long_name','Salinity')
  call nciowa(filename,fid,vname,'standard_name','Salinity')
  call nciowa(filename,fid,vname,'units','psu')
  call nciowa(filename,fid,vname,'_FillValue',pr_spval)

!create data variable 4
  vname="sctemp"
  vtype=NF90_Float
  vdim2d=(/ndim+1,ndim+2/)
  call nciocv(filename,fid,vname,vtype,vdim2d)
  call nciowa(filename,fid,vname,'long_name','WOA23_static_climatological_seawater_potential_temperature')
  call nciowa(filename,fid,vname,'standard_name','WOA23_static_climatological_seawater_ potential temperature')
  call nciowa(filename,fid,vname,'units','degC')
  call nciowa(filename,fid,vname,'_FillValue',pr_spval)

!create data variable 3
  vname="scsaln"
  vtype=NF90_Float
  vdim2d=(/ndim+1,ndim+2/)
  call nciocv(filename,fid,vname,vtype,vdim2d)
  call nciowa(filename,fid,vname,'long_name','WOA23_static_climatological_seawater_salinity')
  call nciowa(filename,fid,vname,'standard_name','WOA23_static_climatological_seawater_salinity')
  call nciowa(filename,fid,vname,'units','')
  call nciowa(filename,fid,vname,'_FillValue',pr_spval)

!create data variable 4
  vname="dctemp"
  vtype=NF90_Float
  vdim2d=(/ndim+1,ndim+2/)
  call nciocv(filename,fid,vname,vtype,vdim2d)
  call nciowa(filename,fid,vname,'long_name','dynamic_climatological_seawater_potential_temperature')
  call nciowa(filename,fid,vname,'standard_name','dynamic_climatological_seawater_ potential temperature')
  call nciowa(filename,fid,vname,'units','degC')
  call nciowa(filename,fid,vname,'_FillValue',pr_spval)

!create data variable 3
  vname="dcsaln"
  vtype=NF90_Float
  vdim2d=(/ndim+1,ndim+2/)
  call nciocv(filename,fid,vname,vtype,vdim2d)
  call nciowa(filename,fid,vname,'long_name','dynamic_climatological_seawater_salinity')
  call nciowa(filename,fid,vname,'standard_name','dynamic_climatological_seawater_salinity')
  call nciowa(filename,fid,vname,'units','')
  call nciowa(filename,fid,vname,'_FillValue',pr_spval)

  vname="ctstd"
  vtype=NF90_Float
  vdim2d=(/ndim+1,ndim+2/)
  call nciocv(filename,fid,vname,vtype,vdim2d)
  call nciowa(filename,fid,vname,'long_name','WOA23_climatological_seawater_potential_temperature_std_dev')
  call nciowa(filename,fid,vname,'standard_name','WOA23_climatological_seawater_potential_temperature_std_dev')
  call nciowa(filename,fid,vname,'units','degC')
  call nciowa(filename,fid,vname,'_FillValue',pr_spval)

  vname="csstd"
  vtype=NF90_Float
  vdim2d=(/ndim+1,ndim+2/)
  call nciocv(filename,fid,vname,vtype,vdim2d)
  call nciowa(filename,fid,vname,'long_name','WOA23_climatological_seawater_potential_salinity_std_dev')
  call nciowa(filename,fid,vname,'standard_name','WOA23_climatological_seawater_potential_salinity_std_dev')
  call nciowa(filename,fid,vname,'units','')
  call nciowa(filename,fid,vname,'_FillValue',pr_spval)

  vname="it"
  vtype=NF90_int
  vdim2d=(/ndim+1,ndim+2/)
  call nciocv(filename,fid,vname,vtype,vdim2d)
  call nciowa(filename,fid,vname,'long_name','temperature_intp_flag')

  vname="is"
  vtype=NF90_int
  vdim2d=(/ndim+1,ndim+2/)
  call nciocv(filename,fid,vname,vtype,vdim2d)
  call nciowa(filename,fid,vname,'long_name','salinity_intp_flag')

  vname="ityp"
  vtype=NF90_int
  vdim1d=(/ndim+1/)
  call nciocv(filename,fid,vname,vtype,vdim1d)
  call nciowa(filename,fid,vname,'long_name','profile_type')

  vname="pvalid"
  vtype=NF90_int
  vdim2d=(/ndim+1,ndim+2/)
  call nciocv(filename,fid,vname,vtype,vdim2d)
  call nciowa(filename,fid,vname,'long_name','valid_flag')

  vname="qctype"
  vtype=NF90_int
  vdim1d=(/ndim+3/)
  call nciodd(filename,fid,vname,3)
  call nciocv(filename,fid,vname,vtype,vdim1d)
  call nciowa(filename,fid,vname,'long_name','qc_comparison_field_type')

  ! end define mode

  call ncioed(filename,fid)


! write data into variables

  call nciowv(filename,fid,"nprof",prof_num)
  call nciowv(filename,fid,"depth",zlevels)
  call nciowv(filename,fid,"qctype",qctype)
  call nciowv(filename,fid,"plon",plon)
  call nciowv(filename,fid,"plat",plat)
  call nciowv(filename,fid,"ptime",ptime)
  call nciowv(filename,fid,"temp",temp)
  call nciowv(filename,fid,"saln",saln)
  call nciowv(filename,fid,"sctemp",sctemp)
  call nciowv(filename,fid,"scsaln",scsaln)
  call nciowv(filename,fid,"dctemp",dctemp)
  call nciowv(filename,fid,"dcsaln",dcsaln)
  call nciowv(filename,fid,"ctstd",ctstd)
  call nciowv(filename,fid,"csstd",csstd)
  call nciowv(filename,fid,"it",it)
  call nciowv(filename,fid,"is",is)
  call nciowv(filename,fid,"ityp",ityp)
  call nciowv(filename,fid,"pvalid",pvalid)

  call nciorv(filename,fid,"obsgrp",obsgrp)
  obsgrp(l2_tsprofiles)=1
  call nciowv(filename,fid,"obsgrp",obsgrp)

   call nciocl(filename,fid)

  end subroutine

  subroutine cobs_add_l3sla(filename,n,lon,lat,time,val,err)

  implicit none

  character(*)              :: filename
  integer(i4)               :: n
  integer(i4)               :: l3sla(n)
  real(r8)                  :: time(n)
  real(r4)                  :: lon(n)
  real(r4)                  :: lat(n)
  real(r4)                  :: val(n)
  real(r4)                  :: err(n)
  integer(i4)               :: obsgrp(nobsgrp)
  integer(i4)               :: i

  call nciopn(filename,fid,'w')
  call ncioiq(filename,fid,ndim,nvar,natt,nulim)
  call nciorc(filename,fid)

  do i=1,n
     l3sla(i)=i
  enddo

  vname="l3sla"
  vtype=NF90_INT
  vdim1d(1)=ndim+1
  call nciodd(filename,fid,vname,n)
  call nciocv(filename,fid,vname,vtype,vdim1d)
  call nciowa(filename,fid,vname,'long_name','l3sla_obs_indx')

  vname="l3sla_lon"
  vtype=NF90_Float
  vdim1d=(/ndim+1/)
  call nciocv(filename,fid,vname,vtype,vdim1d)
  call nciowa(filename,fid,vname,'long_name','l3sla_longitude')

  vname="l3sla_lat"
  vtype=NF90_Float
  vdim1d=(/ndim+1/)
  call nciocv(filename,fid,vname,vtype,vdim1d)
  call nciowa(filename,fid,vname,'long_name','l3sla_latitude')

  vname="l3sla_time"
  vtype=NF90_double
  vdim1d=(/ndim+1/)
  call nciocv(filename,fid,vname,vtype,vdim1d)
  call nciowa(filename,fid,vname,'long_name','l3sla_time')
  call nciowa(filename,fid,vname,'units','days since 1900-12-31')

  vname="l3sla_val"
  vtype=NF90_Float
  vdim1d=(/ndim+1/)
  call nciocv(filename,fid,vname,vtype,vdim1d)
  call nciowa(filename,fid,vname,'long_name','l3sla')

  ! end define mode

  call ncioed(filename,fid)


! write data into variables


  call nciowv(filename,fid,"l3sla",l3sla)
  call nciowv(filename,fid,"l3sla_lon",lon)
  call nciowv(filename,fid,"l3sla_lat",lat)
  call nciowv(filename,fid,"l3sla_time",time)
  call nciowv(filename,fid,"l3sla_val",val)

  call nciorv(filename,fid,"obsgrp",obsgrp)
  obsgrp(l3_sla)=1
  call nciowv(filename,fid,"obsgrp",obsgrp)

 ! close file

  call nciocl(filename,fid) 

  end subroutine cobs_add_l3sla

  subroutine cobs_add_l2isst(filename,n,lon,lat,time,val,err)

  implicit none

  character(*)              :: filename
  integer(i4)               :: n
  integer(i4)               :: l2isst(n)
  real(r4)                  :: lon(n)
  real(r4)                  :: lat(n)
  real(r8)                  :: time(n)
  real(r4)                  :: val(n)
  real(r4)                  :: err(n)
  integer(i4)               :: obsgrp(nobsgrp)
  integer(i4)               :: i
  integer(i4)               :: nin

  call nciopn(filename,fid,'w')
  call ncioiq(filename,fid,ndim,nvar,natt,nulim)
  call nciorc(filename,fid)

  do i=1,n
     l2isst(i)=i
  enddo

  vname="l2isst"
  vtype=NF90_INT
  vdim1d(1)=ndim+1
  call nciodd(filename,fid,vname,n)
  call nciocv(filename,fid,vname,vtype,vdim1d)
  call nciowa(filename,fid,vname,'long_name','l2isst_obs_indx')

  vname="l2isst_lon"
  vtype=NF90_Float
  vdim1d=(/ndim+1/)
  call nciocv(filename,fid,vname,vtype,vdim1d)
  call nciowa(filename,fid,vname,'long_name','l2isst_longitude')

  vname="l2isst_lat"
  vtype=NF90_Float
  vdim1d=(/ndim+1/)
  call nciocv(filename,fid,vname,vtype,vdim1d)
  call nciowa(filename,fid,vname,'long_name','l2isst_latitude')

  vname="l2isst_time"
  vtype=NF90_double
  vdim1d=(/ndim+1/)
  call nciocv(filename,fid,vname,vtype,vdim1d)
  call nciowa(filename,fid,vname,'long_name','l2isst_time')
  call nciowa(filename,fid,vname,'units','days since 1900-12-31')

  vname="l2isst_val"
  vtype=NF90_Float
  vdim1d=(/ndim+1/)
  call nciocv(filename,fid,vname,vtype,vdim1d)
  call nciowa(filename,fid,vname,'long_name','l2isst')


  vname="l2isst_err"
  vtype=NF90_Float
  vdim1d=(/ndim+1/)
  call nciocv(filename,fid,vname,vtype,vdim1d)
  call nciowa(filename,fid,vname,'long_name','l2isst_err')

  ! end define mode

  call ncioed(filename,fid)

  ! write data into variables


  call nciowv(filename,fid,"l2isst",l2isst)
  call nciowv(filename,fid,"l2isst_lon",lon)
  call nciowv(filename,fid,"l2isst_lat",lat)
  call nciowv(filename,fid,"l2isst_time",time)
  call nciowv(filename,fid,"l2isst_val",val)
  call nciowv(filename,fid,"l2isst_err",err)

   call nciorv(filename,fid,"obsgrp",obsgrp)
  obsgrp(l2_sst)=1
  call nciowv(filename,fid,"obsgrp",obsgrp)

 ! close file

  call nciocl(filename,fid)

  end subroutine cobs_add_l2isst

  subroutine cobs_add_l3sst(filename,n,lon,lat,time,val,err)

  implicit none

  character(*)              :: filename
  integer(i4)               :: n
  integer(i4)               :: l3sst(n)
  real(r4)                  :: lon(n)
  real(r4)                  :: lat(n)
  real(r8)                  :: time(n)
  real(r4)                  :: val(n)
  real(r4)                  :: err(n)
  integer(i4)               :: obsgrp(nobsgrp)
  integer(i4)               :: i

  call nciopn(filename,fid,'w')
  call ncioiq(filename,fid,ndim,nvar,natt,nulim)
  call nciorc(filename,fid)

  do i=1,n
     l3sst(i)=i
  enddo

  vname="l3sst"
  vtype=NF90_INT
  vdim1d(1)=ndim+1
  call nciodd(filename,fid,vname,n)
  call nciocv(filename,fid,vname,vtype,vdim1d)
  call nciowa(filename,fid,vname,'long_name','l3sst_obs_indx')

  vname="l3sst_lon"
  vtype=NF90_Float
  vdim1d=(/ndim+1/)
  call nciocv(filename,fid,vname,vtype,vdim1d)
  call nciowa(filename,fid,vname,'long_name','l3sst_longitude')

  vname="l3sst_lat"
  vtype=NF90_Float
  vdim1d=(/ndim+1/)
  call nciocv(filename,fid,vname,vtype,vdim1d)
  call nciowa(filename,fid,vname,'long_name','l3sst_latitude')

  vname="l3sst_time"
  vtype=NF90_double
  vdim1d=(/ndim+1/)
  call nciocv(filename,fid,vname,vtype,vdim1d)
  call nciowa(filename,fid,vname,'long_name','l3sst_time')
  call nciowa(filename,fid,vname,'units','days since 1900-12-31')

  vname="l3sst_val"
  vtype=NF90_Float
  vdim1d=(/ndim+1/)
  call nciocv(filename,fid,vname,vtype,vdim1d)
  call nciowa(filename,fid,vname,'long_name','l3sst')


  vname="l3sst_err"
  vtype=NF90_Float
  vdim1d=(/ndim+1/)
  call nciocv(filename,fid,vname,vtype,vdim1d)
  call nciowa(filename,fid,vname,'long_name','l3sst_err')

  ! end define mode

  call ncioed(filename,fid)

  ! write data into variables


  call nciowv(filename,fid,"l3sst",l3sst)
  call nciowv(filename,fid,"l3sst_lon",lon)
  call nciowv(filename,fid,"l3sst_lat",lat)
  call nciowv(filename,fid,"l3sst_time",time)
  call nciowv(filename,fid,"l3sst_val",val)
  call nciowv(filename,fid,"l3sst_err",err)

   call nciorv(filename,fid,"obsgrp",obsgrp)
  obsgrp(l3_sst)=1
  call nciowv(filename,fid,"obsgrp",obsgrp)

 ! close file

  call nciocl(filename,fid)

  end subroutine cobs_add_l3sst

  subroutine cobs_add_drifter(filename,n,lon,lat,time,uc,vc)

    implicit none
    character(*)              :: filename
    integer(i4)               :: n
    integer(i4)               :: drifter(n)
    real(r8)                  :: time(n)
    real(r4)                  :: lon(n)
    real(r4)                  :: lat(n)
    real(r4)                  :: uc(n)
    real(r4)                  :: vc(n)
    integer(i4)               :: obsgrp(nobsgrp)
    integer(i4)               :: i


    call nciopn(filename,fid,'w')
    call ncioiq(filename,fid,ndim,nvar,natt,nulim)
    call nciorc(filename,fid)

  do i=1,n
     drifter(i)=i
  enddo

  vname="drifter"
  vtype=NF90_INT
  vdim1d(1)=ndim+1
  call nciodd(filename,fid,vname,n)
  call nciocv(filename,fid,vname,vtype,vdim1d)
  call nciowa(filename,fid,vname,'long_name','drifter_obs_indx')

  vname="drifter_lon"
  vtype=NF90_Float
  vdim1d=(/ndim+1/)
  call nciocv(filename,fid,vname,vtype,vdim1d)
  call nciowa(filename,fid,vname,'long_name','drifter_longitude')

  vname="drifter_lat"
  vtype=NF90_Float
  vdim1d=(/ndim+1/)
  call nciocv(filename,fid,vname,vtype,vdim1d)
  call nciowa(filename,fid,vname,'long_name','drifter_latitude')

  vname="drifter_time"
  vtype=NF90_double
  vdim1d=(/ndim+1/)
  call nciocv(filename,fid,vname,vtype,vdim1d)
  call nciowa(filename,fid,vname,'long_name','drifter_time')
  call nciowa(filename,fid,vname,'units','days since 1900-12-31')

  vname="drifter_u"
  vtype=NF90_Float
  vdim1d=(/ndim+1/)
  call nciocv(filename,fid,vname,vtype,vdim1d)
  call nciowa(filename,fid,vname,'long_name','eastward drifter velocity')
   call nciowa(filename,fid,vname,'units','m/s')

  vname="drifter_v"
  vtype=NF90_Float
  vdim1d=(/ndim+1/)
  call nciocv(filename,fid,vname,vtype,vdim1d)
  call nciowa(filename,fid,vname,'long_name','northward drifter velocity')
   call nciowa(filename,fid,vname,'units','m/s')
  
  ! end define mode

  call ncioed(filename,fid)

  ! write data into variables


  call nciowv(filename,fid,"drifter",drifter)
  call nciowv(filename,fid,"drifter_lon",lon)
  call nciowv(filename,fid,"drifter_lat",lat)
  call nciowv(filename,fid,"drifter_time",time)
  call nciowv(filename,fid,"drifter_u",uc)
  call nciowv(filename,fid,"drifter_v",vc)

  call nciorv(filename,fid,"obsgrp",obsgrp)
  obsgrp(l2_suv)=1
  call nciowv(filename,fid,"obsgrp",obsgrp)

 ! close file

  call nciocl(filename,fid)




  end subroutine cobs_add_drifter



!  subroutine cobs_add_l4sst(filename,l4obs_nx,l4obs_ny,l4obs_lon,l4obs_lat,l4sst_val,l4sst_err,l4ice_cov)
 subroutine cobs_add_l4sst(filename,l4obs_nx,l4obs_ny,l4sst_val,l4sst_err,l4ice_cov) 
   

  implicit none
  character(*)              :: filename
  integer(i4)               :: l4obs_nx,l4obs_ny
!  real(r4)                  :: l4obs_lon(l4obs_nx)
!  real(r4)                  :: l4obs_lat(l4obs_ny)
  real(r4)                  :: l4sst_val(l4obs_nx,l4obs_ny)
  real(r4)                  :: l4sst_err(l4obs_nx,l4obs_ny)
  real(r4)                  :: l4ice_cov(l4obs_nx,l4obs_ny)
  integer(i4)               :: obsgrp(nobsgrp)


   call nciopn(filename,fid,'w')
   call nciorc(filename,fid)

   vname="l4sst_val"
   vtype=NF90_float
   vdim2d=(/2,3/)
   call nciocv(filename,fid,vname,vtype,vdim2d)
   call nciowa(filename,fid,vname,'long_name','L4 foundation temperature')
   call nciowa(filename,fid,vname,'units','deg C')
   call nciowa(filename,fid,vname,'_FillValue',spval)

   vname="l4sst_err"
   vtype=NF90_Float
   vdim2d=(/2,3/)
   call nciocv(filename,fid,vname,vtype,vdim2d)
   call nciowa(filename,fid,vname,'long_name','L4 foundation temperature error')
   call nciowa(filename,fid,vname,'_FillValue',spval)


   vname="l4ice_cov"
   vtype=NF90_Float
   vdim2d=(/2,3/)
   call nciocv(filename,fid,vname,vtype,vdim2d)
   call nciowa(filename,fid,vname,'long_name','ice_coverage')
   call nciowa(filename,fid,vname,'_FillValue',spval)

   call ncioed(filename,fid)

! write data into variables
   call nciowv(filename,fid,"l4obs_lon",l4obs_lon)
   call nciowv(filename,fid,"l4obs_lat",l4obs_lat)
   call nciowv(filename,fid,"l4sst_val",l4sst_val)
   call nciowv(filename,fid,"l4sst_err",l4sst_err)
   call nciowv(filename,fid,"l4ice_cov",l4ice_cov)
! close file
   call nciorv(filename,fid,"obsgrp",obsgrp)
   obsgrp(l4_sst)=1
   call nciowv(filename,fid,"obsgrp",obsgrp)

   call nciocl(filename,fid)

end subroutine cobs_add_l4sst

!subroutine cobs_add_l4sla(filename,l4obs_nx,l4obs_ny,l4obs_lon,l4obs_lat,l4sla_val,l4sla_err)

subroutine cobs_add_l4sla(filename,l4obs_nx,l4obs_ny,l4sla_val,l4sla_err)
  implicit none
  character(*)              :: filename
  integer(i4)               :: l4obs_nx,l4obs_ny
!  real(r4)                  :: l4obs_lon(l4obs_nx)
!  real(r4)                  :: l4obs_lat(l4obs_ny)
  real(r4)                  :: l4sla_val(l4obs_nx,l4obs_ny)
  real(r4)                  :: l4sla_err(l4obs_nx,l4obs_ny)
  integer(i4)               :: obsgrp(nobsgrp)

   call nciopn(filename,fid,'w')
   call nciorc(filename,fid)
   
   vname="l4sla_val"
   vtype=NF90_float
   vdim2d=(/2,3/)
   call nciocv(filename,fid,vname,vtype,vdim2d)
   call nciowa(filename,fid,vname,'long_name','L4 sea_level_anomaliy')
   call nciowa(filename,fid,vname,'units','m')
   call nciowa(filename,fid,vname,'_FillValue',spval)

   vname="l4sla_err"
   vtype=NF90_float
   vdim2d=(/2,3/)
   call nciocv(filename,fid,vname,vtype,vdim2d)
   call nciowa(filename,fid,vname,'long_name','L4 error_in_sea_lavel_anomaly')
   call nciowa(filename,fid,vname,'units','m')
   call nciowa(filename,fid,vname,'_FillValue',spval)

   call ncioed(filename,fid)

! write data into variables
   call nciowv(filename,fid,"l4obs_lon",l4obs_lon)
   call nciowv(filename,fid,"l4obs_lat",l4obs_lat)
   call nciowv(filename,fid,"l4sla_val",l4sla_val)
   call nciowv(filename,fid,"l4sla_err",l4sla_err)
! close file

   call nciorv(filename,fid,"obsgrp",obsgrp)
    obsgrp(l4_sla)=1
   call nciowv(filename,fid,"obsgrp",obsgrp)
   call nciocl(filename,fid)

   end subroutine

subroutine cobs_add_l2sss(filename,nx,ny,l2sss_val,l2sss_err)
  implicit none
  character(*)              :: filename
  integer(i4)               :: nx,ny
  real(r4)                  :: l2sss_val(nx,ny)
  real(r4)                  :: l2sss_err(nx,ny)
  integer(i4)               :: obsgrp(nobsgrp)

   call nciopn(filename,fid,'w')
   call nciorc(filename,fid)

   vname="l2sss_val"
   vtype=NF90_float
   vdim2d=(/2,3/)
   call nciocv(filename,fid,vname,vtype,vdim2d)
   call nciowa(filename,fid,vname,'long_name','L2 sea_surface_salinity')
   call nciowa(filename,fid,vname,'units','-')
   call nciowa(filename,fid,vname,'_FillValue',spval)

   vname="l2sss_err"
   vtype=NF90_float
   vdim2d=(/2,3/)
   call nciocv(filename,fid,vname,vtype,vdim2d)
   call nciowa(filename,fid,vname,'long_name','L2 sea_surface_salinity_error')
   call nciowa(filename,fid,vname,'units','-')
   call nciowa(filename,fid,vname,'_FillValue',spval)

   call ncioed(filename,fid)

! write data into variables
   call nciowv(filename,fid,"l2sss_val",l2sss_val)
   call nciowv(filename,fid,"l2sss_err",l2sss_err)
! close file

   call nciorv(filename,fid,"obsgrp",obsgrp)
    obsgrp(l2_sss)=1
   call nciowv(filename,fid,"obsgrp",obsgrp)
   call nciocl(filename,fid)

  end subroutine


end module mcobsio
