module mgmrf
USE OMP_LIB
use mkinds
use mconstants
use mncio

!  define parameters, variables, for the gmrf assimilation scheme.

! information pair:

      integer(i4),parameter              :: maxW=2  ! width of MRF neighborhood (rectangular).
      integer(i4)                        :: maxV    ! number of dynamically independent layers.
      real(r4),target,allocatable        :: infoMatrix(:,:,:,:,:,:)  ! info.matrix.
      real(r4),target,allocatable        :: infoVector(:,:,:,:)      ! info.vector
      real(r4),parameter                 :: vard = 0.04*0.04
      real(r4),parameter                 :: varb = 0.2*0.2
      real(r4),parameter                 :: weightD=(1.0/vard)
      real(r4),parameter                 :: weightB=(1.0/varb)
      real(r4),parameter                 :: sclD=1.0
      integer(i4)                        :: maxIteration
      real(r4)                           :: wthin
      
contains

! 

subroutine initInfoPair(idm,jdm,kdm,ldm)
implicit none
integer(i4)                        :: idm,jdm,kdm,ldm
NAMELIST /gmrf/wthin,maxIteration

allocate(infoMatrix(idm,jdm,-maxW:maxW,-maxW:maxW,kdm,ldm))
      allocate(infoVector(idm,jdm,kdm,ldm))


        OPEN(21,FILE='tsis.nlist')
        READ(21,NML=gmrf)
        CLOSE(21)
      infovector=0.0
      infoMatrix=0.0
end subroutine

subroutine FreeInfoPair()
implicit none

deallocate(infoMatrix)
deallocate(infoVector)

end subroutine


subroutine setInfoMatrix(idm,jdm,kdm,ldm,mask)

      implicit none
      integer(i4)                        :: idm,jdm,kdm,ldm
      integer(i4)                        :: mask(idm,jdm,kdm)

      ! information pair:
      real,pointer                   ::  W(:,:,:,:,:,:)
      
      ! local variables:
      real(r4)                           :: r
      real(r4)                           :: c
      integer(r4)                        :: ia,ja

      ! typical local variables:

      integer(i4) ::  i,j,k,l,mw,nw,iw,jw,iww,jww  ! array/matrix indices.
      
      W=>infoMatrix


      infoMatrix=0.0
      c=1.0
      r=0.05
   do l=1,ldm
     do  k=1,kdm 
       do j=1,jdm
        do i=1,idm
         if(mask(i,j,k)==1) then
        W(i,j,0,0,k,l)=weightB*(20.005*wThin+4.005*(1.-wthin))
        W(i,j,1,0,k,l)=weightB*(-8.0*wThin-(1.-wthin))
        W(i,j,0,1,k,l)=weightB*(-8.0*wThin-(1.-wthin))
        W(i,j,0,-1,k,l)=weightB*(-8.0*wThin-(1.-wthin))
        W(i,j,-1,0,k,l)=weightB*(-8.0*wThin-(1.-wthin))
        W(i,j,1,1,k,l)=weightB*(2.*wThin)
        W(i,j,-1,1,k,l)=weightB*(2.*wThin)
        W(i,j,1,-1,k,l)=weightB*(2.*wThin)
        W(i,j,-1,-1,k,l)=weightB*(2.*wThin)
        W(i,j,2,0,k,l)=weightB*(1.0*wThin)
        W(i,j,0,2,k,l)=weightB*(1.0*wThin)
        W(i,j,0,-2,k,l)=weightB*(1.0*wThin)
        W(i,j,-2,0,k,l)=weightB*(1.0*wThin)
         endif
       enddo
      enddo
     enddo
    enddo 
end subroutine

subroutine setInfoVector(idm,jdm,kdm,ldm,bo,vv)

      implicit none
      integer(i4)                        :: idm,jdm,kdm,ldm
      real(r4)                           :: bo(idm,jdm,kdm,ldm)
      real(r4)                           :: vv(idm,jdm,kdm,ldm)

      ! information pair:
      real,pointer                   ::  z(:,:,:,:)
      real,pointer                   ::  W(:,:,:,:,:,:)

      ! local variables:
      real(r4)                           :: r
      real(r4)                           :: c
      integer(r4)                        :: ia,ja

      ! typical local variables:

      integer(i4) ::  i,j,k,l,mw,nw,iw,jw,iww,jww  ! array/matrix indices.

      W=>infoMatrix;z=>infoVector

      z=0.0

      do   l=1,ldm
       do  k=1,kdm
        do j=1,jdm
         do i=1,idm
            if(bo(i,j,k,l)<spval) then
                z(i,j,k,l)=z(i,j,k,l)+bo(i,j,k,l)*weightD
                W(i,j,0,0,k,l)=W(i,j,0,0,k,l)+weightD

            endif
          enddo
        enddo
       enddo
      enddo

      do l=1,ldm
       do k=1,kdm
        do j=1,jdm
         do i=1,idm
          if(W(i,j,0,0,k,l).eq.0.) W(i,j,0,0,k,l)=1.0
         enddo
        enddo
       enddo
      enddo

end subroutine

subroutine roifPCG(po,idm,jdm,kdm,ldm,mask)


      ! a Preconditioned Conjugate Gradient solution for the information pair.


      implicit none
       
      integer(i4)                        :: idm,jdm,kdm,ldm
      real(r4) :: po(idm,jdm,kdm,ldm)
      integer(i4) :: mask(idm,jdm,kdm)
      integer(i4) :: ki

      ! information pair:
      real(r4),pointer ::  b(:,:,:,:),A(:,:,:,:,:,:)
      !equivalence (b,infoVector),(A,infoMatrix)
      ! print flag:
      logical diagPCG
      parameter (diagPCG=.true.)  ! if set, prints iteration residuals.

      ! local variables:
      real(r4) ::  x(idm,jdm,kdm,ldm),r(idm,jdm,kdm,ldm),p(idm,jdm,kdm,ldm)
      real(r4),target,allocatable ::  z(:,:,:,:)
      real(r4),pointer            ::  w(:,:,:,:)
      real(r4) ::  pw,alphaPCG,betaPCG,rhoNew,rhoOld
      integer(i4) ::  iteration,io,jo
      ! typical local variables:
      integer(i4) ::  i,j,k,l,mw,nw,iw,jw,iww,jww  ! array/matrix indices.

      allocate(z(idm,jdm,kdm,ldm))

       b=>infoVector;A=>infoMatrix
       w=>z

       p=0.0

       
       do l=1,ldm
       do k=1,kdm
        do j=1,jdm
         do i=1,idm
         if(mask(i,j,k)==1) then
           x(i,j,k,l)=0.
           r(i,j,k,l)=b(i,j,k,l)
         endif  
         enddo
        enddo
       enddo
      enddo 
      
      do iteration=1,maxIteration


      rhoNew=0.

!$OMP parallel do schedule(static) private(l,k,j,i) reduction(+:rhonew) collapse(2)

      do l=1,ldm 
       do k=1,kdm
        do j=1,jdm
         do i=1,idm
         if(mask(i,j,k)==1) then

         !z(i,j,iw,k)=r(i,j,iw,k)  ! no preconditioning.
          ! preconditioning:
          if(abs(A(i,j,0,0,k,l)).gt.1.) then
            z(i,j,k,l)=r(i,j,k,l)/A(i,j,0,0,k,l)
          else
            z(i,j,k,l)=r(i,j,k,l)
          endif
          ! preconditioning done.
          rhoNew=rhoNew+r(i,j,k,l)*z(i,j,k,l)
         endif
          enddo
        enddo
       enddo
      enddo 
!$OMP END parallel do 

      if(iteration.gt.1) betaPCG=rhoNew/rhoOld

 !$OMP parallel do schedule(static) private(l,k,j,i) collapse(2)
    do   l=1,ldm
     do  k=1,kdm 
      do j=1,jdm
        do i=1,idm
        if(mask(i,j,k)==1) then

          if(iteration.eq.1) then
            p(i,j,k,l)=z(i,j,k,l)
          else
            p(i,j,k,l)=z(i,j,k,l)+betaPCG*p(i,j,k,l)
          endif
          w(i,j,k,l)=0.
         endif
          enddo
        enddo
      enddo
     enddo 
!$OMP END parallel do     

!$OMP parallel do schedule(static) private(l,k,j,i,mw,nw,io,jo) collapse(2)    

    do l=1,ldm
     do k=1,kdm
        do j=1,jdm
        do i=1,idm
              if(mask(i,j,k)==1) then

              do nw=-maxW,maxW
                jo=j+nw
                if((jo.ge.1).and.(jo.le.jdm)) then
                  do mw=-maxW,maxW
                    io=i+mw
                   if((io.ge.1).and.(io.le.idm)) then
                      if(A(i,j,mw,nw,k,l).ne.0.) then

      w(i,j,k,l)=w(i,j,k,l)+A(i,j,mw,nw,k,l)*p(io,jo,k,l)

                      endif
                    endif
                  enddo
                endif
        enddo
         

        endif
        enddo
        enddo
       enddo
      enddo

!$OMP END parallel do      


     pw=0.
!$OMP parallel do schedule(static) private(l,k,j,i) reduction(+:pw) collapse(2) 

      do l=1,ldm
       do k=1,kdm
        do j=1,jdm
        do i=1,idm
        if(mask(i,j,k)==1) then
          pw=pw+p(i,j,k,l)*w(i,j,k,l)
        endif
       enddo
      enddo
     enddo
    enddo 

!$OMP END parallel do    


        alphaPCG=rhoNew/pw

!$OMP parallel do schedule(static) private(l,k,j,i) collapse(2)         
      do l=1,ldm  
       do k=1,kdm
        do j=1,jdm
         do i=1,idm
           if(mask(i,j,k)==1) then
             x(i,j,k,l)=x(i,j,k,l)+alphaPCG*p(i,j,k,l)
             r(i,j,k,l)=r(i,j,k,l)-alphaPCG*w(i,j,k,l)
           endif
         enddo
        enddo
       enddo
      enddo 
!$OMP END parallel do  
        
        rhoOld=rhoNew
       
!        if(diagPCG) write(6,90) iteration,rhoOld,alphaPCG,betaPCG
        if(iteration==maxIteration) then
         write(6,90) iteration,rhoOld,alphaPCG,betaPCG
        endif
90      format(' ....PCG ',I6,': ',3G12.3)

      enddo ! iteration


    do  l=1,ldm
     do k=1,kdm
      do j=1,jdm
      do i=1,idm
        if(mask(i,j,k)==1) then
         po(i,j,k,l)=x(i,j,k,l)
        endif
       enddo
      enddo
     enddo 
    enddo

where(abs(po)<0.005) po=0.0
  
 end subroutine

!--------------------------------------------------
!      subroutine statInfo(idm,jdm,kdm,ldm)

!              implicit none
!
!               integer(i4)                        :: idm,jdm
!      real(r4) rave,rmax,rr
!      ! typical local variables:
!      integer(i4) ::  i,j,k,l,mw,nw,iw,jw,iww,jww  ! array/matrix indices.

!      ! information pair:
!      real(r4),pointer  :: z(:,:,:,:),W(:,:,:,:,:,:)
!      integer(i4)   :: nmax,mmax,imax,jmax

!      z=>infoVector
!      W=>infomatrix

!      nmax=0
!      mmax=0
!      imax=0
!      jmax=0
!        rmax=0.
!        rave=0.
!        do mw=-maxW,maxW
!        do nw=-maxW,maxW
!          do i=1,idm
!          do j=1,jdm
!            rr=abs(W(i,j,mw,nw,k,l))
!            if(rmax.lt.rr) then
!              rmax=rr
!              imax=i
!              jmax=j
!              mmax=mw
!              nmax=nw
!            endif
!            rave=rave+rr
!          enddo
!          enddo
!        rave=rave/real(25*idm*jdm*kdm)
!      write(*,99) rmax,imax,jmax,mmax,nmax,rave 
!99      format('W',G12.3,4I3,G12.3)
!      enddo
!      enddo
!      end
    
end module mgmrf      
