module mgts
use mkinds
use mdates
use mncio
use mobs


 implicit none

 integer(i4),parameter      :: gts_mxnpr=10000
 integer(i4),parameter      :: gts_mxlvl=2000

 integer(i4),allocatable    :: gts_prlvls(:)
 integer(i4),allocatable    :: gts_prtyp(:)
 real(r4),allocatable       :: gts_prlon(:)
 real(r4),allocatable       :: gts_prlat(:)
 real(r4),allocatable       :: gts_prtime(:)
 real(r4),allocatable       :: gts_prdpth(:,:)
 real(r4),allocatable       :: gts_prtemp(:,:)
 real(r4),allocatable       :: gts_prsaln(:,:)

 integer(i4)                :: npr_bathy
 integer(i4)                :: npr_buoy
 integer(i4)                :: npr_tesac


contains

subroutine get_gts_tesac(cdate,mxnpr,mxlvl,prtyp,prlvls,prlon,prlat,prtime,prdpth,prtemp,prsaln,nstart,nend)
implicit none

character(len=*)           :: cdate
integer(i4)                :: mxnpr
integer(i4)                :: mxlvl
integer(i4)                :: nstart
integer(i4)                :: nend

integer(i4)                :: prtyp(mxnpr)
integer(i4)                :: prlvls(mxnpr)
real(r4)                   :: prlon(mxnpr)
real(r4)                   :: prlat(mxnpr)
real(r4)                   :: prtime(mxnpr)
real(r4)                   :: prdpth(mxnpr,mxlvl)
real(r4)                   :: prtemp(mxnpr,mxlvl)
real(r4)                   :: prsaln(mxnpr,mxlvl)


character(:),allocatable   :: tesacFile

integer(i4)                :: nprof
integer(i4)                :: nlvl
integer(i4)                :: nvalid
integer(i4)                :: fcount
integer(i4)                :: vcount


real(r4),allocatable       :: lon(:)
real(r4),allocatable       :: lat(:)
real(r4),allocatable       :: time(:)
real(r4),allocatable       :: pres(:,:)
real(r4),allocatable       :: temp(:,:)
real(r4),allocatable       :: saln(:,:)
integer(i4),allocatable    :: valid(:)

real(r4),allocatable       :: pres1d(:)
real(r4),allocatable       :: temp1d(:)
real(r4),allocatable       :: saln1d(:)
real(r8),allocatable       :: juldr8(:)
real(r4)                   :: date_offset
real(r4)                   :: t_fill_value
real(r4)                   :: s_fill_value
real(r4)                   :: p_fill_value

logical(bl)                :: lexist
integer(i4)                :: spresent



integer(i4),parameter      :: nprofmx=10000
character(cl)              :: tesacFileNames(3)
integer(i4)                :: tesacFiles
integer(i4)                :: total_files_processed
integer(i4)                :: nqc_fail
integer(i4)                :: nqc_shortfail
integer(i4)                :: nqc_gapsfail

! reject counts
integer(i4)                :: nuniq
real(r4)                   :: ipr,jpr
real(r4)                   :: hday
integer(i4)                :: i,j,k,l,ii
integer(i4)                :: kamx(2)
integer(i4)                :: kmx

! analysix date

call cymd2hday(cdate,hday)
call read_obs_location()

nuniq=0
fcount=nstart

t_fill_value =-99.0
s_fill_value =-99.0

tesacFile=trim(gts_profiles_location)//"/tesac_"//cdate(1:8)//".nc"



tesacFiles=0
nqc_fail=0
nqc_gapsfail=0
nqc_shortfail=0

INQUIRE(FILE=tesacFile, EXIST=lexist)
if(lexist) then
call nciopn(tesacFile,ncid)
call ncioin(tesacFile,ncid,"nprof",nprof)
call ncioin(tesacFile,ncid,"lvl",nlvl)
allocate(lon(nprof),lat(nprof),time(nprof))
call nciorv(tesacFile,ncid,"plon",lon)
call nciorv(tesacFile,ncid,"plat",lat)
call nciorv(tesacFile,ncid,"ptime",time)

do i=1,nprof
        if(allocated(pres)) deallocate(pres,temp,saln,valid)
        allocate(pres(nlvl,1),temp(nlvl,1),saln(nlvl,1),valid(nlvl))
        call nciorv(tesacFile,ncid,"depth",pres,[i,1],[1,nlvl])
        call nciorv(tesacFile,ncid,"temp",temp,[i,1],[1,nlvl])
        call nciorv(tesacFile,ncid,"saln",saln,[i,1],[1,nlvl])

        kamx=maxloc(pres,mask=pres>-99.0)
        kmx=kamx(1)

        !print *, minval(pres,mask=pres>-99.0),maxval(pres,mask=pres>-99.0),kmx
       
        if(kmx<2) nqc_shortfail=nqc_shortfail+1
        ! select profiles without gaps in the vertical, even one gap in either salinity or temperature
        ! disqualifies the profile

        if(kmx>2 .and. all(temp(1:kmx,1) /=t_fill_value) .and. all(saln(1:kmx,1)/=s_fill_value)) then
               fcount=fcount+1
               prtyp(fcount)=3
               prlvls(fcount)=kmx
               prlon(fcount)=lon(i)
               prlat(fcount)=lat(i)
               prdpth(fcount,1:kmx)=pres(1:kmx,1)
               prtemp(fcount,1:kmx)=temp(1:kmx,1)
               prsaln(fcount,1:kmx)=saln(1:kmx,1)
               prtime(fcount)=time(i)
       else
               nqc_gapsfail=nqc_gapsfail+1
       endif

enddo
       call nciocl(tesacFile,ncid)


        nend=fcount
        npr_tesac=fcount-nstart

Write(*,*) "....total no of tesac profiles processed:.... ", nprof
Write(*,*) "....very short tesac profiles: .... ", nqc_shortfail
Write(*,*) "....tesac profiles with gaps:.... ", nqc_gapsfail
Write(*,*) "....total no of tesac profiles selected:.... ", npr_tesac

 else

 write(*,*) ".... no tesac file ...."

 endif        



end subroutine


subroutine get_gts_bathy(cdate,mxnpr,mxlvl,prtyp,prlvls,prlon,prlat,prtime,prdpth,prtemp,prsaln,nstart,nend)
implicit none

character(len=*)           :: cdate
integer(i4)                :: mxnpr
integer(i4)                :: mxlvl
integer(i4)                :: nstart
integer(i4)                :: nend

integer(i4)                :: prtyp(mxnpr)
integer(i4)                :: prlvls(mxnpr)
real(r4)                   :: prlon(mxnpr)
real(r4)                   :: prlat(mxnpr)
real(r4)                   :: prtime(mxnpr)
real(r4)                   :: prdpth(mxnpr,mxlvl)
real(r4)                   :: prtemp(mxnpr,mxlvl)
real(r4)                   :: prsaln(mxnpr,mxlvl)


character(:),allocatable   :: bathyFile

integer(i4)                :: nprof
integer(i4)                :: nlvl
integer(i4)                :: nvalid
integer(i4)                :: fcount
integer(i4)                :: vcount
real(r4),allocatable       :: lon(:)
real(r4),allocatable       :: lat(:)
real(r4),allocatable       :: time(:)
real(r4),allocatable       :: pres(:,:)
real(r4),allocatable       :: temp(:,:)
real(r4),allocatable       :: saln(:,:)
integer(i4),allocatable    :: valid(:)

real(r4),allocatable       :: pres1d(:)
real(r4),allocatable       :: temp1d(:)
real(r4),allocatable       :: saln1d(:)
real(r8),allocatable       :: juldr8(:)
real(r4)                   :: date_offset
real(r4)                   :: t_fill_value
real(r4)                   :: s_fill_value
real(r4)                   :: p_fill_value

logical(bl)                :: lexist
integer(i4)                :: spresent



integer(i4),parameter      :: nprofmx=10000
character(cl)              :: bathyFileNames(3)
integer(i4)                :: tesacFiles
integer(i4)                :: total_files_processed
integer(i4)                :: nqc_fail
integer(i4)                :: nqc_shortfail
integer(i4)                :: nqc_gapsfail

! reject counts
integer(i4)                :: nuniq
real(r4)                   :: ipr,jpr
real(r4)                   :: hday
integer(i4)                :: i,j,k,l,ii
integer(i4)                :: kamx(2)
integer(i4)                :: kmx



! analysix date

call cymd2hday(cdate,hday)
call read_obs_location()

nuniq=0

t_fill_value =-99.0
s_fill_value =-99.0

bathyFile=trim(gts_profiles_location)//"/bathy_"//cdate(1:8)//".nc"
!bathyFiles=0
nqc_fail=0
nqc_gapsfail=0
nqc_shortfail=0
fcount=nstart


INQUIRE(FILE=bathyFile, EXIST=lexist)
if(lexist) then
call nciopn(bathyFile,ncid)
call ncioin(bathyFile,ncid,"nprof",nprof)
call ncioin(bathyFile,ncid,"lvl",nlvl)
allocate(lon(nprof),lat(nprof),time(nprof))
call nciorv(bathyFile,ncid,"plon",lon)
call nciorv(bathyFile,ncid,"plat",lat)
call nciorv(bathyFile,ncid,"ptime",time)
do i=1,nprof
        if(allocated(pres)) deallocate(pres,temp,saln,valid)
        allocate(pres(nlvl,1),temp(nlvl,1),saln(nlvl,1),valid(nlvl))
        call nciorv(bathyFile,ncid,"depth",pres,[i,1],[1,nlvl])
        call nciorv(bathyFile,ncid,"temp",temp,[i,1],[1,nlvl])
        call nciorv(bathyFile,ncid,"saln",saln,[i,1],[1,nlvl])

        kamx=maxloc(pres,mask=pres>-99.0)
        kmx=kamx(1)

        !print *, minval(pres,mask=pres>-99.0),maxval(pres,mask=pres>-99.0),kmx

        if(kmx<2) nqc_shortfail=nqc_shortfail+1
        ! select profiles without gaps in the vertical, even one gap in either salinity or temperature
        ! disqualifies the profile


        ! get clim salinity

!        call  ts_for_cdate_point(cdate(1:8),lon(i),lat(i),ct,cs,cts,css)


        if(kmx>2 .and. all(temp(1:kmx,1) /=t_fill_value)) then
               fcount=fcount+1
               prtyp(fcount)=3
               prlvls(fcount)=kmx
               prlon(fcount)=lon(i)
               prlat(fcount)=lat(i)
               prdpth(fcount,1:kmx)=pres(1:kmx,1)
               prtemp(fcount,1:kmx)=temp(1:kmx,1)
               prsaln(fcount,1:kmx)=-99.0 !cs(1:kmx)!saln(1:kmx,1)
               prtime(fcount)=time(i)
       else
               nqc_gapsfail=nqc_gapsfail+1
       endif

enddo
       call nciocl(bathyFile,ncid)


        npr_bathy=fcount-nstart
        nend=fcount

!        do i=1,fcount
!         do k=1,prlvls(i)
!          print *, prlon(i),prlat(i),prdpth(i,k),prtemp(i,k),prsaln(i,k)
!         enddo
!         print *, "***"
!         enddo


Write(*,*) "....total no of xbt/bathy profiles processed:.... ", nprof
Write(*,*) "....very short xbt profiles: .... ", nqc_shortfail
Write(*,*) "....xbt profiles with gaps:.... ", nqc_gapsfail
Write(*,*) "....total no of xbt profiles selected:.... ", npr_bathy

 else

 write(*,*) ".... no bathy file ...."

 endif

end subroutine


  subroutine get_gts_buoy(cdate,mxnpr,mxlvl,prtyp,prlvls,prlon,prlat,prtime,prdpth,prtemp,prsaln,nstart,nend)

          
  implicit none

  character(len=*)           :: cdate
  integer(i4)                :: mxnpr
  integer(i4)                :: mxlvl
  integer(i4)                :: nstart
  integer(i4)                :: nend

  integer(i4)                :: prtyp(mxnpr)
  integer(i4)                :: prlvls(mxnpr)
  real(r4)                   :: prlon(mxnpr)
  real(r4)                   :: prlat(mxnpr)
  real(r4)                   :: prtime(mxnpr)
  real(r4)                   :: prdpth(mxnpr,mxlvl)
  real(r4)                   :: prtemp(mxnpr,mxlvl)
  real(r4)                   :: prsaln(mxnpr,mxlvl)
  character(:),allocatable   :: buoyFile

integer(i4)                :: nprof
integer(i4)                :: nlvl
integer(i4)                :: nvalid
integer(i4)                :: fcount
integer(i4)                :: vcount
real(r4),allocatable       :: lon(:)
real(r4),allocatable       :: lat(:)
real(r4),allocatable       :: time(:)
real(r4),allocatable       :: pres(:,:)
real(r4),allocatable       :: temp(:,:)
real(r4),allocatable       :: saln(:,:)
integer(i4),allocatable    :: valid(:)

real(r4),allocatable       :: pres1d(:)
real(r4),allocatable       :: temp1d(:)
real(r4),allocatable       :: saln1d(:)
real(r8),allocatable       :: juldr8(:)
real(r4)                   :: date_offset
real(r4)                   :: t_fill_value
real(r4)                   :: s_fill_value
real(r4)                   :: p_fill_value

logical(bl)                :: lexist
integer(i4)                :: spresent



integer(i4),parameter      :: nprofmx=10000
character(cl)              :: buoyFileNames(3)
integer(i4)                :: total_files_processed
integer(i4)                :: nqc_fail
integer(i4)                :: nqc_shortfail
integer(i4)                :: nqc_gapsfail
! reject counts
integer(i4)                :: nuniq
real(r4)                   :: ipr,jpr
real(r4)                   :: hday
integer(i4)                :: i,j,k,l,ii
integer(i4)                :: kamx(2)
integer(i4)                :: kmx

! analysix date

call cymd2hday(cdate,hday)
call read_obs_location()

nuniq=0
fcount=nstart

t_fill_value =-99.0
s_fill_value =-99.0

buoyFile=trim(gts_profiles_location)//"/buoy_"//cdate(1:8)//".nc"
!bathyFiles=0
nqc_fail=0
nqc_gapsfail=0
nqc_shortfail=0

INQUIRE(FILE=buoyFile, EXIST=lexist)
if(lexist) then
call nciopn(buoyFile,ncid)
call ncioin(buoyFile,ncid,"nprof",nprof)
call ncioin(buoyFile,ncid,"lvl",nlvl)
allocate(lon(nprof),lat(nprof),time(nprof))
call nciorv(buoyFile,ncid,"plon",lon)
call nciorv(buoyFile,ncid,"plat",lat)
call nciorv(buoyFile,ncid,"ptime",time)
do i=1,nprof
        if(allocated(pres)) deallocate(pres,temp,saln,valid)
        allocate(pres(nlvl,1),temp(nlvl,1),saln(nlvl,1),valid(nlvl))
        call nciorv(buoyFile,ncid,"depth",pres,[i,1],[1,nlvl])
        call nciorv(buoyFile,ncid,"temp",temp,[i,1],[1,nlvl])
        call nciorv(buoyFile,ncid,"saln",saln,[i,1],[1,nlvl])

        kamx=maxloc(pres,mask=pres>-99.0)
        kmx=kamx(1)
        if(kmx<2) nqc_shortfail=nqc_shortfail+1
        ! select profiles without gaps in the vertical, even one gap in either salinity or temperature
        ! disqualifies the profile


        ! get clim salinity

        !call  ts_for_cdate_point(cdtg(1:8),lon(i),lat(i),ct,cs,cts,css)


        if(kmx>2 .and. all(temp(1:kmx,1) /=t_fill_value)) then
               fcount=fcount+1
               prtyp(fcount)=3
               prlvls(fcount)=kmx
               prlon(fcount)=lon(i)
               prlat(fcount)=lat(i)
               prdpth(fcount,1:kmx)=pres(1:kmx,1)
               prtemp(fcount,1:kmx)=temp(1:kmx,1)
               prsaln(fcount,1:kmx)=saln(1:kmx,1)
               prtime(fcount)=time(i)
       else
               nqc_gapsfail=nqc_gapsfail+1
       endif

enddo
       call nciocl(buoyFile,ncid)

        nend=fcount
        npr_buoy=fcount-nstart

!        do i=1,fcount
!         do k=1,prlvls(i)
!          print *, prlon(i),prlat(i),prdpth(i,k),prtemp(i,k),prsaln(i,k)
!         enddo
!         print *, "***"
!         enddo


Write(*,*) "....total no of buoy profiles processed:.... ", nprof
Write(*,*) "....very short buoy profiles: .... ", nqc_shortfail
Write(*,*) "....buoy profiles with gaps:.... ", nqc_gapsfail
Write(*,*) "....total no of buoy profiles selected:.... ", npr_buoy

 else

 write(*,*) ".... no buoy file ...."


 endif

 end subroutine









end module     
