MODULE mhycom
USE mkinds
USE mconstants
USE mseawater
USE mgrid, only : sigver,mdepth
USE mabio
USE mncio

INTERFACE readFromBlkdat
          MODULE PROCEDURE intFromBlkdat
          MODULE PROCEDURE realFromBlkdat
END INTERFACE readFromBlkdat


CONTAINS

subroutine update_pin(dp,fpin,nx,ny,nl)
implicit none
integer(i4)   :: nx,ny,nl
real(r4)      :: dp(nx,ny,nl)
real(r4)      :: fpin(nx,ny,nl+1)

integer       :: k

fpin(:,:,1)=0.0
do k=1,nl
      fpin(:,:,k+1)= fpin(:,:,k)+dp(:,:,k)*qonem
enddo

end subroutine


subroutine update_dp(pin,dp,mdepth,nx,ny,nl)
implicit none
integer :: nx,ny,nl

real(r4)           :: dp(:,:,:)
real(r4)           :: pin(:,:,:)
real(r4)           :: mdepth(:,:)
integer :: i,j,k



! clean analysis
do j=1,ny
 do i=1,nx
    if(mdepth(i,j)<spval) then
       pin(i,j,nl+1)=mdepth(i,j)
          do k=1,nl
              dp(i,j,k)=(pin(i,j,k+1)-pin(i,j,k))*onem
          enddo
           do k = 1, nl-1
            dp(i,j,k+1) =dp(i,j,k+1)+min(0.0,dp(i,j,k))
            dp(i,j,k)   =max(dp(i,j,k),0.0)
           end do

           ! clean up any negative thicknes 2nd pass
           do k = nl, 2, -1
            dp(i,j,k-1) = dp(i,j,k-1)+min(0.0,dp(i,j,k))
            dp(i,j,k)   =   max(dp(i,j,k),0.0)
           end do
! end post procesing
      endif
    enddo
  enddo
end subroutine

SUBROUTINE readrst(rstFile,nx,ny,nz,kapref,pref,thbase,thref,thflag,msk3d,fssh,&
                fsst,fsss,fmld,fuba,fvba,fpba,fuvl,fvvl,fthk,ftem,fsal,fden,fthkk,fpsikk,fmgp,fpin,c2r)

!--------------------------
! version 0.1 -- 2013/03/08
! ashwanth srinivasan
! Tendral LLC, Key Biscayne
! -------------------------
! This subroutine reads state variables from the hycom restart files in .ab format.
! The variables and the order are listed in the .b summary file. Here is a 
! sample from hycom version 2.2.35:
! RESTART2: iexpt,iversn,yrflag,sigver =    100    22     3     2
! RESTART2: nstep,dtime,thbase =     14745240   40959.0000000000 34.0000000000000
! ...... followed by.......
! 2 timelevels for uvl 2*nz records
! 2 timelevels for vvv 2*nz records
! 2 timelevels for thk 2*nz records
! 2 timelevels for tem 2*nz records
! 2 timelevels for sal 2*nz records
! ..... folowed by 2d variables .....
! ubavg   : layer,tlevel,range =   0  1    -8.3196521E-01   2.8939241E-01
! ubavg   : layer,tlevel,range =   0  2    -8.3003163E-01   2.8942749E-01
! ubavg   : layer,tlevel,range =   0  3    -8.3184856E-01   2.8939536E-01
! vbavg   : layer,tlevel,range =   0  1    -2.7728680E-01   9.4853359E-01
! vbavg   : layer,tlevel,range =   0  2    -2.7770573E-01   9.5031315E-01
! vbavg   : layer,tlevel,range =   0  3    -2.7730376E-01   9.4855434E-01
! pbavg   : layer,tlevel,range =   0  1    -3.3379419E+03   5.6284883E+03
! pbavg   : layer,tlevel,range =   0  2    -3.3375940E+03   5.6294312E+03
! pbavg   : layer,tlevel,range =   0  3    -3.3379399E+03   5.6285479E+03
! pbot    : layer,tlevel,range =   0  1     0.0000000E+00   5.5346384E+07
! psikk   : layer,tlevel,range =   0  1    -1.9182270E+01   3.5327896E-02
! thkk    : layer,tlevel,range =   0  1    -1.5279036E+01   3.1667187E+00
! dpmixl  : layer,tlevel,range =   0  1     1.5899357E+04   1.1576816E+06
! dpmixl  : layer,tlevel,range =   0  2     1.5899357E+04   1.1576816E+06
! Note: Depending on the thermobaric reference state psikk and thkk may have 2
! states. Check to make sure that the fields and the order of reading in this 
! subroutine are are consistent with the appropriate .b file. We always read
! the second time level, change this below if necessary. Also note that when
! reading restart files we need to calculate the surface elevation and can 
! store forecast Montgomery potential for calculating geostrophic adjustment
! after analysis. 

IMPLICIT NONE

! Argument Declarations

CHARACTER(len=*), INTENT(IN)           :: rstFile
INTEGER(i4),      INTENT(IN)           :: nx
INTEGER(i4),      INTENT(IN)           :: ny
INTEGER(i4),      INTENT(IN)           :: nz
INTEGER(i4),      INTENT(IN)           :: kapref
REAL(r4),         INTENT(IN)           :: pref
REAL(r4),         INTENT(IN)           :: thbase
REAL(r4),         INTENT(IN)           :: thref
INTEGER(i4),      INTENT(IN)           :: thflag
integer(i4),      INTENT(INOUT)           :: msk3d(:,:,:)
REAL(r4),         INTENT(INOUT)        :: fssh(:,:)
REAL(r4),         INTENT(INOUT)        :: fsst(:,:)
REAL(r4),         INTENT(INOUT)        :: fsss(:,:)
REAL(r4),         INTENT(INOUT)        :: fmld(:,:)
REAL(r4),         INTENT(INOUT)        :: fuba(:,:)
REAL(r4),         INTENT(INOUT)        :: fvba(:,:)
REAL(r4),         INTENT(INOUT)        :: fuvl(:,:,:)
REAL(r4),         INTENT(INOUT)        :: fvvl(:,:,:)
REAL(r4),         INTENT(INOUT)        :: fthk(:,:,:)
REAL(r4),         INTENT(INOUT)        :: ftem(:,:,:)
REAL(r4),         INTENT(INOUT)        :: fsal(:,:,:)
REAL(r4),         INTENT(INOUT)        :: fden(:,:,:)
REAL(r4),         INTENT(INOUT)        :: fpin(:,:,:)
REAL(r4),         INTENT(INOUT)        :: fmgp(:,:,:)
REAL(r4),         INTENT(INOUT)        :: fpsikk(:,:,:)
REAL(r4),         INTENT(INOUT)        :: fthkk(:,:,:)
REAL(r4),         INTENT(INOUT)        :: fpba(:,:)
integer(i4), INTENT(IN),optional       :: c2r
! Local Variables
INTEGER(i4)                            :: iunit
INTEGER(i4)                            :: k,i,j
CHARACTER(:),ALLOCATABLE               :: FileName
REAL(r4)                               :: hmin
REAL(r4)                               :: hmax
REAL(r4),  ALLOCATABLE                 :: ftmp(:,:)
character(:),allocatable               :: grdFile
integer(i4)                            :: src_nx,src_ny
integer(i4),allocatable        :: xmap(:,:)
   integer(i4),allocatable     :: ymap(:,:)
! Executable Section


FileName=rstFile(1:LEN_TRIM(rstFile))
iunit=11

! Initialize hycom style IO
grdFile="gridinfo.nc"
if(c2r==1) then
        call nciopn(grdFile,fid)
        call ncioin(grdFile,fid,"xcc",src_nx)
        call ncioin(grdFile,fid,"ycc",src_ny)
          allocate(ftmp(src_nx,src_ny))
          allocate(xmap(nx,ny))
          allocate(ymap(nx,ny))
        call nciorv(grdFile,fid,"c2rxmap",xmap)
        call nciorv(grdFile,fid,"c2rymap",ymap)
         call nciocl(grdFile,fid)
Write(*,'(a)') " ....c2r is on ...."

CALL zaiost(src_nx,src_ny)
CALL zaiopn(FileName,iunit)


! skip nz fields and read nz uvl(xward) fields at the second time level 

DO k=1,nz
  CALL zaiosk(iunit)                    
ENDDO

DO k=1,nz
  CALL zaiord(iunit,ftmp,hmin,hmax)
  do j=1,ny
     do i=1,nx
    fuvl(i,j,k)=ftmp(xmap(i,j),ymap(i,j)) 
     enddo
  enddo   

ENDDO

! skip nz fields and read nz vvl(yward) fields at the second time level 

DO k=1,nz
  CALL zaiosk(iunit)                      
ENDDO

DO k=1,nz
   CALL zaiord(iunit,ftmp,hmin,hmax)
   do j=1,ny
     do i=1,nx
    fvvl(i,j,k)=ftmp(xmap(i,j),ymap(i,j))
     enddo
  enddo   
ENDDO

! skip nz fields and read nz thickness fields at the second time level 

DO k=1,nz
  CALL zaiosk(iunit)
ENDDO

DO k=1,nz
   CALL zaiord(iunit,ftmp,hmin,hmax) 
   do j=1,ny
     do i=1,nx
    fthk(i,j,k)=ftmp(xmap(i,j),ymap(i,j))
     enddo
  enddo   
   
ENDDO

! skip nz fields and read nz tem fields at the second time level 

DO k=1,nz
  CALL zaiosk(iunit)
ENDDO

DO k=1,nz
  CALL zaiord(iunit,ftmp,hmin,hmax)
  do j=1,ny
     do i=1,nx
    ftem(i,j,k)=ftmp(xmap(i,j),ymap(i,j))
     enddo
  enddo     
ENDDO

! skip nz fields and read nz sal fields at the second time level 

DO k=1,nz
  CALL zaiosk(iunit)
ENDDO

DO k=1,nz
  CALL zaiord(iunit,ftmp,hmin,hmax)
  do j=1,ny
     do i=1,nx
    fsal(i,j,k)=ftmp(xmap(i,j),ymap(i,j))
     enddo
  enddo     
ENDDO

! now read 2d fields; get the mid-time level
! ubaro 

CALL zaiosk(iunit) 
CALL zaiord(iunit,ftmp,hmin,hmax) ! mid-time ubaro
do j=1,ny
     do i=1,nx
    fuba(i,j)=ftmp(xmap(i,j),ymap(i,j))
     enddo
  enddo   
CALL zaiosk(iunit)

! vbaro 

CALL zaiosk(iunit)
CALL zaiord(iunit,ftmp(:,:),hmin,hmax) ! mid-time vbaro
do j=1,ny
     do i=1,nx
    fvba(i,j)=ftmp(xmap(i,j),ymap(i,j))
     enddo
  enddo
CALL zaiosk(iunit)

! pbaro

CALL zaiosk(iunit)
CALL zaiord(iunit,ftmp(:,:),hmin,hmax) ! mid-time pbaro
   do j=1,ny
     do i=1,nx
       fpba(i,j)=ftmp(xmap(i,j),ymap(i,j))
     enddo
   enddo

CALL zaiosk(iunit)

! skip pbot

CALL zaiosk(iunit)

! read psikk and thkk depending on the thermobaric ref state

IF (kapref >= 0) THEN
    !CALL nciopn("gridinfo.nc",fid)
    !CALL nciorv("gridinfo.nc",fid,"psikk",fpsikk(:,:,1))
    !CALL nciorv("gridinfo.nc",fid,"thkk",fthkk(:,:,1))
    !call nciocl("gridinfo.nc",fid)
    CALL zaiord(iunit,ftmp,hmin,hmax) ! psikk
    do j=1,ny
     do i=1,nx
      fpsikk(i,j,1)=ftmp(xmap(i,j),ymap(i,j))
     enddo
    enddo
    CALL zaiord(iunit,ftmp,hmin,hmax) ! thkk
    do j=1,ny
     do i=1,nx
      fthkk(i,j,1)=ftmp(xmap(i,j),ymap(i,j))
     enddo
    enddo
ELSE

    CALL zaiord(iunit,ftmp,hmin,hmax) ! psikk - ref state 1    
    do j=1,ny
     do i=1,nx
      fpsikk(i,j,1)=ftmp(xmap(i,j),ymap(i,j))
     enddo
    enddo
    CALL zaiord(iunit,ftmp,hmin,hmax) ! psikk - ref state 2
    do j=1,ny
     do i=1,nx
      fpsikk(i,j,2)=ftmp(xmap(i,j),ymap(i,j))
     enddo
    enddo
    
    CALL zaiord(iunit,ftmp,hmin,hmax)  ! thkk  - ref state 1 
      do j=1,ny
     do i=1,nx
      fthkk(i,j,1)=ftmp(xmap(i,j),ymap(i,j))
     enddo
    enddo
    CALL zaiord(iunit,ftmp,hmin,hmax)  ! thkk  - ref state 2  
    do j=1,ny
     do i=1,nx
      fthkk(i,j,2)=ftmp(xmap(i,j),ymap(i,j))
     enddo
    enddo

ENDIF

! skip first dpbl and read second

CALL zaiosk(iunit)
CALL zaiord(iunit,ftmp,hmin,hmax) ! dpbl 
    do j=1,ny
     do i=1,nx
      fmld(i,j)=ftmp(xmap(i,j),ymap(i,j))
     enddo
    enddo

! Close .ab files
call zaiocl(iunit)

else ! no c2r

CALL zaiost(nx,ny)
CALL zaiopn(FileName,iunit)


! skip nz fields and read nz uvl(xward) fields at the second time level 

DO k=1,nz
  CALL zaiosk(iunit)
ENDDO

DO k=1,nz
  CALL zaiord(iunit,fuvl(:,:,k),hmin,hmax)
ENDDO

! skip nz fields and read nz vvl(yward) fields at the second time level 

DO k=1,nz
  CALL zaiosk(iunit)
ENDDO

DO k=1,nz
   CALL zaiord(iunit,fvvl(:,:,k),hmin,hmax)
ENDDO

! skip nz fields and read nz thickness fields at the second time level 

DO k=1,nz
  CALL zaiosk(iunit)
ENDDO

DO k=1,nz
   CALL zaiord(iunit,fthk(:,:,k),hmin,hmax)
ENDDO

! skip nz fields and read nz tem fields at the second time level 

DO k=1,nz
  CALL zaiosk(iunit)
ENDDO

DO k=1,nz
  CALL zaiord(iunit,ftem(:,:,k),hmin,hmax)
ENDDO

! skip nz fields and read nz sal fields at the second time level 

DO k=1,nz
  CALL zaiosk(iunit)
ENDDO

DO k=1,nz
  CALL zaiord(iunit,fsal(:,:,k),hmin,hmax)
ENDDO

! skip uhrly and vhrly
!if(ltides) then
!!Alex DO k=1,4*nz
!DO k=1,49*2
!  CALL zaiosk(iunit)
!ENDDO
!endif


! now read 2d fields; get the mid-time level
! ubaro 

CALL zaiosk(iunit)
CALL zaiord(iunit,fuba(:,:),hmin,hmax) ! mid-time ubaro
CALL zaiosk(iunit)

! vbaro 

CALL zaiosk(iunit)
CALL zaiord(iunit,fvba(:,:),hmin,hmax) ! mid-time vbaro
CALL zaiosk(iunit)

! pbaro

CALL zaiosk(iunit)
CALL zaiord(iunit,fpba(:,:),hmin,hmax) ! mid-time pbaro
CALL zaiosk(iunit)

! skip pbot

CALL zaiosk(iunit)

! read psikk and thkk depending on the thermobaric ref state

IF (kapref >= 0) THEN
    !CALL nciopn("gridinfo.nc",fid)
    !CALL nciorv("gridinfo.nc",fid,"psikk",fpsikk(:,:,1))
    !CALL nciorv("gridinfo.nc",fid,"thkk",fthkk(:,:,1))
    !call nciocl("gridinfo.nc",fid)
    CALL zaiord(iunit,fpsikk(:,:,1),hmin,hmax) ! psikk
    CALL zaiord(iunit,fthkk(:,:,1),hmin,hmax) ! thkk
ELSE

    CALL zaiord(iunit,fpsikk(:,:,1),hmin,hmax) ! psikk - ref state 1    
    CALL zaiord(iunit,fpsikk(:,:,2),hmin,hmax) ! psikk - ref state 2
    CALL zaiord(iunit,fthkk(:,:,1),hmin,hmax)  ! thkk  - ref state 1 
    CALL zaiord(iunit,fthkk(:,:,2),hmin,hmax)  ! thkk  - ref state 2  

ENDIF

! skip first dpbl and read second

CALL zaiosk(iunit)
CALL zaiord(iunit,fmld(:,:),hmin,hmax) ! dpbl 

! Close .ab files
call zaiocl(iunit)



endif  ! c2r
! get the correct mask
msk3d=1
do k=1,nz
 do j=1,ny
  do i=1,nx
  if(fthk(i,j,k)>=spval) then
          msk3d(i,j,k)=0
  endif
  enddo
 enddo
enddo



! !convert baroclinic to total velocities, interpolate to p point and mask bad values

do k=1,nz
fuvl(:,:,k)=fuvl(:,:,k)+fuba
fvvl(:,:,k)=fvvl(:,:,k)+fvba
enddo

! intp u and v to p points
do k=1,nz
 do j=2,ny-2
  do i=2,nx-2
     if(fuvl(i,j,k) .lt. spval .and. fuvl(i+1,j,k) .lt.spval) then
     fuvl(i,j,k)=0.5*(fuvl(i,j,k)+fuvl(i+1,j,k))
     else
     fuvl(i,j,k)=spval
     endif
  enddo
enddo
enddo

do k=1,nz
 do j=2,ny-2
  do i=2,nx-2
     if(fvvl(i,j,k) .lt. spval .and. fvvl(i,j+1,k) .lt.spval) then
     fvvl(i,j,k)=0.5*(fvvl(i,j,k)+fvvl(i,j+1,k))
     else
     fvvl(i,j,k)=spval
     endif
  enddo
 enddo
enddo


! calculate density

do  k=1,nz
 do j=1,ny
  do i=1,nx
    if(mdepth(i,j)<spval) then
    fden(i,j,k)=sig(ftem(i,j,k),fsal(i,j,k),sigver)
    endif
  enddo
 enddo
enddo

do j=1,ny
  do i=1,nx
    if(mdepth(i,j)>=spval) then
      fpba(i,j)=0.0
      fpsikk(i,j,:)=0.0
      fthkk(i,j,:)=0.0
    endif
  enddo
enddo


! calculate ssh
CALL getssh(nx,ny,nz,kapref,pref,thbase,thref,msk3d(:,:,1),fsal,ftem,fthk,fden-thbase,fpba,fpsikk,fthkk,fmgp,fssh)
where(mdepth>spval) fssh=spval

fpin=0.0
! calculate interface depths
do k=1,nz
  do j=1,ny
  do i=1,nx
        if(msk3d(i,j,k)==1) then
         fpin(i,j,k+1)= fpin(i,j,k)+fthk(i,j,k)*qonem
        else 
         fpin(i,j,k+1)= spval
        endif
enddo
enddo
enddo
! convert mixed layer depth to m
where(mdepth<spval)
fmld=fmld*qonem
end where
fsst=ftem(:,:,1)
fsss=fsal(:,:,1)

do  k=1,nz
 do j=1,ny
  do i=1,nx
    if(msk3d(i,j,k)==0) then
    fden(i,j,k)=spval
    fsal(i,j,k)=spval
    ftem(i,j,k)=spval
    fthk(i,j,k)=spval 
    fvvl(i,j,k)=spval
    fuvl(i,j,k)=spval
    endif
  enddo
 enddo
enddo

END SUBROUTINE

subroutine writerst(nx,ny,nz,sigver,kapnum,lglb,thbase,mday,dst_uvel,dst_vvel,dst_temp,dst_saln,dst_dp,&
                dst_ub,dst_vb,dst_pb,dst_pbot,dst_psikk,dst_thkk,dst_dpmixl,&
                dst_covice,dst_temice,dst_thkice)

USE mkinds
USE mabio
USE mstring



integer(i4), intent(in) :: nx
integer(i4), intent(in) :: ny
integer(i4), intent(in) :: nz
integer(i4), intent(in) :: sigver
integer(i4), intent(in) :: kapnum
logical,     intent(in) :: lglb
real(r4),    intent(in) :: thbase
real(r4),    intent(in) :: mday


real(r4)                :: dst_uvel(:,:,:)
real(r4) :: dst_vvel(:,:,:)
real(r4) :: dst_temp(:,:,:)
real(r4) :: dst_saln(:,:,:)
real(r4) :: dst_dp(:,:,:)
real(r4) :: dst_ub(:,:)
real(r4) :: dst_vb(:,:)
real(r4) :: dst_pb(:,:)
real(r4) :: dst_pbot(:,:)
real(r4) :: dst_psikk(:,:,:)
real(r4) :: dst_thkk(:,:,:)
real(r4)              :: dst_dpmixl(:,:)

real(r4),optional     :: dst_covice(:,:)
real(r4),optional     :: dst_temice(:,:)
real(r4),optional     :: dst_thkice(:,:)

integer(i4)                    :: iversn
integer(i4)                    :: iexpt
integer(i4)                    :: yrflag
integer(i4)                    :: iunit
integer(i4)                    :: baclin
integer(i4)                    :: stat
real(r4),allocatable           :: xmin(:),xmax(:)
real(r4)                       :: hmin,hmax
character(8)                   :: cmtime
character(:),allocatable       :: rstFile

iexpt=10
iversn=22
yrflag=3
iunit=11
baclin=720
nstep = nint(mday/(baclin/86400.0d0))

call flt2str(mday,cmtime,'(F8.2)',stat)
rstFile="restart_outr"!"restart_i"//cmtime
allocate(xmin(nz))
allocate(xmax(nz))
! Initialize hycom style IO

CALL zaiost(nx,ny)
CALL zaiopn(rstFile,iunit)
write(iunit,'(a,4i6)') 'RESTART2: iexpt,iversn,yrflag,sigver = ',iexpt,iversn,yrflag,sigver
write(iunit,'(a,I9,3x,F8.2,3x,F4.1)')   'RESTART2: nstep,dtime,thbase = ',nstep,mday,thbase

! uvel
do k=1,nz
CALL zaiowr(iunit,dst_uvel(:,:,k),hmin,hmax)
enddo
do k=1,nz
CALL zaiowr(iunit,dst_uvel(:,:,k),hmin,hmax)
xmin(k)=hmin
xmax(k)=hmax
enddo
do l= 0,1
 do k= 1,nz
    write(iunit,4100) 'u       ',k,l+1,xmin(k),xmax(k)
 enddo
enddo

! vvel
do k=1,nz
CALL zaiowr(iunit,dst_vvel(:,:,k),hmin,hmax)
enddo
do k=1,nz
CALL zaiowr(iunit,dst_vvel(:,:,k),hmin,hmax)
xmin(k)=hmin
xmax(k)=hmax
enddo
do l= 0,1
 do k= 1,nz
    write(iunit,4100) 'v       ',k,l+1,xmin(k),xmax(k)
 enddo
enddo

! dp

do k=1,nz
CALL zaiowr(iunit,dst_dp(:,:,k),hmin,hmax)
enddo
do k=1,nz
CALL zaiowr(iunit,dst_dp(:,:,k),hmin,hmax)
xmin(k)=hmin
xmax(k)=hmax
enddo
do l= 0,1
 do k= 1,nz
    write(iunit,4100) 'dp      ',k,l+1,xmin(k),xmax(k)
 enddo
enddo

! temp

do k=1,nz
CALL zaiowr(iunit,dst_temp(:,:,k),hmin,hmax)
enddo
do k=1,nz
CALL zaiowr(iunit,dst_temp(:,:,k),hmin,hmax)
xmin(k)=hmin
xmax(k)=hmax
enddo
do l= 0,1
 do k= 1,nz
    write(iunit,4100) 'temp    ',k,l+1,xmin(k),xmax(k)
 enddo
enddo

! salin
do k=1,nz
CALL zaiowr(iunit,dst_saln(:,:,k),hmin,hmax)
enddo
do k=1,nz
CALL zaiowr(iunit,dst_saln(:,:,k),hmin,hmax)
xmin(k)=hmin
xmax(k)=hmax
enddo
do l= 0,1
 do k= 1,nz
    write(iunit,4100) 'saln    ',k,l+1,xmin(k),xmax(k)
 enddo
enddo

CALL zaiowr(iunit,dst_ub,hmin,hmax)
CALL zaiowr(iunit,dst_ub,hmin,hmax)
CALL zaiowr(iunit,dst_ub,hmin,hmax)
do l= 1,3
   do k= 0,0
      write(iunit,4100) 'ubavg   ',k,l,  hmin,hmax
   enddo
enddo

CALL zaiowr(iunit,dst_vb,hmin,hmax)
CALL zaiowr(iunit,dst_vb,hmin,hmax)
CALL zaiowr(iunit,dst_vb,hmin,hmax)

do l= 1,3
  do k= 0,0
          write(iunit,4100) 'vbavg   ',k,l, hmin,hmax
  enddo
enddo

CALL zaiowr(iunit,dst_pb,hmin,hmax)
CALL zaiowr(iunit,dst_pb,hmin,hmax)
CALL zaiowr(iunit,dst_pb,hmin,hmax)

do l= 1,3
  do k= 0,0
          write(iunit,4100) 'pbavg   ',k,l, hmin,hmax
  enddo
enddo
k=0
l=1
CALL zaiowr(iunit,dst_pbot,hmin,hmax)
write(iunit,4100) 'pbot    ',k,l, hmin,hmax

do l=1,kapnum
CALL zaiowr(iunit,dst_psikk(:,:,l),hmin,hmax)
write(iunit,4100) 'psikk   ',k,l, hmin,hmax
enddo

do l=1,kapnum
CALL zaiowr(iunit,dst_thkk(:,:,l),hmin,hmax)
write(iunit,4100) 'thkk    ',k,l, hmin,hmax
enddo



CALL zaiowr(iunit,dst_dpmixl,hmin,hmax)
CALL zaiowr(iunit,dst_dpmixl,hmin,hmax)

do l= 1,2
  do k= 0,0
          write(iunit,4100) 'dpmixl  ',k,l, hmin,hmax
  enddo
enddo

k=0
l=1
if(lice) then
  CALL zaiowr(iunit,dst_temice,hmin,hmax)
  write(iunit,4100) 'temice    ',k,l, hmin,hmax

  CALL zaiowr(iunit,dst_covice,hmin,hmax)
  write(iunit,4100) 'covice    ',k,l, hmin,hmax

  CALL zaiowr(iunit,dst_thkice,hmin,hmax)
  write(iunit,4100) 'thkice    ',k,l, hmin,hmax
endif



call zaiocl(iunit)
4100 format(a,': layer,tlevel,range = ',i3,i3,2x,1p2e16.7)
end subroutine






subroutine getssh(nx,ny,nz,kapref,pref,thbase,thref,imask,s,t,dp,r,pb,psikk,thkk,fmontg,fssh)

implicit none

integer(i4), intent(in)         :: nx
integer(i4), intent(in)         :: ny
integer(i4), intent(in)         :: nz
integer(i4), intent(in)         :: kapref
real(r4),    intent(in)         :: pref
real(r4),    intent(in)         :: thbase
real(r4),    intent(in)         :: thref
integer(i4),  intent(in)        :: imask(:,:)
real(r4),    intent(in)         :: pb(:,:)
real(r4),    intent(in)         :: psikk(:,:,:)
real(r4),    intent(in)         :: thkk(:,:,:)
real(r4),    intent(in)         :: s(:,:,:)
real(r4),    intent(in)         :: t(:,:,:)
real(r4),    intent(in)         :: dp(:,:,:)
real(r4),    intent(in)         :: r(:,:,:)
real(r4),    intent(out)        :: fssh(:,:)
real(r4),    intent(out)        :: fmontg(:,:,:)
real(r4),allocatable           :: montgl(:,:,:,:)


real(r4),    allocatable        :: thstar(:,:)
real(r4),    allocatable        :: montg(:,:,:)
real(r4),    allocatable        :: montg1(:,:)
real(r4),    allocatable        :: p(:,:,:)
integer(i4), allocatable        :: kapi(:,:)
real(r4),    allocatable        :: skap(:,:)
real(r4),    allocatable        :: tbaric(:,:)
real(r4)                        :: oneta
real(r4)                        :: hmin
real(r4)                        :: hmax
integer(i4)                     :: i
integer(i4)                     :: j
integer(i4)                     :: k
integer(i4)                     :: kk
integer(i4)                     :: ifunit
character(:),allocatable        :: FileName




allocate(thstar(nz,2))
allocate(montg(nx,ny,2))
allocate(montg1(nx,ny))
allocate(p(nx,ny,nz+1))
allocate(montgl(nx,ny,nz,2))

kk=nz

! set intf pressure.

p(:,:,1)=0.0
do k=1,nz
 do j=1,ny
  do i=1,nx
   p(i,j,k+1)=p(i,j,k)+dp(i,j,k) 
  enddo
 enddo 
enddo

!read tbaric if needed depending on kapref
if(kapref == -1) then
      allocate(skap(nx,ny))
      allocate(kapi(nx,ny))
      allocate(tbaric(0:nx+1,0:ny+1))
      filename="gridinfo.nc"
      CALL nciopn(filename,fid)
      CALL nciorv(filename,fid,"tbaric",tbaric(1:nx,1:ny))
      CALL nciocl(filename,fid)
      
      tbaric(0,:)=tbaric(1,:)
      tbaric(:,0)=tbaric(:,1)
      tbaric(nx+1,:)=tbaric(nx,:)
      tbaric(:,ny+1)=tbaric(:,ny)

      where(tbaric>3.0) tbaric=1.0 
      do j= 1,ny
        do i= 1,nx
          if (imask(i,j) .eq. 1) then
             if (max(tbaric(i,j),tbaric(i,j),tbaric(i+1,j),tbaric(i,j-1),tbaric(i,j+1) ).gt.2.0) then
                  kapi(i,j) = 3              !kapi
                  skap(i,j) = 3.0 - tbaric(i,j)
             else
                  kapi(i,j) = 1              !kapi
                  skap(i,j) = tbaric(i,j) - 1.0
            endif
          endif 
         enddo
      enddo
endif

do j = 1,ny
 do i =1,nx
   if (imask(i,j) .eq. 1) then
    do k=1,nz
      if(kapref.ne.0) then !thermobaric case
   
        if(kapref.gt.0) then
             thstar(k,1) = r(i,j,k)+kappaf(t(i,j,k),s(i,j,k),r(i,j,k)+thbase,p(i,j,k),pref,kapref)
        else
             thstar(k,1) = r(i,j,k)+kappaf(t(i,j,k),s(i,j,k),r(i,j,k)+thbase,p(i,j,k),pref,2)
             thstar(k,2) = r(i,j,k)+kappaf(t(i,j,k),s(i,j,k),r(i,j,k)+thbase,p(i,j,k),pref,kapi(i,j))
        endif

    else ! non thermobaric case

        thstar(k,1) =   r(i,j,k)

    endif
      !print *, thstar(k,1),thstar(k,2) 
     enddo


oneta = 1.0 + pb(i,j)/p(i,j,kk+1)
!    print *, oneta,p(i,j,kk+1)
if (kapref.eq.-1) then
    montg(i,j,1) =  psikk(i,j,1) + (p(i,j,kk+1)*(thkk(i,j,1)-thstar(kk,1))-pb(i,j)*(thstar(kk,1)+thbase))*thref**2
    montg(i,j,2) =  psikk(i,j,2) + (p(i,j,kk+1)*(thkk(i,j,2)-thstar(kk,2))-pb(i,j)*(thstar(kk,2)+thbase))*thref**2
    montgl(i,j,kk,1)=montg(i,j,1)
    montgl(i,j,kk,2)=montg(i,j,2)
else
    montg(i,j,1) =  psikk(i,j,1) +  (p(i,j,kk+1)*(thkk(i,j,1)-thstar(kk,1))-pb(i,j)*(thstar(kk,1)+thbase))*thref**2
    montgl(i,j,kk,1)=montg(i,j,1)
endif !kapref.eq.-1
!    print *, montgl(i,j,kk,1),psikk(i,j,1),thkk(i,j,1)
do k=kk-1,1,-1
    montg(i,j,1) = montg(i,j,1)+p(i,j,k+1)*oneta*(thstar(k+1,1)-thstar(k,1))*thref**2
    montgl(i,j,k,1)=montg(i,j,1)
!    print *, montgl(i,j,k,1)
    if(kapref.eq.-1) then
       montg(i,j,2) = montg(i,j,2)+p(i,j,k+1)*oneta*(thstar(k+1,2)-thstar(k,2))*thref**2
       montgl(i,j,k,2)=montg(i,j,2)
    endif 
!print *,k, montgl(i,j,k,1),montgl(i,j,k,2)
enddo


if(kapref .ge. 0) then
   montg1(i,j) = montg(i,j,1)
   fmontg(i,j,:)=montgl(i,j,:,1)
else
   montg1(i,j) = skap(i,j)*montg(i,j,1) + (1.0-skap(i,j))*montg(i,j,2)
   fmontg(i,j,:)=skap(i,j)*montgl(i,j,:,1)+(1.0-skap(i,j))*montgl(i,j,:,2)
   
endif

   fssh(i,j) = (montg1(i,j) + thref*pb(i,j))/9.8
 !  print *, fssh(i,j),thref*pb(i,j),montg1(i,j),skap(i,j)
 else
   fssh(i,j)=2.00**100

  endif
 enddo
enddo
end subroutine getssh

SUBROUTINE readarchv(archvFile,nx,ny,nz,time,thbase,thflag,sver,mlayer,mdepth,ssh,mld,uba,vba,uvl,vvl,thk,tem,sal)

 IMPLICIT NONE

! Argument Declarations

CHARACTER(len=*), INTENT(IN)           :: archvFile
INTEGER(i4),      INTENT(IN)           :: nx
INTEGER(i4),      INTENT(IN)           :: ny
INTEGER(i4),      INTENT(IN)           :: nz
REAL(r4)                               :: thbase
INTEGER(i4),      INTENT(IN)           :: sver
REAL(r4),         INTENT(INOUT)        :: mlayer(:)
REAL(r4),         INTENT(INOUT)        :: mdepth(:,:)
REAL(r4),         INTENT(INOUT)        :: ssh(:,:)
REAL(r4),         INTENT(INOUT)        :: mld(:,:)
REAL(r4),         INTENT(INOUT)        :: uba(:,:)
REAL(r4),         INTENT(INOUT)        :: vba(:,:)
REAL(r4),         INTENT(INOUT)        :: uvl(:,:,:)
REAL(r4),         INTENT(INOUT)        :: vvl(:,:,:)
REAL(r4),         INTENT(INOUT)        :: thk(:,:,:)
REAL(r4),         INTENT(INOUT)        :: tem(:,:,:)
REAL(r4),         INTENT(INOUT)        :: sal(:,:,:)

! Local Variables

INTEGER(i4)                            :: iunit,iyy,idd,imm,idoy,ihh
INTEGER(i4)                            :: k,ii,jj,kk
REAL(r4)                               :: hmin
REAL(r4)                               :: hmax
REAL(r4)                               :: uzero
REAL(r4)                               :: vzero
!REAL(r4)                               :: dummy(nx,ny)
!REAL(r4)                               :: u_btrop(nx,ny)
!REAL(r4)                               :: v_btrop(nx,ny)
CHARACTER(len=80), DIMENSION(5)        :: preambl
CHARACTER(len=80)                      :: cline
character(len=80)                      :: ctitle(4)
integer(i4)                            :: yrflag,iversn,sigver,iexpt,nstep,thflag
real(r4)                               :: time,coord

! Executable Section

iunit=11
!nstep=int(time*240)
!ctitle(1)="TSIS Analysis for incremental update"
!ctitle(2)="WOA18 Jan. init; KPP; SeaWiFS KPAR; e-loan ice; A=20; Smag=.05; veldf4=.02;"
!ctitle(3)="S-Z(14-18):dp00/f/x/i=3m/1.18/120m/1m;ds=0.5m/1.18/75m; var den: Med+Black Seas;"
!ctitle(4)="Dummy Info in the above two lines"
!iversn=00
!iexpt=000
!yrflag=3


call zaiost(nx,ny)
call zaiopn(trim(archvFile),iunit)
!write(iunit,116) ctitle,iversn,iexpt,yrflag,nx,ny

call zaiord(iunit,ssh,hmin,hmax)
!WRITE (iunit,117) 'montg1  ',nstep,time,sver,thbase,hmin,hmax

call zaiord(iunit,ssh,hmin,hmax)
!WRITE (iunit,117) 'srfhgt  ',nstep,time,0,0,hmin,hmax

!call zaiord(iunit,ssh,hmin,hmax)
!WRITE (iunit,117) 'steric  ',nstep,time,0,0,hmin,hmax

call zaiord(iunit,mld,hmin,hmax)
!WRITE (iunit,117) 'surflx  ',nstep,time,0,0,hmin,hmax

call zaiord(iunit,mld,hmin,hmax)
!WRITE (iunit,117) 'salflx  ',nstep,time,0,0,hmin,hmax

call zaiord(iunit,mld,hmin,hmax)
!WRITE (iunit,117) 'bl_dpth  ',nstep,time,0,0,hmin,hmax

call zaiord(iunit,mld,hmin,hmax)
!WRITE (iunit,117) 'mix_dpth  ',nstep,time,0,0,hmin,hmax

call zaiord(iunit,uba,hmin,hmax)
!WRITE (iunit,117) 'u_btrop  ',nstep,time,0,0,hmin,hmax

call zaiord(iunit,vba,hmin,hmax)
!WRITE (iunit,117) 'v_btrop  ',nstep,time,0,0,hmin,hmax

!     layer loop
!     layer loop

DO k=1,nz

coord=mlayer(k)

CALL zaiord(iunit,uvl(:,:,k),hmin,hmax)
!WRITE (iunit,117) 'u-vel.  ',nstep,time,k,coord,hmin,hmax

CALL zaiord(iunit,vvl(:,:,k),hmin,hmax)
!WRITE (iunit,117) 'v-vel.  ',nstep,time,k,coord,hmin,hmax

CALL zaiord(iunit,thk(:,:,k),hmin,hmax)
!WRITE (iunit,117) 'thknss   ',nstep,time,k,coord,hmin,hmax
CALL zaiord(iunit,tem(:,:,k),hmin,hmax)
!WRITE (iunit,117) 'temp    ',nstep,time,k,coord,hmin,hmax
CALL zaiord(iunit,sal(:,:,k),hmin,hmax)
!WRITE (iunit,117) 'salin   ',nstep,time,k,coord,hmin,hmax
!print *, hmin,hmax
ENDDO

call zaiocl(iunit)

116  format (a80/a80/a80/a80/ &
i5,4x,'''iversn'' = hycom version number x10'/ &
i5,4x,'''iexpt '' = experiment number x10'/    &
i5,4x,'''yrflag'' = days in year flag'/        &
i5,4x,'''idm   '' = longitudinal array size'/  &
i5,4x,'''jdm   '' = latitudinal  array size'/  &
'field       time step  model day','  k  dens        min              max')

117  FORMAT (a8,' =',i11,f11.3,i3,f6.2,1p2e16.7)

END SUBROUTINE

SUBROUTINE writearchvdiff(archvFile,nx,ny,nz,time,thbase,thflag,sver,mlayer,mdepth,ssh,mld,uba,vba,uvl,vvl,thk,tem,sal)

        IMPLICIT NONE

! Argument Declarations

CHARACTER(len=*), INTENT(IN)           :: archvFile
INTEGER(i4),      INTENT(IN)           :: nx
INTEGER(i4),      INTENT(IN)           :: ny
INTEGER(i4),      INTENT(IN)           :: nz
REAL(r4)                               :: thbase
INTEGER(i4),      INTENT(IN)           :: sver
REAL(r4),         INTENT(INOUT)        :: mlayer(:)
REAL(r4),         INTENT(INOUT)        :: mdepth(:,:)
REAL(r4),         INTENT(INOUT)        :: ssh(:,:)
REAL(r4),         INTENT(INOUT)        :: mld(:,:)
REAL(r4),         INTENT(INOUT)        :: uba(:,:)
REAL(r4),         INTENT(INOUT)        :: vba(:,:)
REAL(r4),         INTENT(INOUT)        :: uvl(:,:,:)
REAL(r4),         INTENT(INOUT)        :: vvl(:,:,:)
REAL(r4),         INTENT(INOUT)        :: thk(:,:,:)
REAL(r4),         INTENT(INOUT)        :: tem(:,:,:)
REAL(r4),         INTENT(INOUT)        :: sal(:,:,:)

! Local Variables

INTEGER(i4)                            :: iunit,iyy,idd,imm,idoy,ihh
INTEGER(i4)                            :: k,ii,jj,kk
REAL(r4)                               :: hmin
REAL(r4)                               :: hmax
REAL(r4)                               :: uzero
REAL(r4)                               :: vzero
!REAL(r4)                               :: dummy(nx,ny)
!REAL(r4)                               :: u_btrop(nx,ny)
!REAL(r4)                               :: v_btrop(nx,ny)
CHARACTER(len=80), DIMENSION(5)        :: preambl
CHARACTER(len=80)                      :: cline
character(len=80)                      :: ctitle(4)
integer(i4)                            :: yrflag,iversn,sigver,iexpt,nstep,thflag
real(r4)                               :: time,coord



! Executable Section


iunit=11
nstep=int(time*240)
ctitle(1)="TSIS Analysis for incremental update"
ctitle(2)="WOA18 Jan. init; KPP; SeaWiFS KPAR; e-loan ice; A=20; Smag=.05; veldf4=.02;"
ctitle(3)="S-Z(14-18):dp00/f/x/i=3m/1.18/120m/1m;ds=0.5m/1.18/75m; var den: Med+Black Seas;"
ctitle(4)="Dummy Info in the above two lines"
iversn=00
iexpt=000
yrflag=3

call zaiost(nx,ny)
call zaiopn(trim(archvFile),iunit)
write(iunit,116) ctitle,iversn,iexpt,iexpt,yrflag,nx,ny

call zaiowr(iunit,ssh,hmin,hmax)
WRITE (iunit,117) 'montg1  ',nstep,time,sver,thbase,hmin,hmax

call zaiowr(iunit,ssh,hmin,hmax)
WRITE (iunit,117) 'srfhgt  ',nstep,time,0,0,hmin,hmax

!call zaiowr(iunit,dummy,hmin,hmax)
!WRITE (iunit,117) 'steric  ',nstep,time,0,0,hmin,hmax

call zaiowr(iunit,ssh,hmin,hmax)
WRITE (iunit,117) 'surflx  ',nstep,time,0,0,hmin,hmax

call zaiowr(iunit,ssh,hmin,hmax)
WRITE (iunit,117) 'salflx  ',nstep,time,0,0,hmin,hmax

call zaiowr(iunit,ssh,hmin,hmax)
WRITE (iunit,117) 'bl_dpth  ',nstep,time,0,0,hmin,hmax

call zaiowr(iunit,ssh,hmin,hmax)
WRITE (iunit,117) 'mix_dpth  ',nstep,time,0,0,hmin,hmax

call zaiowr(iunit,uba,hmin,hmax)
WRITE (iunit,117) 'u_btrop  ',nstep,time,0,0,hmin,hmax

call zaiowr(iunit,vba,hmin,hmax)
WRITE (iunit,117) 'v_btrop  ',nstep,time,0,0,hmin,hmax

!     layer loop
DO k=1,nz

coord=mlayer(k)

CALL zaiowr(iunit,uvl(:,:,k),hmin,hmax)
WRITE (iunit,117) 'u-vel.  ',nstep,time,k,coord,hmin,hmax

CALL zaiowr(iunit,vvl(:,:,k),hmin,hmax)
WRITE (iunit,117) 'v-vel.  ',nstep,time,k,coord,hmin,hmax

CALL zaiowr(iunit,thk(:,:,k),hmin,hmax)
WRITE (iunit,117) 'thknss   ',nstep,time,k,coord,hmin,hmax

CALL zaiowr(iunit,tem(:,:,k),hmin,hmax)
WRITE (iunit,117) 'temp    ',nstep,time,k,coord,hmin,hmax

CALL zaiowr(iunit,sal(:,:,k),hmin,hmax)
WRITE (iunit,117) 'salin   ',nstep,time,k,coord,hmin,hmax

ENDDO

call zaiocl(iunit)

116  format (a80/a80/a80/a80/ &
i5,4x,'''iversn'' = hycom version number x10'/ &
i5,4x,'''jexpt '' = experiment number x10'/    &
i5,4x,'''iexpt '' = experiment number x10'/    &
i5,4x,'''yrflag'' = days in year flag'/        &
i5,4x,'''idm   '' = longitudinal array size'/  &
i5,4x,'''jdm   '' = latitudinal  array size'/  &
'field       time step  diff day','  k  dens        min              max')

117  FORMAT (a8,' =',i11,f11.3,i3,f6.2,1p2e16.7)

END SUBROUTINE

SUBROUTINE writearchv(archvFile,nx,ny,nz,time,thbase,thflag,sver,mlayer,mdepth,ssh,mld,uba,vba,uvl,vvl,thk,tem,sal)

        IMPLICIT NONE

! Argument Declarations

CHARACTER(len=*), INTENT(IN)           :: archvFile
INTEGER(i4),      INTENT(IN)           :: nx
INTEGER(i4),      INTENT(IN)           :: ny
INTEGER(i4),      INTENT(IN)           :: nz
REAL(r4)                               :: thbase
INTEGER(i4),      INTENT(IN)           :: sver
REAL(r4),         INTENT(INOUT)        :: mlayer(:)
REAL(r4),         INTENT(INOUT)        :: mdepth(:,:)
REAL(r4),         INTENT(INOUT)        :: ssh(:,:)
REAL(r4),         INTENT(INOUT)        :: mld(:,:)
REAL(r4),         INTENT(INOUT)        :: uba(:,:)
REAL(r4),         INTENT(INOUT)        :: vba(:,:)
REAL(r4),         INTENT(INOUT)        :: uvl(:,:,:)
REAL(r4),         INTENT(INOUT)        :: vvl(:,:,:)
REAL(r4),         INTENT(INOUT)        :: thk(:,:,:)
REAL(r4),         INTENT(INOUT)        :: tem(:,:,:)
REAL(r4),         INTENT(INOUT)        :: sal(:,:,:)

! Local Variables

INTEGER(i4)                            :: iunit,iyy,idd,imm,idoy,ihh
INTEGER(i4)                            :: k,ii,jj,kk
REAL(r4)                               :: hmin
REAL(r4)                               :: hmax
REAL(r4)                               :: uzero
REAL(r4)                               :: vzero
!REAL(r4)                               :: dummy(nx,ny)
!REAL(r4)                               :: u_btrop(nx,ny)
!REAL(r4)                               :: v_btrop(nx,ny)
CHARACTER(len=80), DIMENSION(5)        :: preambl
CHARACTER(len=80)                      :: cline
character(len=80)                      :: ctitle(4)
integer(i4)                            :: yrflag,iversn,sigver,iexpt,nstep,thflag
real(r4)                               :: time,coord



! Executable Section

iunit=11
nstep=int(time*240)
ctitle(1)="TSIS Analysis for incremental update"
ctitle(2)="WOA18 Jan. init; KPP; SeaWiFS KPAR; e-loan ice; A=20; Smag=.05; veldf4=.02;"
ctitle(3)="S-Z(14-18):dp00/f/x/i=3m/1.18/120m/1m;ds=0.5m/1.18/75m; var den: Med+Black Seas;"
ctitle(4)="Dummy Info in the above two lines"
iversn=00
iexpt=000
yrflag=3

call zaiost(nx,ny)
call zaiopn(trim(archvFile),iunit)
write(iunit,116) ctitle,iversn,iexpt,yrflag,nx,ny

call zaiowr(iunit,ssh,hmin,hmax)
WRITE (iunit,117) 'montg1  ',nstep,time,sver,thbase,hmin,hmax

call zaiowr(iunit,ssh,hmin,hmax)
WRITE (iunit,117) 'srfhgt  ',nstep,time,0,0,hmin,hmax

!call zaiowr(iunit,dummy,hmin,hmax)
!WRITE (iunit,117) 'steric  ',nstep,time,0,0,hmin,hmax

call zaiowr(iunit,ssh,hmin,hmax)
WRITE (iunit,117) 'surflx  ',nstep,time,0,0,hmin,hmax

call zaiowr(iunit,ssh,hmin,hmax)
WRITE (iunit,117) 'salflx  ',nstep,time,0,0,hmin,hmax

call zaiowr(iunit,ssh,hmin,hmax)
WRITE (iunit,117) 'bl_dpth  ',nstep,time,0,0,hmin,hmax

call zaiowr(iunit,ssh,hmin,hmax)
WRITE (iunit,117) 'mix_dpth  ',nstep,time,0,0,hmin,hmax

call zaiowr(iunit,uba,hmin,hmax)
WRITE (iunit,117) 'u_btrop  ',nstep,time,0,0,hmin,hmax

call zaiowr(iunit,vba,hmin,hmax)
WRITE (iunit,117) 'v_btrop  ',nstep,time,0,0,hmin,hmax

!     layer loop

DO k=1,nz

coord=mlayer(k)

CALL zaiowr(iunit,uvl(:,:,k),hmin,hmax)
WRITE (iunit,117) 'u-vel.  ',nstep,time,k,coord,hmin,hmax

CALL zaiowr(iunit,vvl(:,:,k),hmin,hmax)
WRITE (iunit,117) 'v-vel.  ',nstep,time,k,coord,hmin,hmax

CALL zaiowr(iunit,thk(:,:,k),hmin,hmax)
WRITE (iunit,117) 'thknss   ',nstep,time,k,coord,hmin,hmax

CALL zaiowr(iunit,tem(:,:,k),hmin,hmax)
WRITE (iunit,117) 'temp    ',nstep,time,k,coord,hmin,hmax

CALL zaiowr(iunit,sal(:,:,k),hmin,hmax)
WRITE (iunit,117) 'salin   ',nstep,time,k,coord,hmin,hmax

ENDDO

call zaiocl(iunit)

116  format (a80/a80/a80/a80/ &
i5,4x,'''iversn'' = hycom version number x10'/ &
i5,4x,'''iexpt '' = experiment number x10'/    &
i5,4x,'''yrflag'' = days in year flag'/        &
i5,4x,'''idm   '' = longitudinal array size'/  &
i5,4x,'''jdm   '' = latitudinal  array size'/  &
'field       time step  model day','  k  dens        min              max')

117  FORMAT (a8,' =',i11,f11.3,i3,f6.2,1p2e16.7)

END SUBROUTINE

SUBROUTINE writeHYCOMXI(FileName,nx,ny,nl,mlon,mlat,mlayer,mdepth,fssh,fuvl,fvvl,fthk,ftem,fsal)
use mkinds
use mncio
use mseawater

implicit none
! Arguments
character(*)                   :: FileName
integer(i4)                    :: nx
integer(i4)                    :: ny
integer(i4)                    :: nl
real(r4)                       :: mlon(nx,ny)
real(r4)                       :: mlat(nx,ny)
real(r4)                       :: mlayer(nl)
real(r4)                       :: mdepth(nx,ny)
real(r4)                       :: fssh(nx,ny)
real(r4)                       :: fuvl(nx,ny,nl)
real(r4)                       :: fvvl(nx,ny,nl)
real(r4)                       :: fthk(nx,ny,nl)
real(r4)                       :: ftem(nx,ny,nl)
real(r4)                       :: fsal(nx,ny,nl)
! local variables
integer(i4)                    :: i
integer(i4)                    :: j
integer(i4)                    :: k
integer(i4)                    :: n
integer(i4)                    :: xc(nx)
integer(i4)                    :: yc(ny)

do i=1,nx
  xc(i)=i
enddo

do j=1,ny
  yc(j)=j
enddo

!### write output to netcdf file XF.nc
!### create the file
call nciocf(FileName,fid)
!create dimension and coordinate variable 1
vname="xc"
vtype=NF90_Float
vdim1d(1)=1
call nciodd(trim(FileName),fid,vname,nx)
call nciocv(trim(FileName),fid,vname,vtype,vdim1d)
call nciowa(trim(FileName),fid,vname,'long_name','xc')
!create dimension and coordinate variable 2
vname="yc"
vtype=NF90_Float
vdim1d(1)=2
call nciodd(trim(FileName),fid,vname,ny)
call nciocv(trim(FileName),fid,vname,vtype,vdim1d)
call nciowa(trim(FileName),fid,vname,'long_name','yc')
!create dimension and coordinate variable 3
vname="layer"
vtype=NF90_Float
vdim1d(1)=3
call nciodd(trim(FileName),fid,vname,nl)
call nciocv(trim(FileName),fid,vname,vtype,vdim1d)
call nciowa(trim(FileName),fid,vname,'long_name','hybrid_layers')
call nciowa(trim(FileName),fid,vname,'standard_name','hybrid_layers')
call nciowa(trim(FileName),fid,vname,'units','sigma')

!create data variable 1
vname="ssh"
vtype=NF90_Float
vdim2d=(/1,2/)
call nciocv(trim(FileName),fid,vname,vtype,vdim2d)
call nciowa(trim(FileName),fid,vname,'long_name','sea_surface_height')
call nciowa(trim(FileName),fid,vname,'standard_name','ocean_surface_elevation')
call nciowa(trim(FileName),fid,vname,'units','m')
call nciowa(trim(FileName),fid,vname,'_FillValue',2.00**100)
!create data variable 5
vname="uvl"
vtype=NF90_Float
vdim3d=(/1,2,3/)
call nciocv(trim(FileName),fid,vname,vtype,vdim3d)
call nciowa(trim(FileName),fid,vname,'long_name','eastward_water_velocity')
call nciowa(trim(FileName),fid,vname,'standard_name','eastward_sea_water_velocity')
call nciowa(trim(FileName),fid,vname,'units','m/s')
call nciowa(trim(FileName),fid,vname,'_FillValue',2.00**100)
!create data variable 6
!create data variable 6
vname="vvl"
vtype=NF90_Float
vdim3d=(/1,2,3/)
call nciocv(trim(FileName),fid,vname,vtype,vdim3d)
call nciowa(trim(FileName),fid,vname,'long_name','northward_water_velocity')
call nciowa(trim(FileName),fid,vname,'standard_name','northward_sea_water_velocity')
call nciowa(trim(FileName),fid,vname,'units','m/s')
call nciowa(trim(FileName),fid,vname,'_FillValue',2.00**100)
!create data variable 7
vname="tem"
vtype=NF90_Float
vdim3d=(/1,2,3/)
call nciocv(trim(FileName),fid,vname,vtype,vdim3d)
call nciowa(trim(FileName),fid,vname,'long_name','sea_water_potential_temperature')
call nciowa(trim(FileName),fid,vname,'standard_name','sea_water_potential_temperature')
call nciowa(trim(FileName),fid,vname,'units','degC')
call nciowa(trim(FileName),fid,vname,'_FillValue',2.00**100)
!create data variable 8
vname="sal"
vtype=NF90_Float
vdim3d=(/1,2,3/)
call nciocv(trim(FileName),fid,vname,vtype,vdim3d)
call nciowa(trim(FileName),fid,vname,'long_name','sea_water_salinity')
call nciowa(trim(FileName),fid,vname,'standard_name','sea_water_salinity')
call nciowa(trim(FileName),fid,vname,'units','pss')
call nciowa(trim(FileName),fid,vname,'_FillValue',2.00**100)
!create data variable 10
vname="thk"
vtype=NF90_Float
vdim3d=(/1,2,3/)
call nciocv(trim(FileName),fid,vname,vtype,vdim3d)
call nciowa(trim(FileName),fid,vname,'long_name','hybrid_layer_thickness')
call nciowa(trim(FileName),fid,vname,'standard_name','hybrid_layer_thickness')
call nciowa(trim(FileName),fid,vname,'units','m')
call nciowa(trim(FileName),fid,vname,'_FillValue',2.00**100)
call ncioed(trim(FileName),fid)
!create data variable 10
!vname="msk3d"
!vtype=NF90_int
!vdim3d=(/1,2,3/)
!call nciocv(trim(FileName),fid,vname,vtype,vdim3d)
!call nciowa(trim(FileName),fid,vname,'long_name','land sea mask')
!call nciowa(trim(FileName),fid,vname,'_FillValue',2.00**100)
!call ncioed(trim(FileName),fid)




!### write forecast data into variables
call nciowv(trim(FileName),fid,"xc",xc)
call nciowv(trim(FileName),fid,"yc",yc)
call nciowv(trim(FileName),fid,"layer",mlayer)
call nciowv(trim(FileName),fid,"ssh",fssh)
call nciowv(trim(FileName),fid,"uvl",fuvl)
call nciowv(trim(FileName),fid,"vvl",fvvl)
call nciowv(trim(FileName),fid,"tem",ftem)
call nciowv(trim(FileName),fid,"sal",fsal)
call nciowv(trim(FileName),fid,"thk",fthk)
!call nciowv(trim(FileName),fid,"msk3d",msk3d)
!###close file
call nciocl(trim(FileName),fid)
end subroutine



SUBROUTINE writeHYCOMXF(FileName,nx,ny,nz,msk3d,mlon,mlat,mlayer,mdepth,fssh,&
                        fmld,fuba,fvba,fuvl,fvvl,fthk,ftem,fsal,fden)
use mkinds
use mncio
use mseawater

implicit none
! Arguments
character(*)                   :: FileName
integer(i4)                    :: nx
integer(i4)                    :: ny
integer(i4)                    :: nz
integer(i4)                    :: msk3d(nx,ny,nz)
real(r4)                       :: mlon(nx,ny)
real(r4)                       :: mlat(nx,ny)
real(r4)                       :: mlayer(nz)
real(r4)                       :: mdepth(nx,ny)
real(r4)                       :: fssh(nx,ny)
real(r4)                       :: fmld(nx,ny)
real(r4)                       :: fuba(nx,ny)
real(r4)                       :: fvba(nx,ny)
real(r4)                       :: fuvl(nx,ny,nz)
real(r4)                       :: fvvl(nx,ny,nz)
real(r4)                       :: fthk(nx,ny,nz)
real(r4)                       :: ftem(nx,ny,nz)
real(r4)                       :: fsal(nx,ny,nz)
real(r4)                       :: fden(nx,ny,nz)
! local variables

real(r4)                       :: hmin
real(r4)                       :: hmax
real(r4),parameter             :: spval=2.00**100
real(r4),parameter             :: hspval=0.5*2.00**100
real(r4),parameter             :: onem=9806.0
real(r4),parameter             :: eps=0.5*9806.0
integer(i4)                    :: i
integer(i4)                    :: j
integer(i4)                    :: k
integer(i4)                    :: n
integer(i4)                    :: xc(nx)
integer(i4)                    :: yc(ny)


do i=1,nx
  xc(i)=i
enddo

do j=1,ny
  yc(j)=j
enddo

!### write output to netcdf file XF.nc
!### create the file
call nciocf(FileName,fid)
!create dimension and coordinate variable 1
vname="xc"
vtype=NF90_Float
vdim1d(1)=1
call nciodd(trim(FileName),fid,vname,nx)
call nciocv(trim(FileName),fid,vname,vtype,vdim1d)
call nciowa(trim(FileName),fid,vname,'long_name','xc')
!create dimension and coordinate variable 2
vname="yc"
vtype=NF90_Float
vdim1d(1)=2
call nciodd(trim(FileName),fid,vname,ny)
call nciocv(trim(FileName),fid,vname,vtype,vdim1d)
call nciowa(trim(FileName),fid,vname,'long_name','yc')
!create dimension and coordinate variable 3
vname="layer"
vtype=NF90_Float
vdim1d(1)=3
call nciodd(trim(FileName),fid,vname,nz)
call nciocv(trim(FileName),fid,vname,vtype,vdim1d)
call nciowa(trim(FileName),fid,vname,'long_name','hybrid_layers')
call nciowa(trim(FileName),fid,vname,'standard_name','hybrid_layers')
call nciowa(trim(FileName),fid,vname,'units','sigma')

!vname="intf"
!vtype=NF90_Float
!vdim1d(1)=4
!call nciodd(trim(FileName),fid,vname,nz+1)
!call nciocv(trim(FileName),fid,vname,vtype,vdim1d)
!call nciowa(trim(FileName),fid,vname,'long_name','hybrid_layers_interfaces')
!call nciowa(trim(FileName),fid,vname,'standard_name','hybrid_layers_interfaces')
!call nciowa(trim(FileName),fid,vname,'units','')

!create dimension and coordinate variable 4
!vname="tstate"
!vtype=NF90_Float
!v1dim(1)=4
!call nciodd(trim(FileName),fid,vname,2)
!call nciocv(trim(FileName),fid,vname,vtype,v1dim)
!call nciowa(trim(FileName),fid,vname,'long_name','thermobaric state')
!call nciowa(trim(FileName),fid,vname,'standard_name','thermobaric state')
!call nciowa(trim(FileName),fid,vname,'units','')

!create data variable 1
vname="ssh"
vtype=NF90_Float
vdim2d=(/1,2/)
call nciocv(trim(FileName),fid,vname,vtype,vdim2d)
call nciowa(trim(FileName),fid,vname,'long_name','sea_surface_height')
call nciowa(trim(FileName),fid,vname,'standard_name','ocean_surface_elevation')
call nciowa(trim(FileName),fid,vname,'units','m')
call nciowa(trim(FileName),fid,vname,'_FillValue',2.00**100)
!create data variable 2
vname="uba"
vtype=NF90_Float
vdim2d=(/1,2/)
call nciocv(trim(FileName),fid,vname,vtype,vdim2d)
call nciowa(trim(FileName),fid,vname,'long_name','barotrophic_eastward_velocity')
call nciowa(trim(FileName),fid,vname,'standard_name','barotrophic_eastward_velocity')
call nciowa(trim(FileName),fid,vname,'units','m/s')
call nciowa(trim(FileName),fid,vname,'_FillValue',2.00**100)
!create data variable 3
vname="vba"
vtype=NF90_Float
vdim2d=(/1,2/)
call nciocv(trim(FileName),fid,vname,vtype,vdim2d)
call nciowa(trim(FileName),fid,vname,'long_name','barotrophic_northward_velocity')
call nciowa(trim(FileName),fid,vname,'standard_name','barotrophic_northward_velocity')
call nciowa(trim(FileName),fid,vname,'units','m/s')
call nciowa(trim(FileName),fid,vname,'_FillValue',2.00**100)
!create data variable 4
vname="mld"
vtype=NF90_Float
vdim2d=(/1,2/)
call nciocv(trim(FileName),fid,vname,vtype,vdim2d)
call nciowa(trim(FileName),fid,vname,'long_name','ocean_mixed_layer_depth')
call nciowa(trim(FileName),fid,vname,'standard_name','ocean_mixed_layer_depth')
call nciowa(trim(FileName),fid,vname,'units','m')
call nciowa(trim(FileName),fid,vname,'_FillValue',2.00**100)
!create data variable 5
vname="uvl"
vtype=NF90_Float
vdim3d=(/1,2,3/)
call nciocv(trim(FileName),fid,vname,vtype,vdim3d)
call nciowa(trim(FileName),fid,vname,'long_name','eastward_water_velocity')
call nciowa(trim(FileName),fid,vname,'standard_name','eastward_sea_water_velocity')
call nciowa(trim(FileName),fid,vname,'units','m/s')
call nciowa(trim(FileName),fid,vname,'_FillValue',2.00**100)
!create data variable 6
vname="vvl"
vtype=NF90_Float
vdim3d=(/1,2,3/)
call nciocv(trim(FileName),fid,vname,vtype,vdim3d)
call nciowa(trim(FileName),fid,vname,'long_name','northward_water_velocity')
call nciowa(trim(FileName),fid,vname,'standard_name','northward_sea_water_velocity')
call nciowa(trim(FileName),fid,vname,'units','m/s')
call nciowa(trim(FileName),fid,vname,'_FillValue',2.00**100)
!create data variable 7
vname="tem"
vtype=NF90_Float
vdim3d=(/1,2,3/)
call nciocv(trim(FileName),fid,vname,vtype,vdim3d)
call nciowa(trim(FileName),fid,vname,'long_name','sea_water_potential_temperature')
call nciowa(trim(FileName),fid,vname,'standard_name','sea_water_potential_temperature')
call nciowa(trim(FileName),fid,vname,'units','degC')
call nciowa(trim(FileName),fid,vname,'_FillValue',2.00**100)
!create data variable 8
vname="sal"
vtype=NF90_Float
vdim3d=(/1,2,3/)
call nciocv(trim(FileName),fid,vname,vtype,vdim3d)
call nciowa(trim(FileName),fid,vname,'long_name','sea_water_salinity')
call nciowa(trim(FileName),fid,vname,'standard_name','sea_water_salinity')
call nciowa(trim(FileName),fid,vname,'units','pss')
call nciowa(trim(FileName),fid,vname,'_FillValue',2.00**100)

!create data variable 9
vname="den"
vtype=NF90_Float
vdim3d=(/1,2,3/)
call nciocv(trim(FileName),fid,vname,vtype,vdim3d)
call nciowa(trim(FileName),fid,vname,'long_name','sea_water_potential_density')
call nciowa(trim(FileName),fid,vname,'standard_name','seawater_potential_density')
call nciowa(trim(FileName),fid,vname,'units','kg/m3')
call nciowa(trim(FileName),fid,vname,'_FillValue',2.00**100)

!create data variable 10
vname="thk"
vtype=NF90_Float
vdim3d=(/1,2,3/)
call nciocv(trim(FileName),fid,vname,vtype,vdim3d)
call nciowa(trim(FileName),fid,vname,'long_name','hybrid_layer_thickness')
call nciowa(trim(FileName),fid,vname,'standard_name','hybrid_layer_thickness')
call nciowa(trim(FileName),fid,vname,'units','m')
call nciowa(trim(FileName),fid,vname,'_FillValue',2.00**100)

!vname="mgp"
!vtype=NF90_Float
!v3dim=(/1,2,3/)
!call nciocv(trim(FileName),fid,vname,vtype,v3dim)
!call nciowa(trim(FileName),fid,vname,'long_name','Montgomery Potential')
!call nciowa(trim(FileName),fid,vname,'standard_name','Montgomery Potential')
!call nciowa(trim(FileName),fid,vname,'units','')
!call nciowa(trim(FileName),fid,vname,'_FillValue',2.00**100)

!create data variable 12
!vname="thkk"
!vtype=NF90_Float
!v3dim=(/1,2,4/)
!call nciocv(trim(FileName),fid,vname,vtype,v3dim)
!call nciowa(trim(FileName),fid,vname,'long_name','thkk')
!call nciowa(trim(FileName),fid,vname,'standard_name','thkk')
!call nciowa(trim(FileName),fid,vname,'units','m')
!call nciowa(trim(FileName),fid,vname,'_FillValue',2.00**100)

!create data variable 13
!vname="psikk"
!vtype=NF90_Float
!v3dim=(/1,2,4/)
!call nciocv(trim(FileName),fid,vname,vtype,v3dim)
!call nciowa(trim(FileName),fid,vname,'long_name','psikk')
!call nciowa(trim(FileName),fid,vname,'standard_name','psikk')
!call nciowa(trim(FileName),fid,vname,'units','m')
!call nciowa(trim(FileName),fid,vname,'_FillValue',2.00**100)

!create data variable 14
!vname="pba"
!vtype=NF90_Float
!v2dim=(/1,2/)
!call nciocv(trim(FileName),fid,vname,vtype,v2dim)
!call nciowa(trim(FileName),fid,vname,'long_name','barotropic pressure')
!call nciowa(trim(FileName),fid,vname,'standard_name','barotropic pressure')
!call nciowa(trim(FileName),fid,vname,'units','m')
!call nciowa(trim(FileName),fid,vname,'_FillValue',2.00**100)
!### end definitions
!create data variable 10
vname="msk3d"
vtype=NF90_int
vdim3d=(/1,2,3/)
call nciocv(trim(FileName),fid,vname,vtype,vdim3d)
call nciowa(trim(FileName),fid,vname,'long_name','land sea mask')
call ncioed(trim(FileName),fid)




!### write forecast data into variables
call nciowv(trim(FileName),fid,"xc",xc)
call nciowv(trim(FileName),fid,"yc",yc)
call nciowv(trim(FileName),fid,"layer",mlayer)
!call nciowv(trim(FileName),fid,"intf",intf)
!call nciowv(trim(FileName),fid,"tstate",(/1,2/))
call nciowv(trim(FileName),fid,"ssh",fssh)
call nciowv(trim(FileName),fid,"uba",fuba)
call nciowv(trim(FileName),fid,"vba",fvba)
call nciowv(trim(FileName),fid,"mld",fmld)
!call nciowv(trim(FileName),fid,"pba",fpba)
call nciowv(trim(FileName),fid,"uvl",fuvl)
call nciowv(trim(FileName),fid,"vvl",fvvl)
call nciowv(trim(FileName),fid,"tem",ftem)
call nciowv(trim(FileName),fid,"sal",fsal)
call nciowv(trim(FileName),fid,"den",fden)
call nciowv(trim(FileName),fid,"thk",fthk)
call nciowv(trim(FileName),fid,"msk3d",msk3d)
!call nciowv(trim(FileName),fid,"mgp",fmgp)
!call nciowv(trim(FileName),fid,"thkk",fthkk)
!call nciowv(trim(FileName),fid,"psikk",fpsikk)
!###close file
call nciocl(trim(FileName),fid)
end subroutine

SUBROUTINE writeHYCOMXFZ(FileName,nx,ny,nl,mlon,mlat,mlayer,mtime,mdepth,fssh,&
                        fmld,fuvl,fvvl,fthk,ftem,fsal)
use mkinds
use mncio
use mseawater

implicit none
! Arguments
character(*)                   :: FileName
integer(i4)                    :: nx
integer(i4)                    :: ny
integer(i4)                    :: nl
real(r4)                       :: mlon(nx,ny)
real(r4)                       :: mlat(nx,ny)
real(r4)                       :: mlayer(nl)
real(r8)                       :: mtime
real(r4)                       :: mdepth(nx,ny)
real(r4)                       :: fssh(nx,ny)
real(r4)                       :: fmld(nx,ny)
!real(r4)                       :: fuba(nx,ny)
!real(r4)                       :: fvba(nx,ny)
real(r4)                       :: fuvl(nx,ny,nl)
real(r4)                       :: fvvl(nx,ny,nl)
real(r4)                       :: fthk(nx,ny,nl)
real(r4)                       :: ftem(nx,ny,nl)
real(r4)                       :: fsal(nx,ny,nl)
!real(r4)                       :: fden(nx,ny,nl)
! local variables
integer(i4),parameter          :: nzl=50,nt=1
real(r4)                       :: hmin
real(r4)                       :: hmax
real(r4),parameter             :: spval=2.00**100
real(r4),parameter             :: hspval=0.5*2.00**100
real(r4),parameter             :: onem=9806.0
real(r4),parameter             :: qonem=1.0/onem
real(r4),parameter             :: eps=0.5*9806.0
integer(i4)                    :: i
integer(i4)                    :: j
integer(i4)                    :: k
integer(i4)                    :: n
integer(i4),allocatable        :: xc(:)
integer(i4),allocatable        :: yc(:)
real(r8),allocatable                       :: time(:)
real(r4),allocatable           ::  pin(:,:,:)
real(r4),allocatable           ::  ztem(:,:,:,:),zsal(:,:,:,:),zuvl(:,:,:,:),zvvl(:,:,:,:)
real(r4)                       ::  zlevs(nzl)

zlevs=[0., 5., 10., 15., 20., 25., 30., 35., 40., 45., 50., 55., 60., 65., 70., 80., 90., &
    100., 125., 150., 175., 200., 225., 250., 275., 300., 350., 400., 450., 500., 550.,&
    600., 700., 800., 900., 1000., 1100., 1200., 1300., 1400., 1500., 1750., 2000., 2500.,&
    3000., 3500., 4000., 4500., 5000., 5500.]


allocate(xc(nx))
allocate(yc(ny))
allocate(time(nt))

allocate(pin(nx,ny,nl+1))
allocate(ztem(nx,ny,nzl,1))
allocate(zsal(nx,ny,nzl,1))
allocate(zuvl(nx,ny,nzl,1))
allocate(zvvl(nx,ny,nzl,1))



do i=1,nx
  xc(i)=i
enddo

do j=1,ny
  yc(j)=j
enddo

time(1)=mtime
pin(:,:,1)=0.0
do k=1,nl
  pin(:,:,k+1)=pin(:,:,k)+fthk(:,:,k)*qonem
enddo
pin(:,:,nl+1)=mdepth

call layer2z(ftem,pin,ztem(:,:,:,1),zlevs,spval,nx,ny,nl,nzl,-1)
call layer2z(fsal,pin,zsal(:,:,:,1),zlevs,spval,nx,ny,nl,nzl,-1)
call layer2z(fuvl,pin,zuvl(:,:,:,1),zlevs,spval,nx,ny,nl,nzl,-1)
call layer2z(fvvl,pin,zvvl(:,:,:,1),zlevs,spval,nx,ny,nl,nzl,-1)


!### write output to netcdf file XF.nc
!### create the file
call nciocf(FileName,fid)
!create dimension and coordinate variable 1
vname="xc"
vtype=NF90_Float
vdim1d(1)=1
call nciodd(trim(FileName),fid,vname,nx)
call nciocv(trim(FileName),fid,vname,vtype,vdim1d)
call nciowa(trim(FileName),fid,vname,'long_name','xc')
!create dimension and coordinate variable 2
vname="yc"
vtype=NF90_Float
vdim1d(1)=2
call nciodd(trim(FileName),fid,vname,ny)
call nciocv(trim(FileName),fid,vname,vtype,vdim1d)
call nciowa(trim(FileName),fid,vname,'long_name','yc')
!create dimension and coordinate variable 3
vname="depth"
vtype=NF90_Float
vdim1d(1)=3
call nciodd(trim(FileName),fid,vname,nzl)
call nciocv(trim(FileName),fid,vname,vtype,vdim1d)
call nciowa(trim(FileName),fid,vname,'long_name','depth')
call nciowa(trim(FileName),fid,vname,'standard_name','depth')
call nciowa(trim(FileName),fid,vname,'units','m')

!create dimension and coordinate variable 4
vname="time"
vtype=NF90_double
vdim1d(1)=4
call nciodd(trim(FileName),fid,vname,nt)
call nciocv(trim(FileName),fid,vname,vtype,vdim1d)
call nciowa(trim(FileName),fid,vname,'long_name','time')
call nciowa(trim(FileName),fid,vname,'standard_name','time')
call nciowa(trim(FileName),fid,vname,'units','days since 1900-12-31 00:00:00')

!create data variable 1
vname="ssh"
vtype=NF90_Float
vdim3d=(/1,2,4/)
call nciocv(trim(FileName),fid,vname,vtype,vdim3d)
call nciowa(trim(FileName),fid,vname,'long_name','sea_surface_height')
call nciowa(trim(FileName),fid,vname,'standard_name','ocean_surface_elevation')
call nciowa(trim(FileName),fid,vname,'units','m')
call nciowa(trim(FileName),fid,vname,'_FillValue',2.00**100)

!create data variable 4
vname="mld"
vtype=NF90_Float
vdim3d=(/1,2,4/)
call nciocv(trim(FileName),fid,vname,vtype,vdim3d)
call nciowa(trim(FileName),fid,vname,'long_name','ocean_mixed_layer_depth')
call nciowa(trim(FileName),fid,vname,'standard_name','ocean_mixed_layer_depth')
call nciowa(trim(FileName),fid,vname,'units','m')
call nciowa(trim(FileName),fid,vname,'_FillValue',2.00**100)
!create data variable 5
vname="uvl"
vtype=NF90_Float
vdim4d=(/1,2,3,4/)
call nciocv(trim(FileName),fid,vname,vtype,vdim4d)
call nciowa(trim(FileName),fid,vname,'long_name','eastward_water_velocity')
call nciowa(trim(FileName),fid,vname,'standard_name','eastward_sea_water_velocity')
call nciowa(trim(FileName),fid,vname,'units','m/s')
call nciowa(trim(FileName),fid,vname,'_FillValue',2.00**100)
!create data variable 6
vname="vvl"
vtype=NF90_Float
vdim4d=(/1,2,3,4/)
call nciocv(trim(FileName),fid,vname,vtype,vdim4d)
call nciowa(trim(FileName),fid,vname,'long_name','northward_water_velocity')
call nciowa(trim(FileName),fid,vname,'standard_name','northward_sea_water_velocity')
call nciowa(trim(FileName),fid,vname,'units','m/s')
call nciowa(trim(FileName),fid,vname,'_FillValue',2.00**100)
!create data variable 7
vname="tem"
vtype=NF90_Float
vdim4d=(/1,2,3,4/)
call nciocv(trim(FileName),fid,vname,vtype,vdim4d)
call nciowa(trim(FileName),fid,vname,'long_name','sea_water_potential_temperature')
call nciowa(trim(FileName),fid,vname,'standard_name','sea_water_potential_temperature')
call nciowa(trim(FileName),fid,vname,'units','degC')
call nciowa(trim(FileName),fid,vname,'_FillValue',2.00**100)
!create data variable 8
vname="sal"
vtype=NF90_Float
vdim4d=(/1,2,3,4/)
call nciocv(trim(FileName),fid,vname,vtype,vdim4d)
call nciowa(trim(FileName),fid,vname,'long_name','sea_water_salinity')
call nciowa(trim(FileName),fid,vname,'standard_name','sea_water_salinity')
call nciowa(trim(FileName),fid,vname,'units','pss')
call nciowa(trim(FileName),fid,vname,'_FillValue',2.00**100)

!create data variable 9
!vname="den"
!vtype=NF90_Float
!vdim3d=(/1,2,3/)
!call nciocv(trim(FileName),fid,vname,vtype,vdim3d)
!call nciowa(trim(FileName),fid,vname,'long_name','sea_water_potential_density')
!call nciowa(trim(FileName),fid,vname,'standard_name','seawater_potential_density')
!call nciowa(trim(FileName),fid,vname,'units','kg/m3')
!call nciowa(trim(FileName),fid,vname,'_FillValue',2.00**100)

!create data variable 10
!vname="thk"
!vtype=NF90_Float
!vdim3d=(/1,2,3/)
!call nciocv(trim(FileName),fid,vname,vtype,vdim3d)
!call nciowa(trim(FileName),fid,vname,'long_name','hybrid_layer_thickness')
!call nciowa(trim(FileName),fid,vname,'standard_name','hybrid_layer_thickness')
!call nciowa(trim(FileName),fid,vname,'units','m')
!call nciowa(trim(FileName),fid,vname,'_FillValue',2.00**100

call ncioed(trim(FileName),fid)


!### write forecast data into variables
call nciowv(trim(FileName),fid,"xc",xc)
call nciowv(trim(FileName),fid,"yc",yc)
call nciowv(trim(FileName),fid,"depth",zlevs)
call nciowv(trim(FileName),fid,"time",time)
call nciowv(trim(FileName),fid,"ssh",fssh)
call nciowv(trim(FileName),fid,"mld",fmld)
call nciowv(trim(FileName),fid,"uvl",zuvl)
call nciowv(trim(FileName),fid,"vvl",zvvl)
call nciowv(trim(FileName),fid,"tem",ztem)
call nciowv(trim(FileName),fid,"sal",zsal)
!call nciowv(trim(FileName),fid,"den",fden)
!call nciowv(trim(FileName),fid,"thk",fthk)
!call nciowv(trim(FileName),fid,"mgp",fmgp)
!call nciowv(trim(FileName),fid,"thkk",fthkk)
!call nciowv(trim(FileName),fid,"psikk",fpsikk)
!###close file
call nciocl(trim(FileName),fid)
end subroutine

SUBROUTINE intFromBlkdat(vname,val)

IMPLICIT NONE

! Argument Declarations

CHARACTER(LEN=*), INTENT(IN)      :: vname
INTEGER           ,  INTENT(OUT)   :: val

! Local Variables

CHARACTER(LEN=80)                 :: buffer
INTEGER, PARAMETER                :: IACHAR_SPACE = 32
INTEGER                           :: i
INTEGER                           :: indx
INTEGER                           :: IACHAR_Character

OPEN(UNIT=20,FILE="blkdat.input",FORM="formatted",STATUS="old",ACCESS="stream")
rewind(20)
Outer :DO WHILE (.true.)
   READ (20, '(A)', end=99) buffer
   buffer=ADJUSTL(buffer)
   indx = index (buffer,vname)
   IF(indx .ne. 0) THEN
      DO i = 1, LEN(buffer)

      ! Convert the current character to its position in the ASCII collating sequence

      IACHAR_Character = IACHAR(buffer(i:i))

      ! Find the index of first whitespace ' ' 
      IF ( IACHAR_Character == IACHAR_SPACE) THEN
               READ(buffer(1:i),*) val
               EXIT outer
      END IF

      END DO
    END IF
ENDDO outer
99 continue
CLOSE(20)
END SUBROUTINE

SUBROUTINE realFromBlkdat(vname,val)

IMPLICIT NONE

! Argument Declarations

CHARACTER(LEN=*), INTENT(IN)      :: vname
REAL           ,  INTENT(OUT)     :: val

! Local Variables

CHARACTER(LEN=80)                 :: buffer
INTEGER, PARAMETER                :: IACHAR_SPACE = 32
INTEGER                           :: i
INTEGER                           :: indx
INTEGER                           :: IACHAR_Character

OPEN(UNIT=20,FILE="blkdat.input",FORM="formatted",STATUS="old",ACCESS="stream")

Outer :DO WHILE (.true.)
   READ (20, '(A)', end=99) buffer
   buffer=ADJUSTL(buffer)
   indx = index (buffer,vname)
   IF(indx .ne. 0) THEN
      DO i = 1, LEN(buffer)

      ! Convert the current character to its position in the ASCII collating sequence

      IACHAR_Character = IACHAR(buffer(i:i))

      ! Find the index of first whitespace ' ' 
      IF ( IACHAR_Character == IACHAR_SPACE) THEN
               READ(buffer(1:i),*) val
               EXIT outer
      END IF

      END DO
    END IF
ENDDO outer
99 continue
CLOSE(20)
END SUBROUTINE

SUBROUTINE readTargetDensities(nz,tsigma)
IMPLICIT NONE

! Argument Declarations

INTEGER(i4),          INTENT(IN)      :: nz
REAL(r4)           ,  INTENT(OUT)     :: tsigma(nz)

! Local Variables

CHARACTER(LEN=cl)                     :: buffer
INTEGER(i4), PARAMETER                :: IACHAR_SPACE = 32
INTEGER(i4)                           :: i,j
INTEGER(i4)                           :: indx
INTEGER(I4)                           :: IACHAR_Character

j=0
OPEN(UNIT=20,FILE="blkdat.input",FORM="formatted",STATUS="old",ACCESS="stream")
DO WHILE (.true.)
   READ (20, '(A)', end=99) buffer
   buffer=ADJUSTL(buffer)
   indx = index (buffer,"isopycnal target density")
   IF(indx .ne. 0) THEN
      j=j+1
      DO i = 1, LEN(buffer)

      ! Convert the current character to its position in the ASCII collating sequence

      IACHAR_Character = IACHAR(buffer(i:i))

      ! Find the index of first whitespace ' ' 
      IF ( IACHAR_Character == IACHAR_SPACE) THEN
               READ(buffer(1:i),*) tsigma(j)
      END IF

      END DO
    END IF
ENDDO
99 continue
close(20)
END SUBROUTINE

SUBROUTINE hybrid_min_thickness(nhybrd,nsigma,dp00,dp00x,dp00f,ds00,ds00x,ds00f,dp0k,ds0k)
IMPLICIT NONE

! Input hybrid grid related variables
INTEGER(i4)                   ::      nsigma,nhybrd
REAL(r4)                      ::      dp00,dp00x,dp00f,ds00,ds00x,ds00f
REAL(r4)                      ::      dp0kf,dpms,ds0kf,dsms,dpck(0:nhybrd)
REAL(r4)                      ::      dp0k(nhybrd),ds0k(nsigma)
REAL(r4)                      ::      sigscl(nhybrd),dsck(0:nhybrd),sigcum(nhybrd)
REAL(r4)                      ::      depth,dmin
REAL(r4)                      ::      sigma(nhybrd)
LOGICAL(bl)                   ::      isopyc
INTEGER(i4)                   ::      k

      IF (nsigma.le.1) THEN

        nsigma=1
        ds00  =dp00
        ds00x =dp00x
        ds00f =dp00f

      ENDIF

!      IF(isopyc) THEN
!        dp0k(1)=thkmin
!      ELSE
        dp0k(1)=dp00
!      ENDIF

        dpms=dp0k(1)
        dp0kf=1.0
        dpck(0)=0.0
        dpck(1)=dpms

      DO K=2,nhybrd
        dp0kf=dp0kf*dp00f
        IF(K.LE.nhybrd) THEN
          dp0k(K)=MIN(dp00*dp0kf,dp00x)
        ELSE
          dp0k(k)=0.0
        ENDIF
          dpms=dpms+dp0k(K)
          dpck(K)=dpms
        ENDDO

!     MINIMUM (SHALLOW) LAYER THICKNESSES.

!      IF (isopyc) THEN
!        ds0k(1)=thkmin
!      ELSE
        ds0k(1)=ds00
!      ENDIF

      dsms=ds0k(1)
      ds0kf=1.0
      dsck(0)=0.0
      dsck(1)=dsms
 DO k=2,nsigma
        ds0kf=ds0kf*ds00f
        ds0k(K)=MIN(ds00*ds0kf,ds00x)
        dsms=dsms+ds0k(K)
        dsck(k)=dsms
      ENDDO

!      DO k=1,nsigma
!        sigscl(K)=ds0k(k)/dsms
!        sigcum(K)=dsck(k)/dsms
!      ENDDO

!      DO k= nsigma+1,nl
!        ds0k(k)=dp0k(k)
!        sigscl(k)=0.0
!        sigcum(k)=1.0
!      ENDDO


END SUBROUTINE

SUBROUTINE read_hybrid_min_thickness(nz,ns,dp0k,ds0k)
IMPLICIT NONE

! Argument Declarations

INTEGER(i4),          INTENT(IN)      :: nz
INTEGER(i4),          INTENT(IN)      :: ns
REAL(r4)           ,  INTENT(OUT)     :: dp0k(nz)
REAL(r4)           ,  INTENT(OUT)     :: ds0k(ns)

! Local Variables

CHARACTER(LEN=cl)                     :: buffer
INTEGER(i4), PARAMETER                :: IACHAR_SPACE = 32
INTEGER(i4)                           :: i,j
INTEGER(i4)                           :: indx,mxindx
INTEGER(I4)                           :: IACHAR_Character
j=0
!try dp0k
OPEN(UNIT=20,FILE="blkdat.input",FORM="formatted",STATUS="old",ACCESS="stream")
DO WHILE (.true.)
   READ (20, '(A)', end=99) buffer
   buffer=ADJUSTL(buffer)
   indx = index (buffer,"dp0k")
   IF(indx .ne. 0) THEN
      mxindx=max(0,indx)
      j=j+1
      DO i = 1, LEN(buffer)

      ! Convert the current character to its position in the ASCII collating sequence

      IACHAR_Character = IACHAR(buffer(i:i))

      ! Find the index of first whitespace ' ' 
      IF ( IACHAR_Character == IACHAR_SPACE) THEN
               READ(buffer(1:i),*) dp0k(j)
      END IF

      END DO
    END IF
ENDDO
99 continue
CLOSE(20)

!try ds0k
j=0
OPEN(UNIT=20,FILE="blkdat.input",FORM="formatted",STATUS="old",ACCESS="stream")
DO WHILE (.true.)
   READ (20, '(A)', end=100) buffer
   buffer=ADJUSTL(buffer)
   indx = index (buffer,"ds0k")
   IF(indx .ne. 0) THEN
      mxindx=max(0,indx)
      j=j+1
      DO i = 1, LEN(buffer)

      ! Convert the current character to its position in the ASCII collating sequence

      IACHAR_Character = IACHAR(buffer(i:i))

      ! Find the index of first whitespace ' ' 
      IF ( IACHAR_Character == IACHAR_SPACE) THEN
               READ(buffer(1:i),*) ds0k(j)
      END IF

      END DO
    END IF
ENDDO
100 continue
CLOSE(20)

IF (mxindx == 0) then

Print *, "Calling HYBrid"
dp0k=0.0
ds0k=0.0
ENDIF

END SUBROUTINE














END MODULe mhycom
