module mmgmrf
USE OMP_LIB
use mkinds
use mconstants
use mncio

!  define parameters, variables, for the gmrf assimilation scheme.

! information pair:

      integer(i4),parameter              :: maxW=2  ! width of MRF neighborhood (rectangular).
      integer(i4)                        :: maxV    ! number of dynamically independent layers.
      real(r4),target,allocatable        :: infoMatrix(:,:,:,:,:,:,:)  ! info.matrix.
      real(r4),target,allocatable        :: infoVector(:,:,:,:)      ! info.vector
      real(r4),parameter                 :: vard = 0.04*0.04
      real(r4),parameter                 :: varb = 0.2*0.2
      real(r4),parameter                 :: weightD=(1.0/vard)
      real(r4),parameter                 :: weightB=(1.0/varb)
      real(r4),parameter                 :: sclD=1.0
      integer(i4)                        :: maxIteration
      real(r4)                           :: wthin
      
contains

! 

subroutine initInfoPairM(idm,jdm,ldm,kdm)
implicit none
integer(i4)                        :: idm,jdm,kdm,ldm
NAMELIST /gmrf/wthin,maxIteration

allocate(infoMatrix(idm,jdm,-maxW:maxW,-maxW:maxW,ldm,ldm,kdm))
      allocate(infoVector(idm,jdm,ldm,kdm))


        OPEN(21,FILE='tsis.nlist')
        READ(21,NML=gmrf)
        CLOSE(21)
      infovector=0.0
      infoMatrix=0.0
end subroutine

subroutine FreeInfoPairM()
implicit none

deallocate(infoMatrix)
deallocate(infoVector)

end subroutine

subroutine setInfoMatrixM(idm,jdm,ldm,kdm,mask)

      implicit none
      integer(i4)                        :: idm,jdm,kdm,ldm
      integer(i4)                        :: mask(idm,jdm,kdm)

      ! information pair:
      real,pointer                   ::  W(:,:,:,:,:,:,:)
      
      ! local variables:
      real(r4)                           :: r
      real(r4)                           :: c
      integer(r4)                        :: ia,ja

      ! typical local variables:

      integer(i4) ::  i,j,k,l,mw,nw,iw,jw,iww,jww  ! array/matrix indices.
      
      W=>infoMatrix


      infoMatrix=0.0
      c=1.0
      r=0.05
     do  k=1,kdm 
      do l=1,ldm
       do j=1,jdm
        do i=1,idm
         if(mask(i,j,k)==1) then
        W(i,j,0,0,l,l,k)=weightB*(20.005*wThin+4.005*(1.-wthin))
        W(i,j,1,0,l,l,k)=weightB*(-8.0*wThin-(1.-wthin))
        W(i,j,0,1,l,l,k)=weightB*(-8.0*wThin-(1.-wthin))
        W(i,j,0,-1,l,l,k)=weightB*(-8.0*wThin-(1.-wthin))
        W(i,j,-1,0,l,l,k)=weightB*(-8.0*wThin-(1.-wthin))
        W(i,j,1,1,l,l,k)=weightB*(2.*wThin)
        W(i,j,-1,1,l,l,k)=weightB*(2.*wThin)
        W(i,j,1,-1,l,l,k)=weightB*(2.*wThin)
        W(i,j,-1,-1,l,l,k)=weightB*(2.*wThin)
        W(i,j,2,0,l,l,k)=weightB*(1.0*wThin)
        W(i,j,0,2,l,l,k)=weightB*(1.0*wThin)
        W(i,j,0,-2,l,l,k)=weightB*(1.0*wThin)
        W(i,j,-2,0,l,l,k)=weightB*(1.0*wThin)
         endif

       enddo
      enddo
     enddo
    enddo
end subroutine


subroutine geothin(f,dy,dx,mask,idm,jdm,kdm)

        implicit none

integer(i4) :: idm,jdm,kdm
integer(i4)                        :: mask(idm,jdm,kdm)
real(r4) :: f(idm,jdm)
real(r4) :: dy(idm,jdm)
real(r4) :: dx(idm,jdm)

! information pair:
      real(r4),pointer ::  z(:,:,:,:)
      real(r4),pointer ::  W(:,:,:,:,:,:,:)

! local variables:
real(r4)   ::  r,ru,rv,ofmax,ds4,wGeo,wThin,wOne,wLid
real(r4),parameter   ::  pi=3.14159,gg=9.8

! typical local variables:
      integer(i4) ::  i,j,k,l,mw,nw,iw,jw,iww,jww  ! array/matrix indices.

 maxv=kdm
 W=>infoMatrix;z=>infoVector
! geostrophy constraint:
      ofmax=86400./4./pi  ! inverse of fmax=4*pi/86400 [1/s].
      wGeo=5.0
      wLid=1.0
      r=1
      do k=1,maxV
        do j=2,(jdm-2)
         do i=2,(idm-2)
         if(mask(i,j,k)==1) then
          !
          ru=(f(i,j)+f(i,j-1))*0.5*ofmax
          rv=(f(i,j)+f(i-1,j))*0.5*ofmax
          !
          W(i,j,0,0,1,1,k)=W(i,j,0,0,1,1,k)+ ru*ru*wGeo
          W(i,j,0,0,2,2,k)=W(i,j,0,0,2,2,k)+ rv*rv*wGeo

          !
          !ru=1.
          ru=ru*r*ofmax/dy(i,j)
          !rv=1.
          rv=rv*r*ofmax/dx(i,j)

 !
          W(i,j,0,0,1,3,k)=W(i,j,0,0,1,3,k)+ru*wGeo
          W(i,j,0,-1,1,3,k)=W(i,j,0,-1,1,3,k)-ru*wGeo

          W(i,j,0,0,3,1,k)=W(i,j,0,0,3,1,k)+ru*wGeo
          W(i,j,0,1,3,1,k)=W(i,j,0,1,3,1,k)-ru*wGeo


          W(i,j,0,0,2,3,k)=W(i,j,0,0,2,3,k)-rv*wGeo
          W(i,j,-1,0,2,3,k)=W(i,j,-1,0,2,3,k)+rv*wGeo
          W(i,j,0,0,3,2,k)=W(i,j,0,0,3,2,k)-rv*wGeo
          W(i,j,1,0,3,2,k)=W(i,j,1,0,3,2,k)+rv*wGeo

          !

          ru=(r*ofmax*2/(dx(i,j)+dx(i+1,j)))**2
          rv=(r*ofmax*2/(dy(i,j)+dy(i,j+1)))**2
          !


          W(i,j,0,0,3,3,k)=W(i,j,0,0,3,3,k)+(ru+rv)*wGeo

          W(i,j,1,0,3,3,k)=W(i,j,1,0,3,3,k)-ru*wGeo
          W(i,j,0,0,3,3,k)=W(i,j,0,0,3,3,k)+ru*wGeo
          W(i,j,-1,0,3,3,k)=W(i,j,-1,0,3,3,k)-ru*wGeo

          W(i,j,0,1,3,3,k)=W(i,j,0,1,3,3,k)-rv*wGeo
          W(i,j,0,0,3,3,k)=W(i,j,0,0,3,3,k)+rv*wGeo
          W(i,j,0,-1,3,3,k)=W(i,j,0,-1,3,3,k)-rv*wGeo

          endif
          enddo
        enddo
      enddo

! weak lid (preservation of relative vorticity):
      r=0.0  ! fraction of membrane-ness to thinplate.
      r=r*0.5
      ds4=8.
      do k=1,maxV
       wThin=wLid/ds4
       do iww=3,3
        do j=2,(jdm-2)
        do i=2,(idm-2)
          W(i,j,0,0,iww,iww,k)=W(i,j,0,0,iww,iww,k)+8*wThin
          W(i,j,-1,0,iww,iww,k)=W(i,j,-1,0,iww,iww,k)-2*wThin
          W(i,j,1,0,iww,iww,k)=W(i,j,1,0,iww,iww,k)-2*wThin
          W(i,j,0,-1,iww,iww,k)=W(i,j,0,-1,iww,iww,k)-2*wThin
          W(i,j,0,1,iww,iww,k)=W(i,j,0,1,iww,iww,k)-2*wThin

          W(i-1,j,0,0,iww,iww,k)=W(i-1,j,0,0,iww,iww,k)+(1+r)*wThin
          W(i-1,j,1,0,iww,iww,k)=W(i-1,j,1,0,iww,iww,k)-2*wThin
          W(i-1,j,2,0,iww,iww,k)=W(i-1,j,2,0,iww,iww,k)+(1-r)*wThin
          W(i+1,j,-2,0,iww,iww,k)=W(i+1,j,-2,0,iww,iww,k)+(1-r)*wThin
          W(i+1,j,-1,0,iww,iww,k)=W(i+1,j,-1,0,iww,iww,k)-2*wThin
          W(i+1,j,0,0,iww,iww,k)=W(i+1,j,0,0,iww,iww,k)+(1+r)*wThin

          W(i,j,0,-1,iww,iww,k)=W(i,j,0,-1,iww,iww,k)-2*wThin
          W(i,j,0,1,iww,iww,k)=W(i,j,0,1,iww,iww,k)-2*wThin


          W(i,j-1,0,0,iww,iww,k)=W(i,j-1,0,0,iww,iww,k)+(1+r)*wThin
          W(i,j-1,0,1,iww,iww,k)=W(i,j-1,0,1,iww,iww,k)-2*wThin
          W(i,j-1,0,2,iww,iww,k)=W(i,j-1,0,2,iww,iww,k)+(1-r)*wThin
          W(i,j+1,0,-2,iww,iww,k)=W(i,j+1,0,-2,iww,iww,k)+(1-r)*wThin
          W(i,j+1,0,-1,iww,iww,k)=W(i,j+1,0,-1,iww,iww,k)-2*wThin
          W(i,j+1,0,0,iww,iww,k)=W(i,j+1,0,0,iww,iww,k)+(1+r)*wThin
        enddo
        enddo
       enddo
      enddo

      ! diagonal fortification:
      r=ofmax/sin(3./180.*pi)  ! 3-degree = equatorial deformation radius.
      do k=1,maxV
       wOne=0.0
       do iww=1,3
        do j=2,jdm-2
        do i=2,idm-2
          if( W(i,j,0,0,iww,iww,k) .eq. 0.) then
            W(i,j,0,0,iww,iww,k)=wOne
          else
            W(i,j,0,0,iww,iww,k)=W(i,j,0,0,iww,iww,k)+wOne
          endif
          !
          if( iww.eq.1 ) then
            ru=abs(f(i,j)+f(i,j-1))*0.5*r
            W(i,j,0,0,iww,iww,k)=W(i,j,0,0,iww,iww,k)+exp(-ru)!*q(k)
          endif
          if( iww.eq.2 ) then
            rv=abs(f(i,j)+f(i-1,j))*0.5*r
            W(i,j,0,0,iww,iww,k)=W(i,j,0,0,iww,iww,k)+exp(-rv)!*q(k)
          endif
        enddo
        enddo
       enddo
      enddo
end subroutine

subroutine setInfoVectorM(idm,jdm,ldm,kdm,bo,vv)

      implicit none
      integer(i4)                        :: idm,jdm,kdm,ldm
      real(r4)                           :: bo(idm,jdm,ldm,kdm)
      real(r4)                           :: vv(idm,jdm,ldm,kdm)

      ! information pair:
      real,pointer                   ::  z(:,:,:,:)
      real,pointer                   ::  W(:,:,:,:,:,:,:)

      ! local variables:
      real(r4)                           :: r
      real(r4)                           :: c
      integer(r4)                        :: ia,ja

      ! typical local variables:

      integer(i4) ::  i,j,k,l,mw,nw,iw,jw,iww,jww  ! array/matrix indices.

      W=>infoMatrix;z=>infoVector

      z=0.0

       do  k=1,kdm
       do   l=1,ldm
        do j=1,jdm
         do i=1,idm
            if(bo(i,j,l,k)<spval) then
                z(i,j,l,k)=z(i,j,l,k)+bo(i,j,l,k)*weightD
                W(i,j,0,0,l,l,k)=W(i,j,0,0,l,l,k)+weightD

            endif
          enddo
        enddo
       enddo
      enddo

       do k=1,kdm
       do l=1,ldm
        do j=1,jdm
         do i=1,idm
          if(W(i,j,0,0,l,l,k).eq.0.) W(i,j,0,0,l,l,k)=1.0
         enddo
        enddo
       enddo
      enddo

end subroutine

subroutine roifPCGM(po,idm,jdm,ldm,kdm,mask)


      ! a Preconditioned Conjugate Gradient solution for the information pair.


      implicit none
       
      integer(i4)                        :: idm,jdm,kdm,ldm
      real(r4) :: po(idm,jdm,ldm,kdm)
      integer(i4) :: mask(idm,jdm,kdm)
      integer(i4) :: ki

      ! information pair:
      real(r4),pointer ::  b(:,:,:,:),A(:,:,:,:,:,:,:)
      !equivalence (b,infoVector),(A,infoMatrix)
      ! print flag:
      logical diagPCG
      parameter (diagPCG=.true.)  ! if set, prints iteration residuals.

      ! local variables:
      real(r4) ::  x(idm,jdm,ldm,kdm),r(idm,jdm,ldm,kdm),p(idm,jdm,ldm,kdm)
      real(r4),target,allocatable ::  z(:,:,:,:)
      real(r4),pointer            ::  w(:,:,:,:)
      real(r4) ::  pw,alphaPCG,betaPCG,rhoNew,rhoOld
      integer(i4) ::  iteration,io,jo
      ! typical local variables:
      integer(i4) ::  i,j,k,l,mw,nw,iw,jw,iww,jww  ! array/matrix indices.

      allocate(z(idm,jdm,ldm,kdm))

       b=>infoVector;A=>infoMatrix
       w=>z

       p=0.0

       
       do k=1,kdm
       do l=1,ldm
        do j=1,jdm
         do i=1,idm
         if(mask(i,j,k)==1) then
           x(i,j,l,k)=0.
           r(i,j,l,k)=b(i,j,l,k)
         endif  
         enddo
        enddo
       enddo
      enddo 
      
      do iteration=1,maxIteration


      rhoNew=0.

!$OMP parallel do schedule(static) private(k,l,j,i) reduction(+:rhonew) collapse(2)

       do k=1,kdm
       do l=1,ldm
        do j=1,jdm
         do i=1,idm
         if(mask(i,j,k)==1) then

         !z(i,j,iw,k)=r(i,j,iw,k)  ! no preconditioning.
          ! preconditioning:
          if(abs(A(i,j,0,0,l,l,k)).gt.1.) then
            z(i,j,l,k)=r(i,j,l,k)/A(i,j,0,0,l,l,k)
          else
            z(i,j,l,k)=r(i,j,l,k)
          endif
          ! preconditioning done.
          rhoNew=rhoNew+r(i,j,l,k)*z(i,j,l,k)
         endif
          enddo
        enddo
       enddo
      enddo 
!$OMP END parallel do 



      if(iteration.gt.1) betaPCG=rhoNew/rhoOld

 !$OMP parallel do schedule(static) private(k,l,j,i) collapse(2)
   
    do  k=1,kdm
      do l=1,ldm 
       do j=1,jdm
        do i=1,idm
        if(mask(i,j,k)==1) then

          if(iteration.eq.1) then
            p(i,j,l,k)=z(i,j,l,k)
          else
            p(i,j,l,k)=z(i,j,l,k)+betaPCG*p(i,j,l,k)
          endif
          w(i,j,l,k)=0.
         endif
          enddo
        enddo
      enddo
     enddo 
!$OMP END parallel do     

!$OMP parallel do schedule(static) private(k,l,jw,j,i,iw,nw,jo,mw,io) collapse(2)    

    do k=1,kdm
     do jw=1,ldm

        do j=1,jdm
        do i=1,idm
              if(mask(i,j,k)==1) then
          do iw=1,ldm
              do nw=-maxW,maxW
                jo=j+nw
                if((jo.ge.1).and.(jo.le.jdm)) then
                  do mw=-maxW,maxW
                    io=i+mw
                   if((io.ge.1).and.(io.le.idm)) then
                      if(A(i,j,mw,nw,iw,jw,k).ne.0.) then

      w(i,j,iw,k)=w(i,j,iw,k)+A(i,j,mw,nw,iw,jw,k)*p(io,jo,jw,k)

                      endif
                    endif
                  enddo
                endif
        enddo
       enddo  

        endif
        enddo
        enddo
       enddo
      enddo
    

!$OMP END parallel do      


     pw=0.
!$OMP parallel do schedule(static) private(k,l,j,i) reduction(+:pw) collapse(2) 

      do k=1,kdm
       do l=1,ldm
        do j=1,jdm
        do i=1,idm
        if(mask(i,j,k)==1) then
          pw=pw+p(i,j,l,k)*w(i,j,l,k)
        endif
       enddo
      enddo
     enddo
    enddo 

!$OMP END parallel do    


        alphaPCG=rhoNew/pw

!$OMP parallel do schedule(static) private(k,l,j,i) collapse(2)         
      do k=1,kdm  
       do l=1,ldm
        do j=1,jdm
         do i=1,idm
           if(mask(i,j,k)==1) then
             x(i,j,l,k)=x(i,j,l,k)+alphaPCG*p(i,j,l,k)
             r(i,j,l,k)=r(i,j,l,k)-alphaPCG*w(i,j,l,k)
           endif
         enddo
        enddo
       enddo
      enddo 
!$OMP END parallel do  
        
        rhoOld=rhoNew
       
!        if(diagPCG) write(6,90) iteration,rhoOld,alphaPCG,betaPCG
        if(iteration==maxIteration) then
         write(6,90) iteration,rhoOld,alphaPCG,betaPCG
        endif
90      format(' ....PCG ',I6,': ',3G12.3)

      enddo ! iteration


    do  k=1,kdm
     do l=1,ldm
      do j=1,jdm
      do i=1,idm
        if(mask(i,j,k)==1) then
         po(i,j,l,k)=x(i,j,l,k)
        endif
       enddo
      enddo
     enddo 
    enddo

where(abs(po)<0.001) po=0.0
  
 end subroutine

!--------------------------------------------------
!      subroutine statInfo(idm,jdm,kdm,ldm)

!              implicit none
!
!               integer(i4)                        :: idm,jdm
!      real(r4) rave,rmax,rr
!      ! typical local variables:
!      integer(i4) ::  i,j,k,l,mw,nw,iw,jw,iww,jww  ! array/matrix indices.

!      ! information pair:
!      real(r4),pointer  :: z(:,:,:,:),W(:,:,:,:,:,:)
!      integer(i4)   :: nmax,mmax,imax,jmax

!      z=>infoVector
!      W=>infomatrix

!      nmax=0
!      mmax=0
!      imax=0
!      jmax=0
!        rmax=0.
!        rave=0.
!        do mw=-maxW,maxW
!        do nw=-maxW,maxW
!          do i=1,idm
!          do j=1,jdm
!            rr=abs(W(i,j,mw,nw,k,l))
!            if(rmax.lt.rr) then
!              rmax=rr
!              imax=i
!              jmax=j
!              mmax=mw
!              nmax=nw
!            endif
!            rave=rave+rr
!          enddo
!          enddo
!        rave=rave/real(25*idm*jdm*kdm)
!      write(*,99) rmax,imax,jmax,mmax,nmax,rave 
!99      format('W',G12.3,4I3,G12.3)
!      enddo
!      enddo
!      end
    
end module mmgmrf      
