module mobs

use mkinds
use mconstants
use mncio


  implicit none

  TYPE lyrobs
   integer(i4)                   :: id
   character(Len=3)              :: typ
   integer(i4)                   :: pid
   integer(i4)                   :: qc
   real(r4)                      :: lon
   real(r4)                      :: lat
   integer(i4)                   :: lyr
   real(r4)                      :: time
   real(r4)                      :: age
   real(r4)                      :: val
   real(r4)                      :: err
   real(r4)                      :: bkg
   real(r4)                      :: inov
   real(r4)                      :: res
   real(r4)                      :: grdi
   real(r4)                      :: grdj
  END TYPE lyrobs

  TYPE lvlobs
   integer(i4)                   :: id
   character(Len=3)              :: typ
   integer(i4)                   :: pid
   real(r4)                      :: lon
   real(r4)                      :: lat
   integer(i4)                   :: lvl
   real(r4)                      :: time
   real(r4)                      :: age
   real(r4)                      :: val(5)
   real(r4)                      :: err(5)
   real(r4)                      :: bkg(5)
   real(r4)                      :: inov(5)
   real(r4)                      :: res(5)
   real(r4)                      :: grdi(5)
   real(r4)                      :: grdj(5)
  END TYPE lvlobs

  TYPE lvlinov
   integer(i4)                   :: id
   character(Len=3)              :: typ
   integer(i4)                   :: pid
   real(r4)                      :: lon
   real(r4)                      :: lat
   integer(i4)                   :: lvl
   real(r4)                      :: time
   real(r4)                      :: age
   real(r4)                      :: val
   real(r4)                      :: inov
   real(r4)                      :: res
   real(r4)                      :: bkg
   real(r4)                      :: grdi
   real(r4)                      :: grdj
   integer(i4)                   :: valid
  END TYPE lvlinov

! observation locations


    character(len=cl)                  ::  l3sla_data_location
    character(len=cl)                  ::  l4sla_data_location
    character(len=cl)                  ::  l3csst_data_location
    character(len=cl)                  ::  l4sst_data_location
    character(len=cl)                  ::  l2i_sst_location
    character(len=cl)                  ::  drifter_data_location
    character(len=cl)                  ::  tops_master_data_location
    character(len=cl)                  ::  l2sss_data_location
    character(len=cl)                  ::  tsis_obs_location
    character(len=cl)                  ::  sclim_data_location
    character(len=cl)                  ::  dclim_data_location
    character(len=cl)                  ::  argo_profiles_location
    character(len=cl)                  ::  gts_profiles_location
    character(len=cl)                  ::  pies_profiles_location
    character(len=cl)                  ::  godae_profiles_location
    character(len=cl)                  ::  cobs_location
    character(4)                       ::  cobs_pfx

    character(:),allocatable           :: obsFile
    character(:),allocatable           :: cobsFile

!    logical(bl)                        :: l3sla
!    logical(bl)                        :: l4sla
!    logical(bl)                        :: l3sst
!    logical(bl)                        :: l4sst
    logical(bl)                        :: c_argo
    logical(bl)                        :: p_argo
    logical(bl)                        :: p_tesac
    logical(bl)                        :: p_bathy
    logical(bl)                        :: p_buoy

    real(r4)                           :: lnmn
    real(r4)                           :: lnmx
    real(r4)                           :: ltmn
    real(r4)                           :: ltmx

    real(r4)                           :: glnmn
    real(r4)                           :: glnmx
    real(r4)                           :: gltmn
    real(r4)                           :: gltmx

    integer(i4)                        :: l4obs_nx
    integer(i4)                        :: l4obs_ny
    real(r4),allocatable               :: l4obs_lon(:)
    real(r4),allocatable               :: l4obs_lat(:)


    integer(i4)                        :: nobs,nactiveobs,ntobs
    type (lyrobs), allocatable         :: work_lyr_obs(:,:,:),profile_lyr_obs(:,:,:)
    type (lvlobs), allocatable         :: work_lvl_obs(:,:,:),profile_lvl_obs(:,:,:)
    type(lvlinov), allocatable         :: zinov(:,:,:),work_inov(:,:,:)

! sla params
   character(25)             :: l4_sla_file_pfx
   integer(i4)               :: naltim
   integer(i4)               :: sla_twin_start
   integer(i4)               :: sla_twin_end
   real(r4)                  :: sla_tau_corr 
   character(5),allocatable  :: altim(:)
   
   ! sst params
   character(25)             :: l4_sst_file_pfx
   character(25)             :: l3c_sst_file_pfx
   integer(i4)               :: sst_twin_start
   integer(i4)               :: sst_twin_end
   real(r4)                  :: sst_tau_corr 

! profile data parameters

integer(i4)                  :: p_twin_start    ! time start lookback        
integer(i4)                  :: p_twin_end      ! time end
integer(i4)                  :: nmin            ! minimum no of points
real(r4)                     :: dmin            ! minimum depth
integer(i4)                  :: nsig            ! outliers sigma
integer(i4)                  :: noutliers       ! max number of outliers
integer(i4)                  :: pxtnd_typ       ! climtalogy or forecast
real(r4)                     :: bot_check_tol   ! check for bottom comatibility
real(r4)                     :: drho_tol        ! instability check
real(r4)                     :: p_tau_corr      ! decorrelation time
real(r4)                     :: sd_max          ! min  varriabnce
real(r4)                     :: td_max          ! min variance
real(r4)                     :: scl_bot         ! vertical scale at the bottom
logical(bl)                  :: lqc             ! super obs or not
logical(bl)                  :: lob_lvl         ! full water column or only observed levels
logical(bl)                  :: lreport         ! dump text output

logical(bl)                  :: l2profile
logical(bl)                  :: l3sla
logical(bl)                  :: l3sst
logical(bl)                  :: l2sst
logical(bl)                  :: l2sss
logical(bl)                  :: l4sla
logical(bl)                  :: l2uv
logical(bl)                  :: l4sst
logical(bl)                  :: l4uv

contains 

subroutine  initialize_obs()

        call read_obs_location()
        call set_obs_system()
        call read_global_window()
        call read_analysis_window()
        call read_profile_analysis_params()
        call read_sst_analysis_params()
        call read_sla_analysis_params()
        
end subroutine 

subroutine set_l4obs_grid(cnx,cny,clon,clat)
        
        implicit none

        integer(i4)  :: cnx
        integer(i4)  :: cny
        real(r4)     :: clon(cnx)
        real(r4)     :: clat(cny)


        
        l4obs_nx=cnx
        l4obs_ny=cny
        allocate(l4obs_lon(cnx))
        allocate(l4obs_lat(cny))
        l4obs_lon=clon
        l4obs_lat=clat



end subroutine        








subroutine read_obs_location()

      NAMELIST /obs_loc/l3sla_data_location,l4sla_data_location,l3csst_data_location,&
        l2i_sst_location,l4sst_data_location,drifter_data_location,l2sss_data_location,&
        tsis_obs_location,sclim_data_location,dclim_data_location,argo_profiles_location,&
        gts_profiles_location,tops_master_data_location,cobs_location,cobs_pfx

        OPEN(21,FILE='tsis.nlist')
        READ(21,NML=obs_loc)
        CLOSE(21)

    end subroutine

subroutine set_obs_system()
       NAMELIST /obs_system/l3sla,l4sla,l3sst,l4sst,c_argo,p_argo,p_tesac,p_buoy,p_bathy,l2sst,l2sss,l2uv,l4uv
       OPEN(21,FILE='tsis.nlist')
       READ(21,NML=obs_system)
       CLOSE(21)
end subroutine


subroutine read_profile_analysis_params()
NAMELIST /profile_params/p_twin_start,p_twin_end,p_tau_corr,lqc,lob_lvl,drho_tol,&
                                  nsig,noutliers,dmin,nmin,bot_check_tol,sd_max,td_max,scl_bot,&
                                  pxtnd_typ,lreport
OPEN(21,FILE='tsis.nlist')
READ(21,NML=profile_params)
CLOSE(21)
end subroutine

subroutine read_sst_analysis_params()
NAMELIST /sst_params/sst_twin_start,sst_twin_end,sst_tau_corr,l4_sst_file_pfx,l3c_sst_file_pfx
OPEN(21,FILE='tsis.nlist')
READ(21,NML=sst_params)
CLOSE(21)
end subroutine

subroutine read_sla_analysis_params()
NAMELIST /sla_params/sla_twin_start,sla_twin_end,sla_tau_corr,naltim,l4_sla_file_pfx
NAMELIST /altim_list/ altim

OPEN(21,FILE='tsis.nlist')
READ(21,NML=sla_params)
allocate(altim(naltim))
READ(21,NML=altim_list)
CLOSE(21)
end subroutine


subroutine read_analysis_window()
IMPLICIT NONE
NAMELIST /analysis_window/lnmn,lnmx,ltmn,ltmx
! read analysis parameters
OPEN(21,FILE='tsis.nlist')
READ(21,NML=analysis_window)
CLOSE(21)
end subroutine

subroutine read_global_window()
IMPLICIT NONE
NAMELIST /global_obs_window/glnmn,glnmx,gltmn,gltmx
! read analysis parameters
OPEN(21,FILE='tsis.nlist')
READ(21,NML=global_obs_window)
CLOSE(21)
end subroutine


END MODULE
