module modyssea
 use mkinds
 use mconstants
 use mdates
 use mncio
 use mobs
 use msearch

 real(r4),allocatable      :: l3sst_lon(:)
 real(r4),allocatable      :: l3sst_lat(:)
 real(r4),allocatable      :: l3sst_val(:)
 real(r4),allocatable      :: l3sst_err(:)
 real(r8),allocatable      :: l3sst_time(:)
 integer(i4),parameter     :: nmx=5000000
 integer(i4)               :: nl3sst


contains

        subroutine odyssea_l3sst_2cobs(cdate,iflag)
implicit none


character(*)              :: cdate
integer(i4)               :: gnx
integer(i4)               :: gny
real(r4)                  :: scl_sst,off_sst,fill_value_sst
real(r4)                  :: scl_err,off_err,fill_value_err
real(r4)                  :: scl_dt,off_dt,fill_value_dt
real(r4)                  :: grdi,grdj,ptlon
integer(i4)               :: i,j,k
character(cl)             :: l3c_sst_prefix
character(:),allocatable  :: inFile
logical(bl)               :: lexist

real(r4),allocatable      :: tmp_lon(:)
real(r4),allocatable      :: tmp_lat(:)
real(r4),allocatable      :: tmp_sst(:,:)
real(r4),allocatable      :: tmp_sst_err(:,:)
real(r4),allocatable      :: tmp_time(:,:)
real(r4)                  :: mtime
integer(i4)               :: ncount
integer(i4)               :: iflag
integer(i4)               :: istat

allocate(l3sst_val(nmx))
allocate(l3sst_lon(nmx))
allocate(l3sst_lat(nmx))
allocate(l3sst_err(nmx))
allocate(l3sst_time(nmx))

call cymd2hday(cdate,mtime)

inFile=trim(l3csst_data_location)//cdate(1:4)//"/"//trim(l3c_sst_file_pfx)//"_"//cdate(1:8)//".nc"
INQUIRE(FILE=inFile, EXIST=lexist)
 if(lexist) then
   write(*,*) "....L3C SST FILE....", inFile

   call nciopn(inFile,fid)
         call ncioin(inFile,fid,"lon",gnx)
         call ncioin(inFile,fid,"lat",gny)


         allocate(tmp_lon(gnx))
         allocate(tmp_lat(gny))
         allocate(tmp_sst(gnx,gny))
         allocate(tmp_sst_err(gnx,gny))
         allocate(tmp_time(gnx,gny))

         call nciorv(inFile,fid,"lon",tmp_lon)
         call nciorv(inFile,fid,"lat",tmp_lat)

         call nciorv(inFile,fid,"adjusted_sea_surface_temperature",tmp_sst)
         call nciora(inFile,fid,"adjusted_sea_surface_temperature","_FillValue",fill_value_sst)
         call nciora(inFile,fid,"adjusted_sea_surface_temperature","scale_factor",scl_sst)
         call nciora(inFile,fid,"adjusted_sea_surface_temperature","add_offset",off_sst)

         call ncchkv(inFile,fid,"adjusted_standard_deviation_error",istat)
         if(istat==0) then
         call nciorv(inFile,fid,"adjusted_standard_deviation_error",tmp_sst_err)
         call nciora(inFile,fid,"adjusted_standard_deviation_error","scale_factor",scl_err)
         call nciora(inFile,fid,"adjusted_standard_deviation_error","_FillValue",fill_value_sst)
         call nciora(inFile,fid,"adjusted_standard_deviation_error","add_offset",off_err)
         endif

         call ncchkv(inFile,fid,"sses_standard_deviation",istat)
         if(istat==0) then
           call nciorv(inFile,fid,"sses_standard_deviation",tmp_sst_err)
           call nciora(inFile,fid,"sses_standard_deviation","scale_factor",scl_err)
           call nciora(inFile,fid,"sses_standard_deviation","_FillValue",fill_value_sst)
           call nciora(inFile,fid,"sses_standard_deviation","add_offset",off_err)
         endif

         call nciorv(inFile,fid,"sst_dtime",tmp_time)
         call ncchkat(inFile,fid,"sst_dtime","scale_factor",istat)
         if(istat==0) then
         call nciora(inFile,fid,"sst_dtime","scale_factor",scl_dt)
         call nciora(inFile,fid,"sst_dtime","add_offset",off_dt)
         else
         scl_dt=1.0
         off_dt=0.0
         endif
         call nciora(inFile,fid,"sst_dtime","_FillValue",fill_value_dt)

         call nciocl(inFile,fid)

         where(tmp_sst>fill_value_sst)
                 tmp_sst=tmp_sst*scl_sst+off_sst
                 tmp_sst_err=tmp_sst_err*scl_err+off_err
                 tmp_time=tmp_time*scl_dt+off_dt
         else where
                 tmp_sst=spval
                 tmp_sst_err=spval
                 tmp_time=spval
         end where

         ncount=0

         do j=1,gny
          do i=1,gnx
           if(tmp_sst(i,j)<spval) then 
           ncount=ncount+1
           l3sst_lon(ncount)=tmp_lon(i)
           l3sst_lat(ncount)=tmp_lat(j)
           l3sst_val(ncount)=tmp_sst(i,j)-273.15
           l3sst_err(ncount)=tmp_sst_err(i,j)
           l3sst_time(ncount)=tmp_time(i,j)/86400.0 + mtime
           endif
         enddo
         enddo
          nl3sst=ncount   

          if(nl3sst>0) then
                  iflag=0
                  Write(*,*) "....Total no of L3c SST observations points: ",nl3sst
          else
                  iflag=-1
          endif

          else
                iflag=-1
          Write(*,*) "....L3C SST file not present...."
          endif

end subroutine


end module modyssea            
