module mprep

use mkinds
use mconstants
use mglobals
use mobs
use mgrid
use mlinint
use msplint
use msearch
use mdates
use mlprofiles
use mstate
use msstat
use mminfo
use mobsio
use mstio
use mtsclim


integer(i4)              :: ilvl
integer(i4)              :: ivar
integer(i4)              :: nvalid
character(:),allocatable :: var

real(r4),parameter    :: t_inov_max=5.0
real(r4),parameter    :: s_inov_max=3.0
real(r4),parameter    :: p_inov_max=300.0
real(r4),parameter    :: rho_inov_max=0.03
real(r4),parameter    :: mx_thkerr=100.0

contains 

! initialize_xprep()
! prep_profiles




subroutine initialize_xprep()
implicit none

integer(i4)                  :: k
logical(bl)                  :: lexist

!initialize analysis grid first for spmd processing

call initialize_analysis_grid()

!call set analysis date
call set_analysis_date()
call set_model_info()
call initialize_obs()
call read_stats_location()


tsis_stat_file=trim(tsis_stat_location)//"tops_"//trim(pfx)//"_stats.nc"

INQUIRE(FILE=tsis_stat_file, EXIST=lexist)
 IF(lexist) then
            write(*,*) "....stats file is .... " ,"tops_"//trim(pfx)//"_stats.nc"
 ELSE
          write(*,*) "....no stats file ...."
          write(*,*) "....stop ...."
         stop
ENDIF

call init_stat(analysis_date,pfx)
call init_sclim_grid(sclim_data_location)
analysis_levels=cdpth

CALL malloc_state(start,count)
CALL getFCstate(trim(ForecastFile),trim(fcmodel),c2r)


end subroutine

subroutine prep_profiles()        


 implicit none

! character(*)                :: analysis_date
! character(*)                :: pfx
 
 character(10)               :: wanted_date
 real(r4),allocatable        :: plon(:)
 real(r4),allocatable        :: plat(:)
 real(r4),allocatable        :: ptime(:)
 integer(i4),allocatable        :: pvalid(:)
 real(r4),allocatable        :: tz(:,:)
 real(r4),allocatable        :: sz(:,:)
 real(r4),allocatable        :: sct(:,:)
 real(r4),allocatable        :: scs(:,:)
 real(r4),allocatable        :: dct(:,:)
 real(r4),allocatable        :: dcs(:,:)
 real(r4),allocatable        :: tzerr(:)
 real(r4),allocatable        :: szerr(:)
 integer(i4),allocatable        :: it(:,:)
 integer(i4),allocatable        :: is(:,:)

 real(r4),allocatable        :: dp(:)
 real(r4),allocatable        :: tl(:)
 real(r4),allocatable        :: sl(:)
 real(r4),allocatable        :: tm(:)
 real(r4),allocatable        :: sm(:)
 real(r4),allocatable        :: pl(:)
 real(r4),allocatable        :: pm(:)
 real(r4),allocatable        :: rl(:)
 real(r4),allocatable        :: terr(:)
 real(r4),allocatable        :: serr(:)
 real(r4),allocatable        :: thkerr(:)
 real(r4),allocatable        :: derr(:)

 real(r4),allocatable        :: cdpth(:)
 real(r4),allocatable        :: bt(:)
 real(r4),allocatable        :: bs(:)
 integer(i4)                 :: nprof
 integer(i4)                 :: ntot
 integer(i4)                 :: ncount
 integer(i4)                 :: nlvl
 real(r4),allocatable        :: depth(:)
 real(r4)                    :: grdi
 real(r4)                    :: grdj
 real(r4)                    :: mld
 integer(i4)                 :: i,j,k,kmx,klast,nobsgrp
 integer(i4)                 :: ii,jj
 real(r4)                    :: date_offset,wday,wsday
 logical(bl)                 :: lexist
 real(r4)                    :: error_growth_factor
 integer(i4)                 :: nobs_typ
 integer(i4)                 :: ninov_typ
 integer(i4)                 :: nprofmx
 real(r4)                    :: ftinov,dtinov,stinov
 real(r4)                    :: fsinov,dsinov,ssinov

 nprofmx=5000
!!Alex for eddy simulaitons
 nprofmx=7500
 nobs_typ=4
 ninov_typ=2
 nlvl=nz

 allocate(work_lyr_obs(nobs_typ,nl,nprofmx))
 allocate(work_inov(ninov_typ,nz,nprofmx))

call cymd2hday(analysis_date,wday)
nprof=0
ncount=0
!1st pass to get the no of profiles
do ii=p_twin_start,p_twin_end
  wsday=wday+real(ii,r4)
  call hday2cymd(wsday,wanted_date)
  cobsFile=trim(cobs_location)//analysis_date(1:4)//"/tsis_"//trim(cobs_pfx)//"_"//wanted_date//".nc"
  INQUIRE(FILE=cobsFile, EXIST=lexist)
  if(lexist) then
    call nciopn(cobsFile,fid)
    call nciopn(cobsFile,fid)
    !call ncioin(cobsFile,fid,"nobsgrp",nobsgrp)
    !if( .not. allocated(obsgrp))  allocate(obsgrp(nobsgrp))
    call nciorv(cobsFile,fid,"obsgrp",obsgrp)
   if(obsgrp(l2_tsprofiles)==1) then
     call ncioin(cobsFile,fid,"nprof",nprof)
     call ncioin(cobsFile,fid,"depth",nlvl)
     ncount=ncount+nprof
     call nciocl(cobsFile,fid)
   endif ! profiles in file
  endif  ! tsisdb file exists
  
!Write(*,'(a,2I5)') "...."//trim(cobsFile),nprof,ncount
enddo

  allocate(depth(nlvl))
  allocate(tz(ncount,nlvl))
  allocate(sz(ncount,nlvl))
  allocate(dct(ncount,nlvl))
  allocate(dcs(ncount,nlvl))
  allocate(sct(ncount,nlvl))
  allocate(scs(ncount,nlvl))
  allocate(it(ncount,nlvl))
  allocate(is(ncount,nlvl))
  allocate(plon(ncount))
  allocate(plat(ncount))
  allocate(ptime(ncount))
  allocate(pvalid(ncount))

  allocate(dp(nl))
  allocate(tl(nl))
  allocate(sl(nl))
  allocate(tm(nl))
  allocate(sm(nl))
  allocate(pm(nl+1))
  allocate(rl(nl))
  allocate(pl(nl+1))
  allocate(serr(nl))
  allocate(terr(nl))
  allocate(thkerr(nl))
  allocate(derr(nl))


  allocate(cdpth(nl))
  allocate(bt(nlvl))
  allocate(bs(nlvl))

ncount=0
nprof=0
do ii=p_twin_start,p_twin_end
  wsday=wday+real(ii,r4)
  call hday2cymd(wsday,wanted_date)
  cobsFile=trim(cobs_location)//analysis_date(1:4)//"/tsis_"//trim(cobs_pfx)//"_"//wanted_date//".nc"
  !cobsFile=trim(cobs_location)//analysis_date(1:4)//"/tsis_cobs_"//wanted_date//".nc"

  INQUIRE(FILE=cobsFile, EXIST=lexist)
  if(lexist) then
    call nciopn(cobsFile,fid)
    call nciorv(cobsFile,fid,"obsgrp",obsgrp)
   if( obsgrp(l2_tsprofiles)==1) then
    call ncioin(cobsFile,fid,"nprof",nprof)
    call nciorv(cobsFile,fid,"plon",plon(ncount+1:ncount+nprof),[1],[nprof])
    call nciorv(cobsFile,fid,"plat",plat(ncount+1:ncount+nprof),[1],[nprof])
    call nciorv(cobsFile,fid,"depth",depth)
    call nciorv(cobsFile,fid,"ptime",ptime(ncount+1:ncount+nprof),[1],[nprof])
    call nciorv(cobsFile,fid,"temp",tz(ncount+1:ncount+nprof,:),[1,1],[nprof,nlvl])
    call nciorv(cobsFile,fid,"saln",sz(ncount+1:ncount+nprof,:),[1,1],[nprof,nlvl])
    call nciorv(cobsFile,fid,"sctemp",sct(ncount+1:ncount+nprof,:),[1,1],[nprof,nlvl])
    call nciorv(cobsFile,fid,"scsaln",scs(ncount+1:ncount+nprof,:),[1,1],[nprof,nlvl])
    call nciorv(cobsFile,fid,"dctemp",dct(ncount+1:ncount+nprof,:),[1,1],[nprof,nlvl])
    call nciorv(cobsFile,fid,"dcsaln",dcs(ncount+1:ncount+nprof,:),[1,1],[nprof,nlvl])
    call nciorv(cobsFile,fid,"it",it(ncount+1:ncount+nprof,:),[1,1],[nprof,nlvl])
    call nciorv(cobsFile,fid,"is",is(ncount+1:ncount+nprof,:),[1,1],[nprof,nlvl])
     ncount=ncount+nprof
   endif ! profiles in file
   call nciocl(cobsFile,fid)
  endif  ! file exists
 
enddo


ntot=ncount
print*,"Alex total number of profiles :",ntot
ncount=0
if(ntot>0) then
l2profile=.true.



 do i=1,ntot 


   grdi=ridx(mlon(:,1),plon(i))
   grdj=ridx(mlat(1,:),plat(i))

   ii=int(grdi)
   jj=int(grdj)

   if(grdi >rsfill .and. grdj>rsfill ) then
   if (plon(i) > lnmn .and. plon(i)<lnmx .and. plat(i)>ltmn .and.plat(i)<ltmx &
           .and. mdepth(int(grdi),int(grdj))>cr_depth .and. mdepth(int(grdi),int(grdj))<spval) then

           !ncount=ncount+1
   !print *,i,plon(i),plat(i),mdepth(int(grdi),int(grdj))

   
   dp(:)=fthk(int(grdi),int(grdj),:)
   tm(:)=ftem(int(grdi),int(grdj),:)
   sm(:)=fsal(int(grdi),int(grdj),:)
   mld=fmld(int(grdi),int(grdj))



         pm(1)=0.0
         do k=1,nl
          pm(k+1)=pm(k)+dp(k)*qonem
         enddo

         cdpth(1)=0.0
         cdpth(nl)=pm(nl+1)
         do k=2,nl-1
           cdpth(k)=0.5*(pm(k+1)+pm(k))
         enddo

         klast=nl
         do k=2,nl
         if(abs(cdpth(k)-cdpth(k-1))<=kbot_tol) then
                 klast=k-1
                 exit
         endif
         enddo


         do k=1,nl
         if(tm(k)>=spval) then
                  tm(k)=tm(k-1)
                  sm(k)=sm(k-1)
          endif
          enddo




          kmx=nlvl
          do k=1,nlvl
            if(sz(i,k)==rsfill) then
             kmx=k-1
             exit
            endif
           enddo

           if(kmx/=0) then
                   ncount=ncount+1

!           do k=1,nlvl
!            print *, k,kmx,tz(i,k),sz(i,k)
!           enddo


         kmx=nlvl
         do k=1,nlvl
         if(depth(k)>cdpth(nl)) then
          kmx=k-1
          exit
         endif
         enddo

         call akspline(klast,cdpth(1:klast),tm(1:klast),kmx,depth(1:kmx),bt(1:kmx))
         call akspline(klast,cdpth(1:klast),sm(1:klast),kmx,depth(1:kmx),bs(1:kmx))

         if(kmx<nlvl) then

         do k=kmx+1,nlvl
           bt(k)=bt(kmx)
           bs(k)=bs(kmx)
         enddo

         endif

         pvalid(i)=qcheck(nz,tz(i,:),sz(i,:),bt,bs,sct(i,:),scs(i,:),dct(i,:),dcs(i,:))

         if(pvalid(i)==0 .and. lreport) then

        do k=1,nlvl
              ftinov=0.0
              fsinov=0.0
              dtinov=0.0
              dsinov=0.0
              stinov=0.0
              ssinov=0.0
         if(tz(i,k)>rsfill .and. bt(k)>rsfill) then
          ftinov=tz(i,k)-bt(k)
          fsinov=sz(i,k)-bs(k)
          else
          ftinov=rsfill
          fsinov=rsfill
         endif

         if(tz(i,k)>rsfill .and. dct(i,k)>rsfill) then
          dtinov=tz(i,k)-dct(i,k)
          dsinov=sz(i,k)-dcs(i,k)
         else
          dtinov=rsfill
          dsinov=rsfill
         endif

         if(tz(i,k)>rsfill .and. sct(i,k)>rsfill) then

          stinov=tz(i,k)-sct(i,k)
          ssinov=sz(i,k)-scs(i,k)
         else
                 stinov=rsfill
                 ssinov=rsfill
         endif

          write(*,'(2i5,10F10.2)') ncount,k,plon(i),plat(i),mdepth(ii,jj),depth(k),ftinov,dtinov,stinov,fsinov,dsinov,ssinov

          enddo

         endif


         call pmerge(nz,depth,tz(i,:),bt,it(i,:))
         call pmerge(nz,depth,sz(i,:),bs,is(i,:))

                

          work_inov(1,:,ncount)%inov=rsfill
          work_inov(2,:,ncount)%inov=rsfill
          work_inov(1,:,ncount)%val=rsfill
          work_inov(2,:,ncount)%val=rsfill
          work_inov(:,:,ncount)%valid=0

         do k=1,kmx
        if(abs(tz(i,k)-bt(k))<t_inov_max .and. abs(sz(i,k)-bs(k))<s_inov_max .and. it(i,k)==1 .and. is(i,k)==1) then
          work_inov(1:2,k,ncount)%lon=plon(i)
          work_inov(1:2,k,ncount)%lat=plat(i)
          work_inov(1:2,k,ncount)%time=ptime(i)
          work_inov(1,k,ncount)%val=tz(i,k)
          work_inov(2,k,ncount)%val=sz(i,k)
          work_inov(1,k,ncount)%bkg=bt(k)
          work_inov(2,k,ncount)%bkg=bs(k)
          work_inov(1,k,ncount)%inov=tz(i,k)-bt(k)
          work_inov(2,k,ncount)%inov=sz(i,k)-bs(k)
          work_inov(1:2,k,ncount)%grdi=ii
          work_inov(1:2,k,ncount)%grdj=jj
          work_inov(1:2,k,ncount)%valid=1
        else
          work_inov(1:2,k,ncount)%lon=plon(i)
          work_inov(1:2,k,ncount)%lat=plat(i)
          work_inov(1:2,k,ncount)%time=ptime(i)
          work_inov(1,k,ncount)%val=tz(i,k)
          work_inov(2,k,ncount)%val=sz(i,k)
          work_inov(1,k,ncount)%bkg=bt(k)
          work_inov(2,k,ncount)%bkg=bs(k)
          work_inov(1,k,ncount)%inov=tz(i,k)-bt(k)
          work_inov(2,k,ncount)%inov=sz(i,k)-bs(k)
          work_inov(1:2,k,ncount)%grdi=ii
          work_inov(1:2,k,ncount)%grdj=jj
          work_inov(1:2,k,ncount)%valid=0
          !print *, tz(i,k),bt(k),sz(i,k),bs(k)
         endif
        enddo


         error_growth_factor=(abs(wday-wsday))/p_tau_corr

        
         if(all(isosigma/=0)) then
         mlayer=isosigma(ii,jj,:)
         endif ! needed for vsigma
         pl=0.0
         tl=0.0
         rl=0.0
         terr=0.0
         serr=0.0
         thkerr=0.0
         call zprofile2lyr(kmx,nl,error_growth_factor,depth,tz(i,1:kmx),sz(i,1:kmx),mlayer,pl,tl,sl,rl,terr,serr,thkerr)
       

         ! find the last layer with info
           kmx=nl
            do k=1,nl-1
            if(abs(tl(K+1)-tl(k))<kbot_tol .and. abs(sl(K+1)-sl(k))<kbot_tol .and. abs(pl(K+1)-pl(k))<kbot_tol )then
                kmx=k
            exit
            endif
           enddo

           do k=1,nl
           !write(*,'(I5,8F16.4)') k,pl(k),tl(k),terr(k),sl(k),serr(k),rl(k),derr(k),thkerr(k)

               if(k<kmx) then
                 derr(k)=min(abs(dsig(tl(k),sl(k),terr(k),serr(k),sigver)),max(abs(mlayer(k)-rl(k)),sigdmin))
                 derr(k)=max(sigdmin,derr(k))
                 thkerr(k)=max(sigdpmin, (factdp*pl(k) -(factdp*pl(k)-sigdpmin)*(derr(k)-sigdmin)/derr(k)))
                 thkerr(k)=min(mx_thkerr,thkerr(k))
               else
                 derr(k)=sigdmin
                 thkerr(k)=sigdpmin
                endif
           enddo

            !write(*,'(a,I5,7F12.4)') "Profile No: ", ncount,wday,ptime(i),plon(i),plat(i),mdepth(ii,jj),pl(kmx),real(kmx)
           !write(*,*) "****************************************************************"
           do k=1,nl
!           if(k<kmx .and. grdi>700 .and. grdi<870 .and. grdj>530 .and. grdj<610) then
!            print *, k,tl(k)-tm(k),sl(k)-sm(k)
           !if(k<kmx) write(*,'(I5,8F16.4)') k,pl(k),tl(k),terr(k),sl(k),serr(k),rl(k),derr(k),thkerr(k)
!           write(*,'(I5,8F16.4)') k,pl(k),tl(k),terr(k),sl(k),serr(k),rl(k),derr(k),thkerr(k)
!           endif
             work_lyr_obs(1,k,ncount)%lon=plon(i)
             work_lyr_obs(1,k,ncount)%lat=plat(i)
             work_lyr_obs(1,k,ncount)%time=ptime(i)
             work_lyr_obs(1,k,ncount)%qc=pvalid(i)
             work_lyr_obs(1,k,ncount)%val=tl(k)
             work_lyr_obs(1,k,ncount)%err=terr(k)
             work_lyr_obs(1,k,ncount)%grdi=ii
             work_lyr_obs(1,k,ncount)%grdj=jj
             if(abs(tl(k)-tm(k))<t_inov_max) then
              work_lyr_obs(1,k,ncount)%inov=tl(k)-tm(k)
             else
                     work_lyr_obs(1,k,ncount)%inov=t_inov_max*(tl(k)-tm(k))/(abs(tl(k)-tm(k)))
             endif
             if(k<kmx) then
                work_lyr_obs(1,k,ncount)%id=1
             else
                work_lyr_obs(1,k,ncount)%id=0
             endif

             work_lyr_obs(2,k,ncount)%lon=plon(i)
             work_lyr_obs(2,k,ncount)%lat=plat(i)
             work_lyr_obs(2,k,ncount)%time=ptime(i)
             work_lyr_obs(2,k,ncount)%qc=pvalid(i)
             work_lyr_obs(2,k,ncount)%val=sl(k)
             work_lyr_obs(2,k,ncount)%err=serr(k)
             work_lyr_obs(2,k,ncount)%grdi=ii
             work_lyr_obs(2,k,ncount)%grdj=jj
             if(abs(sl(k)-sm(k))<s_inov_max) then
             work_lyr_obs(2,k,ncount)%inov=sl(k)-sm(k)
             else
                     work_lyr_obs(2,k,ncount)%inov=s_inov_max*(sl(k)-sm(k))/(abs(sl(k)-sm(k)))
             endif
             
             if(k<kmx) then
                work_lyr_obs(2,k,ncount)%id=1
             else
                work_lyr_obs(2,k,ncount)%id=0
             endif

             work_lyr_obs(3,k,ncount)%lon=plon(i)
             work_lyr_obs(3,k,ncount)%lat=plat(i)
             work_lyr_obs(3,k,ncount)%time=ptime(i)
             work_lyr_obs(3,k,ncount)%qc=pvalid(i)
             work_lyr_obs(3,k,ncount)%val=rl(k)
             work_lyr_obs(3,k,ncount)%err=derr(k)
             work_lyr_obs(3,k,ncount)%grdi=ii
             work_lyr_obs(3,k,ncount)%grdj=jj
             work_lyr_obs(3,k,ncount)%inov=rl(k)-sig(tm(k),sm(k),sigver)
!             print *, ncount,rl(k),tm(k),sm(k),sig(tm(k),sm(k),sigver)
             if(k<kmx) then
                work_lyr_obs(3,k,ncount)%id=1
             else
                work_lyr_obs(3,k,ncount)%id=0
             endif

             work_lyr_obs(4,k,ncount)%lon=plon(i)
             work_lyr_obs(4,k,ncount)%lat=plat(i)
             work_lyr_obs(4,k,ncount)%time=ptime(i)
             work_lyr_obs(4,k,ncount)%qc=pvalid(i)
             work_lyr_obs(4,k,ncount)%val=pl(k)
             work_lyr_obs(4,k,ncount)%err=thkerr(k)
             work_lyr_obs(4,k,ncount)%grdi=ii
             work_lyr_obs(4,k,ncount)%grdj=jj
             if(abs(pl(k)-pm(k))<p_inov_max) then
             work_lyr_obs(4,k,ncount)%inov=pl(k)-pm(k)
             else
                     work_lyr_obs(4,k,ncount)%inov=p_inov_max*(pl(k)-pm(k))/(abs(pl(k)-pm(k)))
             endif
             if(k<kmx) then
                work_lyr_obs(4,k,ncount)%id=1
             else
                work_lyr_obs(4,k,ncount)%id=0
             endif
           enddo

   endif ! kmx==0
       endif
      endif
      enddo
! tsis_profile_model_grid

allocate(profile_lyr_obs(nobs_typ,nl,ncount))
allocate(zinov(ninov_typ,nz,ncount))
profile_lyr_obs=work_lyr_obs(:,:,1:ncount)
zinov=work_inov(:,:,1:ncount)

call write_tsis_obsFile(analysis_date,pfx,ncount,nl,nobs_typ,nx,ny)
call write_tsis_inovFile(analysis_date,pfx,mlon(:,1),mlat(1,:),ncount,nz,ninov_typ,nx,ny)

write(*,*) "....Total no of profiles: ", ncount

else
        write(*,*) "....no profile obs .... "

endif



end subroutine prep_profiles

subroutine pmerge(nz,cdpth,t,ct,its)
      implicit none

      integer(i4)  :: nz
      real(r4)     :: cdpth(nz)
      real(r4)     :: t(nz)
      real(r4)     :: ct(nz)
      integer(i4),intent(inout)  :: its(nz)
      integer      :: i,j,k,nin,nout
      real(r4)     :: fact
      integer      :: kmn_dt,kmx_dt
      integer      :: kmn_ct,kmx_ct
      real(r4)     :: vscl_bot


      fact=0.
      vscl_bot=300.0


      ! if observed profiles go to the bottom then return the observed profile
      ! as is.
      kmx_dt=nz
      kmn_dt=1

     do k=1,nz
       if(t(k)<=rsfill) then
          kmx_dt=max(1,k-1)
          exit
       endif
     enddo

    kmx_ct=nz
      kmn_ct=1


     do k=nz,2
       if(abs(ct(k)-ct(k-1))>1.e-5) then
          kmx_ct=max(1,k)
          exit
       endif
     enddo

   if(kmx_dt>=kmx_ct) then
      do k=1,nz
       if(k>kmx_ct) then
         t(k)=t(k-1)
       endif
      enddo
   else ! normal case where profile ends before the model bottom
     
       do k=kmx_dt+1,nz
           fact=exp(-1.0*((cdpth(k)-cdpth(kmx_dt))/vscl_bot))
           t(k)=ct(k)+(t(kmx_dt)-ct(kmx_dt))*fact
           its(k)=2
      enddo
   endif


end subroutine


integer(i4) FUNCTION qcheck(nz,tz,sz,ft,fs,st,ss,dt,ds)  RESULT(ival)

              implicit none
              integer(i4)    :: nz
              real(r4)       :: tz(:)
              real(r4)       :: sz(:)
              real(r4)       :: ft(:)
              real(r4)       :: fs(:)
              real(r4)       :: st(:)
              real(r4)       :: ss(:)
              real(r4)       :: dt(:)
              real(r4)       :: ds(:)

!              real(r4)       :: tstd(:)
!              real(r4)       :: sstd(:)


              real(r4)       :: ftinov,fsinov
              real(r4)       :: dtinov,dsinov
              real(r4)       :: stinov,ssinov


              integer(i4)    :: nbad
              integer(i4)    :: i,j,k
              real(r4),parameter :: t_tol=5.0,s_tol=3.0

              nbad=0

              ftinov=0.0
              fsinov=0.0
              dtinov=0.0
              dsinov=0.0
              stinov=0.0
              ssinov=0.0


              do k=1,nz
         if(tz(k)>rsfill .and. ft(k)>rsfill) then
          ftinov=tz(k)-ft(k)
          fsinov=sz(k)-fs(k)
          else
          ftinov=0.0
          fsinov=0.0        
         endif


         if(tz(k)>rsfill .and. dt(k)>rsfill) then
          dtinov=tz(k)-dt(k)
          dsinov=sz(k)-ds(k)
         else
          dtinov=0.0
          dsinov=0.0
         endif

         if(tz(k)>rsfill .and. st(k)>rsfill) then
         
          stinov=tz(k)-st(k)
          ssinov=sz(k)-ss(k)
         else
                 stinov=0.0
                 ssinov=0.0
         endif

          if((abs(ftinov)>t_tol .and. abs(dtinov)>t_tol .and. abs(stinov)>t_tol) .or. &
                  (abs(fsinov)>s_tol .and. abs(dsinov)>s_tol .and. abs(ssinov)>s_tol)) then
                nbad=nbad+1
         endif  

         enddo

          if(nbad>0) then 
                  ival=0
          else
                  ival=1
          endif

          end Function 







end module mprep      
