module mpsst

use mkinds
use mconstants
use mobs
use mgrid
use mlinint
use msplint
use msearch
use mdates
use mstate
use msstat
use mminfo
use mobsio
use mfld


contains 

! prep_l3sla


subroutine prep_l3sst()
implicit none


character(10)             :: wanted_date
integer(i4)               :: ncount
integer(i4)               :: ix
integer(i4)               :: jy
real(r4)                  :: wsday,wday
logical(bl)               :: lobs
integer(i4)               :: i,j,k,ii,jj
real(r4)                  :: il,jl
character(:),allocatable  :: inFile
character(:),allocatable  :: outFile
character(:),allocatable  :: cobsFile
logical(bl)               :: lexist
real(r4),allocatable      :: l3sst_grid_val(:,:)
real(r4),allocatable      :: l3sst_grid_err(:,:)
real(r4),allocatable      :: l3sst_grid_inov(:,:)
integer(i4),allocatable   :: idx(:,:)
integer(i4),allocatable   :: bin(:,:)
integer(i4)               :: nob_grp
integer(i4),allocatable   :: ob_grp_present(:)
real(r4),allocatable      :: slon(:)
real(r4),allocatable      :: slat(:)
real(r4),allocatable      :: sval(:)
real(r4),allocatable      :: serr(:)
integer(i4)               :: scount
real(r4),parameter        :: sst_err=0.15
integer(i4)               :: stat

call cymd2hday(analysis_date,wday)
allocate(l3sst_grid_val(nx,ny))
allocate(l3sst_grid_err(nx,ny))
allocate(l3sst_grid_inov(nx,ny))
allocate(idx(nx,ny))
allocate(bin(nx,ny))

l3sst_grid_val=0.0
l3sst_grid_err=0.0
l3sst_grid_inov=0.0

idx=0
bin=0

ncount=0
scount=0

!1st pass to get the no of profiles
do k=sst_twin_start,sst_twin_end
  wsday=wday+real(k,r4)
  call hday2cymd(wsday,wanted_date)
  cobsFile=trim(cobs_location)//"/"//analysis_date(1:4)//"/tsis_cobs_"//wanted_date//".nc"
  INQUIRE(FILE=cobsFile, EXIST=lexist)
  if(lexist) then
    call nciopn(cobsFile,fid)
    call ncioiv(cobsFile,fid,"l3sst_val",stat)
   if(stat==0) then
    call ncioin(cobsFile,fid,"l3sst",scount)
     allocate(slon(scount))
     allocate(slat(scount))
     allocate(sval(scount))
     allocate(serr(scount))
    call nciorv(cobsFile,fid,"l3sst_lon",slon)
    call nciorv(cobsFile,fid,"l3sst_lat",slat)
    call nciorv(cobsFile,fid,"l3sst_val",sval)
    call nciorv(cobsFile,fid,"l3sst_err",serr)
    call nciocl(cobsFile,fid)
      do i=1,scount
        ncount=ncount+1
        if(slon(i)>=lnmn .and. slon(i)<lnmx .and. slat(i)>=ltmn .and. slat(i)<ltmx) then

                ii=ridx(mlon(:,1),slon(i))
                jj=ridx(mlat(1,:),slat(i))

      if(ii/=-99.0 .and. jj/=-99.0) then
              if(mdepth(ii,jj)<spval) then
       l3sst_grid_val(ii,jj)=l3sst_grid_val(ii,jj)+sval(i)
       l3sst_grid_err(ii,jj)=serr(i)
       bin(ii,jj)=bin(ii,jj)+1
       endif
       endif


     endif
      enddo

      deallocate(slon,slat,sval,serr)

      endif ! l3sst in file
      endif ! exist

enddo

where(bin>=1)
        l3sst_grid_val=l3sst_grid_val/bin
        l3sst_grid_inov=l3sst_grid_val-fssh
        idx=1
else where
        l3sst_grid_val=spval
        l3sst_grid_inov=spval
end where

!print *, minval(l3sst_grid_val,mask=l3sst_grid_val<spval), maxval(l3sst_grid_val,mask=l3sst_grid_val<spval)
!print *, minval(l3sst_grid_inov,mask=l3sst_grid_inov<spval), maxval(l3sst_grid_inov,mask=l3sst_grid_inov<spval)


if(ncount>0) then
           write(*,*) "....Total no of L3 SST observations points: ",ncount
           write(*,*) "....No of L3 SST observations for bin averages: ",sum(bin)
           write(*,*) "....Final no of binned L3 SST observations points: ",sum(idx)


           l3sst=.true.
outFile=trim(tsis_obs_location)//"tsis_obs_"//trim(pfx)//"_"//analysis_date//".nc"
call nciopn(outFile,fid,'w')
call nciowv(outFile,fid,"l3sst",l3sst_grid_val)
call nciowv(outFile,fid,"l3sst_err",l3sst_grid_err)
call nciorv(outFile,fid,"obsgrp",obsgrp)
obsgrp(l3_sst)=1
call nciowv(outFile,fid,"obsgrp",obsgrp)
call nciocl(outFile,fid)

outFile=trim(tsis_obs_location)//"tsis_inov_"//trim(pfx)//"_"//analysis_date//".nc"
         call nciopn(outFile,fid,'w')
         call nciowv(outFile,fid,"l3sst_inov",l3sst_grid_inov)
         call nciocl(outFile,fid)

else

         write(*,*) "....no L3 SST observations ...."


 endif
end subroutine

subroutine prep_l4sst()
implicit none



integer(i4)               :: gnx
integer(i4)               :: gny
integer(i4)               :: snx
integer(i4)               :: sny
integer(i4)               :: is,js,ie,je
integer(i4)               :: rs,re,ss,se
integer(i4)               :: ix,jy
real(r4),allocatable      :: glon(:)
real(r4),allocatable      :: glat(:)
real(r4),allocatable      :: slon(:),slat(:),ssst(:,:),serr(:,:)
real(r4),allocatable      :: l4sst(:,:),l4sst_err(:,:),l4sst_inov(:,:)
real(r4)                  :: scl_sst,off_sst,fill_value_sst
real(r4)                  :: scl_err,off_err,fill_value_err
real(r4)                  :: ptlat,ptlon
integer(i4)               :: i,j,k
character(20)             :: l4_sst_file_pfx
character(:),allocatable  :: tsidbFile
character(:),allocatable  :: inFile
logical(bl)               :: lexist
real(r4)                  :: wday
logical(bl)               :: lpresent
integer(i4)               :: stat





call cymd2hday(analysis_date,wday)
allocate(l4sst(nx,ny))
allocate(l4sst_err(nx,ny))
allocate(l4sst_inov(nx,ny))


cobsFile=trim(cobs_location)//analysis_date(1:4)//"/"//"tsis_cobs_"//analysis_date//".nc"
INQUIRE(FILE=tsidbFile, EXIST=lexist)
 if(lexist) then
!   write(*,*) "Avhrr: OISST ", tsidbFile

   call nciopn(cobsFile,fid)
          call ncioiv(cobsFile,fid,"l4sst_val",stat)

       if(stat==0) then
         call ncioin(cobsFile,fid,"l4obs_lon",gnx)
         call ncioin(cobsFile,fid,"l4obs_lat",gny)

         allocate(glon(gnx))
         allocate(glat(gny))

         call nciorv(cobsFile,fid,"l4obs_lon",glon)
         call nciorv(cobsFile,fid,"l4obs_lat",glat)

         glat(1)=-90.0
         glat(gny)=90.0
         glon(1)=-180.0
         glon(gnx)=180.0

         rs=ridx(glon,lnmn)
         re=ridx(glon,lnmx)
         ss=ridx(glat,ltmn)
         se=ridx(glat,ltmx)


         snx=(re-rs)+1
         sny=(se-ss)+1
         is=int(rs)
         js=int(ss)

         allocate(slon(snx))
         allocate(slat(sny))
         allocate(ssst(snx,sny))
         allocate(serr(snx,sny))

         slon=glon(rs:re)
         slat=glat(ss:se)

         call nciorv(cobsFile,fid,"l4sst_val",ssst,[is,js],[snx,sny])
         call nciorv(cobsFile,fid,"l4sst_err",serr,[is,js],[snx,sny])


         call nciocl(cobsFile,fid)


         ! extend sst

         call hextend(ssst,snx,sny,50,spval)
         call hextend(serr,snx,sny,50,spval)


         call intp_fld (snx,sny,nx,ny,slon,slat,mlon,mlat,ssst,l4sst)
         call intp_fld (snx,sny,nx,ny,slon,slat,mlon,mlat,serr,l4sst_err)

         l4sst_inov=spval
         l4sst_inov=l4sst-fsst

         where(l4sst<-1.9 .or. l4sst>35.0 .or. mdepth>=spval)
                 l4sst=spval
                 l4sst_err=spval
                 l4sst_inov=spval
         end where

         write(*,*) "....l4 sst min/max: ", minval(l4sst,mask=l4sst<spval), maxval(l4sst,mask=l4sst<spval)
         inFile=trim(tsis_obs_location)//"tsis_obs_"//trim(pfx)//"_"//analysis_date//".nc"

         call nciopn(inFile,fid,'w')
         call nciowv(inFile,fid,"l4sst",l4sst)
         call nciowv(inFile,fid,"l4sst_err",l4sst_err)
         call nciorv(inFile,fid,"obsgrp",obsgrp)
         obsgrp(l4_sst)=1
         call nciowv(inFile,fid,"obsgrp",obsgrp)
         call nciocl(inFile,fid)

         inFile=trim(tsis_obs_location)//"tsis_inov_"//trim(pfx)//"_"//analysis_date//".nc"
         call nciopn(inFile,fid,'w')
         call nciowv(inFile,fid,"l4sst_inov",l4sst_inov)
         call nciocl(inFile,fid)
         endif !l4sst not infile

 else
  Write(*,*) "....l4 sst file not present...."
 endif
         end subroutine













end module mpsst      
