module mstio
use mkinds
use mncio
use mgrid
use mconstants

contains


subroutine create_stzf(filename,lon,lat,level,time,nx,ny,nz,nt)

character(*)  :: filename
integer(i4)   :: nx,ny,nz,nt
real(r4)      :: lon(nx),lat(ny)
real(r4)      :: time(nt), level(nz)

integer(i4)               :: i,j,iunit
integer(i4)               :: vtype
integer(i4)               :: vdims(4),vdim(1)

!create file
      call nciocf(filename,fid)

      !create dimension and coordinate variable 1
      vname="lon"
      vtype=NF90_Float
      vdim(1)=1
      call nciodd(filename,fid,vname,nx)
      call nciocv(filename,fid,vname,vtype,vdim)
      call nciowa(filename,fid,vname,'long_name','Longitude')
      call nciowa(filename,fid,vname,'standard_name','Longitude')
      call nciowa(filename,fid,vname,'units','degrees_east')

      !create dimension and coordinate variable 2
      vname="lat"
      vtype=NF90_Float
      vdim(1)=2
      call nciodd(filename,fid,vname,ny)
      call nciocv(filename,fid,vname,vtype,vdim)
      call nciowa(filename,fid,vname,'long_name','Latitude')
      call nciowa(filename,fid,vname,'standard_name','Latitude')
      call nciowa(filename,fid,vname,'units','degrees_north')

      !create dimension and coordinate variable 2
      vname="level"
      vtype=NF90_Float
      vdim(1)=3
      call nciodd(filename,fid,vname,nz)
      call nciocv(filename,fid,vname,vtype,vdim)
      call nciowa(filename,fid,vname,'long_name','depth')
      call nciowa(filename,fid,vname,'standard_name','depth')
      call nciowa(filename,fid,vname,'units','m')

      !create dimension and coordinate variable 2
      vname="time"
      vtype=NF90_Float
      vdim(1)=4
      call nciodd(filename,fid,vname,nt)
      call nciocv(filename,fid,vname,vtype,vdim)
      call nciowa(filename,fid,vname,'long_name','time')
      call nciowa(filename,fid,vname,'standard_name','time')
      call nciowa(filename,fid,vname,'units','days since 1900-12-31 00:00:00')

      !create forecast variables
      vname="hhxf"
      vtype=NF90_Float
      vdim3d=(/1,2,4/)
      call nciocv(filename,fid,vname,vtype,vdim3d)
      call nciowa(filename,fid,vname,'long_name','surface-heigh-increments')
      call nciowa(filename,fid,vname,'units','m')
      call nciowa(filename,fid,vname,'_FillValue',spval)

      vname="uuxf"
      vtype=NF90_Float
      vdims=(/1,2,3,4/)
      call nciocv(filename,fid,vname,vtype,vdims)
      call nciowa(filename,fid,vname,'long_name','u-velocity-increments')
      call nciowa(filename,fid,vname,'units','ms-1')
      call nciowa(filename,fid,vname,'_FillValue',spval)

      vname="vvxf"
      vtype=NF90_Float
      vdims=(/1,2,3,4/)
      call nciocv(filename,fid,vname,vtype,vdims)
      call nciowa(filename,fid,vname,'long_name','v-velocity increments')
      call nciowa(filename,fid,vname,'units','ms-1')
      call nciowa(filename,fid,vname,'_FillValue',spval)

      vname="ttxf"
      vtype=NF90_Float
      vdims=(/1,2,3,4/)
      call nciocv(filename,fid,vname,vtype,vdims)
      call nciowa(filename,fid,vname,'long_name','surface_temperature_increments')
      call nciowa(filename,fid,vname,'units','degC')
      call nciowa(filename,fid,vname,'_FillValue',spval)

      vname="ssxf"
      vtype=NF90_Float
      vdims=(/1,2,3,4/)
      call nciocv(filename,fid,vname,vtype,vdims)
      call nciowa(filename,fid,vname,'long_name','surface_temperature_increments')
      call nciowa(filename,fid,vname,'units','degC')
      call nciowa(filename,fid,vname,'_FillValue',spval)

      ! create innovation vectors

      vname="hinv"
      vtype=NF90_Float
      vdim3d=(/1,2,4/)
      call nciocv(filename,fid,vname,vtype,vdim3d)
      call nciowa(filename,fid,vname,'long_name','surface-heigh-increments')
      call nciowa(filename,fid,vname,'units','m')
      call nciowa(filename,fid,vname,'_FillValue',spval)

      vname="uinv"
      vtype=NF90_Float
      vdims=(/1,2,3,4/)
      call nciocv(filename,fid,vname,vtype,vdims)
      call nciowa(filename,fid,vname,'long_name','u-velocity-increments')
      call nciowa(filename,fid,vname,'units','ms-1')
      call nciowa(filename,fid,vname,'_FillValue',spval)

      vname="vinv"
      vtype=NF90_Float
      vdims=(/1,2,3,4/)
      call nciocv(filename,fid,vname,vtype,vdims)
      call nciowa(filename,fid,vname,'long_name','v-velocity increments')
      call nciowa(filename,fid,vname,'units','ms-1')
      call nciowa(filename,fid,vname,'_FillValue',spval)

      vname="tinv"
      vtype=NF90_Float
      vdims=(/1,2,3,4/)
      call nciocv(filename,fid,vname,vtype,vdims)
      call nciowa(filename,fid,vname,'long_name','surface_temperature_increments')
      call nciowa(filename,fid,vname,'units','degC')
      call nciowa(filename,fid,vname,'_FillValue',spval)

      vname="sinv"
      vtype=NF90_Float
      vdims=(/1,2,3,4/)
      call nciocv(filename,fid,vname,vtype,vdims)
      call nciowa(filename,fid,vname,'long_name','surface_temperature_increments')
      call nciowa(filename,fid,vname,'units','degC')
      call nciowa(filename,fid,vname,'_FillValue',spval)

      ! create clim

      vname="hclm"
      vtype=NF90_Float
      vdim3d=(/1,2,4/)
      call nciocv(filename,fid,vname,vtype,vdim3d)
      call nciowa(filename,fid,vname,'long_name','surface-heigh-increments')
      call nciowa(filename,fid,vname,'units','m')
      call nciowa(filename,fid,vname,'_FillValue',spval)

!      vname="uclm"
!      vtype=NF90_Float
!      vdims=(/1,2,3,4/)
!      call nciocv(filename,fid,vname,vtype,vdims)
!      call nciowa(filename,fid,vname,'long_name','u-velocity-increments')
!      call nciowa(filename,fid,vname,'units','ms-1')
!      call nciowa(filename,fid,vname,'_FillValue',2.00**100)

!      vname="vclm"
!      vtype=NF90_Float
!      vdims=(/1,2,3,4/)
!      call nciocv(filename,fid,vname,vtype,vdims)
!      call nciowa(filename,fid,vname,'long_name','v-velocity increments')
!      call nciowa(filename,fid,vname,'units','ms-1')
!      call nciowa(filename,fid,vname,'_FillValue',2.00**100)

      vname="tclm"
      vtype=NF90_Float
      vdims=(/1,2,3,4/)
      call nciocv(filename,fid,vname,vtype,vdims)
      call nciowa(filename,fid,vname,'long_name','surface_temperature_increments')
      call nciowa(filename,fid,vname,'units','degC')
      call nciowa(filename,fid,vname,'_FillValue',spval)

      vname="sclm"
      vtype=NF90_Float
      vdims=(/1,2,3,4/)
      call nciocv(filename,fid,vname,vtype,vdims)
      call nciowa(filename,fid,vname,'long_name','surface_temperature_increments')
      call nciowa(filename,fid,vname,'units','degC')
      call nciowa(filename,fid,vname,'_FillValue',spval)

      ! create inc vectors

      vname="hinc"
      vtype=NF90_Float
      vdim3d=(/1,2,4/)
      call nciocv(filename,fid,vname,vtype,vdim3d)
      call nciowa(filename,fid,vname,'long_name','surface-heigh-increments')
      call nciowa(filename,fid,vname,'units','m')
      call nciowa(filename,fid,vname,'_FillValue',spval)

      vname="uinc"
      vtype=NF90_Float
      vdims=(/1,2,3,4/)
      call nciocv(filename,fid,vname,vtype,vdims)
      call nciowa(filename,fid,vname,'long_name','u-velocity-increments')
      call nciowa(filename,fid,vname,'units','ms-1')
      call nciowa(filename,fid,vname,'_FillValue',spval)

      vname="vinc"
      vtype=NF90_Float
      vdims=(/1,2,3,4/)
      call nciocv(filename,fid,vname,vtype,vdims)
      call nciowa(filename,fid,vname,'long_name','v-velocity increments')
      call nciowa(filename,fid,vname,'units','ms-1')
      call nciowa(filename,fid,vname,'_FillValue',spval)

      vname="tinc"
      vtype=NF90_Float
      vdims=(/1,2,3,4/)
      call nciocv(filename,fid,vname,vtype,vdims)
      call nciowa(filename,fid,vname,'long_name','surface_temperature_increments')
      call nciowa(filename,fid,vname,'units','degC')
      call nciowa(filename,fid,vname,'_FillValue',spval)

      vname="sinc"
      vtype=NF90_Float
      vdims=(/1,2,3,4/)
      call nciocv(filename,fid,vname,vtype,vdims)
      call nciowa(filename,fid,vname,'long_name','surface_temperature_increments')
      call nciowa(filename,fid,vname,'units','degC')
      call nciowa(filename,fid,vname,'_FillValue',spval)









      call ncioed(filename,fid)

      ! write data into variables
      call nciowv(filename,fid,"lon",lon)
      call nciowv(filename,fid,"lat",lat)
      call nciowv(filename,fid,"level",level)
      call nciowv(filename,fid,"time",time)
      !call nciowv(filename,fid,"uinc",inc(:,:,:,:,1))
      !call nciowv(filename,fid,"vinc",inc(:,:,:,:,2))
      !call nciowv(filename,fid,"hinc",inc(:,:,:,:,3))
      !call nciowv(filename,fid,"tinc",inc(:,:,:,:,4))

      ! close file

      call nciocl(filename,fid)

end subroutine

subroutine ncfldio(filename,lon,lat,level,time,var,varname,std_name,units,nx,ny,nz,nt)

character(*)  :: filename,varname,std_name,units
character(:),allocatable :: vname
integer(i4)   :: nx,ny,nz,nt
real(r4)      :: lon(nx),lat(ny),var(nx,ny,nz,1)
real(r4)      :: time(nt), level(nz)

integer(i4)               :: fid,i,j,iunit
integer(i4)               :: vtype
integer(i4)               :: vdims(4),vdim(1)

!create file
      call nciocf(filename,fid)

      !create dimension and coordinate variable 1
      vname="lon"
      vtype=NF90_Float
      vdim(1)=1
      call nciodd(filename,fid,vname,nx)
      call nciocv(filename,fid,vname,vtype,vdim)
      call nciowa(filename,fid,vname,'long_name','Longitude')
      call nciowa(filename,fid,vname,'standard_name','Longitude')
      call nciowa(filename,fid,vname,'units','degrees_east')
      
      !create dimension and coordinate variable 2
      vname="lat"
      vtype=NF90_Float
      vdim(1)=2
      call nciodd(filename,fid,vname,ny)
      call nciocv(filename,fid,vname,vtype,vdim)
      call nciowa(filename,fid,vname,'long_name','Latitude')
      call nciowa(filename,fid,vname,'standard_name','Latitude')
      call nciowa(filename,fid,vname,'units','degrees_north')

      !create dimension and coordinate variable 2
      vname="level"
      vtype=NF90_Float
      vdim(1)=3
      call nciodd(filename,fid,vname,nz)
      call nciocv(filename,fid,vname,vtype,vdim)
      call nciowa(filename,fid,vname,'long_name','level')
      call nciowa(filename,fid,vname,'standard_name','level')
      call nciowa(filename,fid,vname,'units','m')

      !create dimension and coordinate variable 2
      vname="time"
      vtype=NF90_Float
      vdim(1)=4
      call nciodd(filename,fid,vname,nt)
      call nciocv(filename,fid,vname,vtype,vdim)
      call nciowa(filename,fid,vname,'long_name','time')
      call nciowa(filename,fid,vname,'standard_name','time')
      call nciowa(filename,fid,vname,'units','days since 1900-12-31 00:00:00')
      
      !create data variable 1
      vtype=NF90_Float
      vdims=(/1,2,3,4/)
      call nciocv(filename,fid,varname,vtype,vdims)
      call nciowa(filename,fid,varname,'long_name',std_name)
      call nciowa(filename,fid,varname,'standard_name',std_name)
      call nciowa(filename,fid,varname,'units',units)
      call nciowa(filename,fid,varname,'_FillValue',2.00**100)

      call ncioed(filename,fid)

      ! write data into variables
      call nciowv(filename,fid,"lon",lon)
      call nciowv(filename,fid,"lat",lat)
      call nciowv(filename,fid,"level",level)
      call nciowv(filename,fid,"time",time)
      call nciowv(filename,fid,varname,var)

      ! close file

      call nciocl(filename,fid)

end subroutine



end module
