module mtsclim

use mkinds
use mconstants
use mncio
use mstring
use mdates
use msearch

implicit none

integer(i4),parameter :: cnx=1440
integer(i4),parameter :: cny=720
integer(i4),parameter :: cnz=60

real(r4)              :: clon(cnx)
real(r4)              :: clat(cny)
real(r4)              :: cdpth(cnz)
real(r4)              :: czbot(cnx,cny)
real(r4),allocatable  :: at(:,:,:)
real(r4),allocatable  :: as(:,:,:)
real(r4),allocatable  :: ats(:,:,:)
real(r4),allocatable  :: ass(:,:,:)


character(cl)         :: tops_clim_loc
character(27)         :: cpfx

integer(i4)           :: kk


contains



       subroutine init_sclim_grid(clim_loc)

               implicit none

               character(*)                :: clim_loc
               character(:),allocatable    :: f_zbot


               tops_clim_loc=trim(clim_loc)
               f_zbot=trim(tops_clim_loc)//"tops_woa23_grid.nc"
               call nciopn(f_zbot,fid)
               call nciorv(f_zbot,fid,"lon",clon)
               call nciorv(f_zbot,fid,"lat",clat)
               call nciorv(f_zbot,fid,"depth",cdpth)
               call nciorv(f_zbot,fid,"zbot",czbot)
               call nciocl(f_zbot,fid)
               clon(1)=-180.0
               clon(1440)=180.0

       end subroutine        


       subroutine read_sclim(cdate,ipresent)

               implicit none

               character(*)                :: cdate
               character(:),allocatable    :: fsclim
               integer(i4)                 :: ipresent
               logical(bl)                 :: lexist

               allocate(at(cnx,cny,cnz))
               allocate(as(cnx,cny,cnz))
               allocate(ats(cnx,cny,cnz))
               allocate(ass(cnx,cny,cnz))

               fsclim="sclim_"//cdate//".nc"
               
               inquire(file=fsclim,exist=lexist)
               if(lexist) then
                  ipresent=0
                  call nciopn(fsclim,fid)
                  call nciorv(fsclim,fid,"ptemp",at)
                  call nciorv(fsclim,fid,"saln",as)
                  call nciorv(fsclim,fid,"tstd",ats)
                  call nciorv(fsclim,fid,"sstd",ass)
                  call nciocl(fsclim,fid)
               else
                       ipresent=1
               endif

       end subroutine

       subroutine sclim_mpts(cdtg,lon,lat,t,s,ts,ss,ncount,nz)

       implicit none
       character(*)  :: cdtg

       integer(i4)   :: ncount
       integer(i4)   :: nz
       real(r4)      :: lon(ncount)
       real(r4)      :: lat(ncount)

       real(r4)      :: t(ncount,nz)
       real(r4)      :: s(ncount,nz)
       real(r4)      :: ts(ncount,nz)
       real(r4)      :: ss(ncount,nz)
       real(r4)      :: pzbot(ncount)

       integer(i4)   :: ix
       integer(i4)   :: jy
       real(r4)      :: grdi
       real(i4)      :: grdj
       integer(i4)   :: i,j,k

       do kk=1,ncount
           grdi=ridx(clon,lon(kk))
           grdj=ridx(clat,lat(kk))
           ix=int(grdi)
           jy=int(grdj)
           pzbot(kk)=czbot(ix,jy)
          
           if(pzbot(kk)>0.0) then
           t(kk,:)=at(ix,jy,:)
           s(kk,:)=as(ix,jy,:)
           ts(kk,:)=max(0.05,ats(ix,jy,:))
           ss(kk,:)=max(0.01,ass(ix,jy,:))
       
           do k=2,nz
           if(t(kk,k)>=spval) then
                   t(kk,k)=t(kk,k-1)
                   s(kk,k)=s(kk,k-1)
                   ts(kk,k)=ts(kk,k-1)
                   ss(kk,k)=ss(kk,k-1)
           endif

           enddo    
           
          else
           t(kk,:)=rsfill
           s(kk,:)=rsfill
           ts(kk,:)=rsfill
           ss(kk,:)=rsfill
          endif
           !where(ts(kk,:)>=spval) ts(kk,:)=rsfill
           !where(ss(kk,:)>=spval) ss(kk,:)=rsfill       
       
       enddo

      
     end subroutine







end module      
