module mzprofiles

    use mkinds
    use mdates
    use mncio
    use mglobals
    use mlinint
    use mtsclim
    use margo
    use mgts
    use mobs
    use marmor

    implicit none

    integer(i4)                :: mxnpr
    integer(i4)                :: mxlvl
    integer(i4)                :: gpcnt
    integer(i4)                :: ntesac
    integer(i4)                :: nbathy
    integer(i4)                :: nbuoy
    integer(i4)                :: nargo
    integer(i4)                :: n1_tot
    integer(i4)                :: ncount

    integer(i4),allocatable    :: prnum(:)
    integer(i4),allocatable    :: prtyp(:)
    real(r4),allocatable       :: prlon(:)
    real(r4),allocatable       :: prlat(:)
    real(r4),allocatable       :: prtime(:)
    integer(i4),allocatable    :: prmxlvl(:)
    integer(i4),allocatable    :: prvalid(:)
    real(r4),allocatable       :: prdpth(:,:)
    real(r4),allocatable       :: prtemp(:,:)
    real(r4),allocatable       :: prsaln(:,:)

    integer(i4),allocatable    :: a_prnum(:)
    integer(i4),allocatable    :: a_prtyp(:)
    real(r4),allocatable       :: a_prlon(:)
    real(r4),allocatable       :: a_prlat(:)
    real(r4),allocatable       :: a_prtime(:)
    real(r4),allocatable       :: a_prbot(:)
    real(r4),allocatable       :: a_prtemp(:,:)
    real(r4),allocatable       :: a_prsaln(:,:)
    real(r4),allocatable       :: a_csaln(:,:)
    real(r4),allocatable       :: a_ctemp(:,:)
    real(r4),allocatable       :: a_ctstd(:,:)
    real(r4),allocatable       :: a_csstd(:,:)
    real(r4),allocatable       :: a_dcsaln(:,:)
    real(r4),allocatable       :: a_dctemp(:,:)

    integer(i4),allocatable    :: a_it(:,:)
    integer(i4),allocatable    :: a_is(:,:)
    integer(i4),allocatable    :: a_valid(:,:,:)

    ! bin sorted profiles

    integer(i4),allocatable    :: indx(:,:)
    integer(i4),allocatable    :: pnr(:,:)
    integer(i4),allocatable    :: it(:,:)
    integer(i4),allocatable    :: is(:,:)
    real(r4),allocatable       :: gp_temp(:,:)
    real(r4),allocatable       :: gp_saln(:,:)
    real(r4),allocatable       :: gp_lon(:)
    real(r4),allocatable       :: gp_lat(:)
    real(r4),allocatable       :: gp_time(:)
    integer(i4),allocatable    :: gp_valid(:,:,:)
    integer(i4),allocatable    :: gp_typ(:)

    ! climate profiles

    real(r4),allocatable       :: c_zbot(:)
    real(r4),allocatable       :: c_temp(:,:)
    real(r4),allocatable       :: c_saln(:,:)
    real(r4),allocatable       :: c_tstd(:,:)
    real(r4),allocatable       :: c_sstd(:,:) 
    
    ! gross check min/max
    real(r4),parameter         :: gt_min=-1.9
    real(r4),parameter         :: gt_max=34.0
    real(r4),parameter         :: gs_min=5.0
    real(r4),parameter         :: gs_max=45.0
    real(r4),parameter         :: ckval=-98.9


    contains

    subroutine init_profiles

        implicit none

        mxnpr=10000
        mxlvl=5000
        gpcnt=0
        ntesac=0
        nbathy=0
        nbuoy=0

        allocate(prlon(mxnpr))
        allocate(prlat(mxnpr))
        allocate(prtime(mxnpr))
        allocate(prmxlvl(mxnpr))
        allocate(prvalid(mxnpr))
        allocate(prdpth(mxnpr,mxlvl))
        allocate(prtemp(mxnpr,mxlvl))
        allocate(prsaln(mxnpr,mxlvl))
        allocate(prtyp(mxnpr))
        allocate(prnum(mxnpr))

        prvalid=0


    end subroutine

    subroutine process_profiles(cdate,nptot)

           
    implicit none

    character(*)             :: cdate

    integer(i4)              :: nptot
    integer(i4)              :: nstart
    integer(i4)              :: nend
    integer(i4)              :: ngts
    integer(i4)              :: ncount
    integer(i4)              :: i,j,k
    integer(i4)              :: iflag

    call set_obs_system()
    call init_profiles()

    npr_argo=0
    npr_tesac=0
    npr_buoy=0
    npr_bathy=0

    nstart=0

    if(p_argo) then
       call get_argo_files(cdate,mxnpr,mxlvl,prtyp,prmxlvl,prlon,prlat,prtime,prdpth,prtemp,prsaln,nstart,nend)
       nstart=nend
    endif

    if(c_argo) then
       call get_cargo_files(cdate,mxnpr,mxlvl,prtyp,prmxlvl,prlon,prlat,prtime,prdpth,prtemp,prsaln,nstart,nend)
       nstart=nend
    endif

    if(p_tesac) then
      call get_gts_tesac(cdate,mxnpr,mxlvl,prtyp,prmxlvl,prlon,prlat,prtime,prdpth,prtemp,prsaln,nstart,nend)
      nstart=nend
    endif

    if(p_bathy) then
      call get_gts_bathy(cdate,mxnpr,mxlvl,prtyp,prmxlvl,prlon,prlat,prtime,prdpth,prtemp,prsaln,nstart,nend)
      nstart=nend
    endif

    if(p_buoy) then
      call get_gts_buoy(cdate,mxnpr,mxlvl,prtyp,prmxlvl,prlon,prlat,prtime,prdpth,prtemp,prsaln,nstart,nend)
    endif

     ncount=npr_argo+npr_tesac+npr_bathy+npr_buoy

     ! Write out data to tsidb
   if(ncount>0) then
    allocate(a_prnum(ncount))
    allocate(a_prtyp(ncount))
    allocate(a_prlon(ncount))
    allocate(a_prlat(ncount))
    allocate(a_prtime(ncount))
    allocate(a_prbot(ncount))
    allocate(a_prtemp(ncount,cnz))
    allocate(a_prsaln(ncount,cnz))
    allocate(a_csaln(ncount,cnz))
    allocate(a_ctemp(ncount,cnz))
    allocate(a_ctstd(ncount,cnz))
    allocate(a_csstd(ncount,cnz))
    allocate(a_dctemp(ncount,cnz))
    allocate(a_dcsaln(ncount,cnz))
    allocate(a_it(ncount,cnz))
    allocate(a_is(ncount,cnz))
    allocate(a_valid(ncount,cnz,3))

    a_ctemp=rsfill
    a_csaln=rsfill
    a_ctstd=rsfill
    a_csstd=rsfill
    a_dctemp=rsfill
    a_dcsaln=rsfill
    a_valid=0


    do i=1,ncount
     a_prnum(i)=i
     a_prlon(i)=prlon(i)
     a_prlat(i)=prlat(i)
     a_prtime(i)=prtime(i)
     a_prtyp(i)=prtyp(i)

     call pr2stdz(prmxlvl(i),prdpth(i,1:prmxlvl(i)),&
                 prtemp(i,1:prmxlvl(i)),prsaln(i,1:prmxlvl(i)),&
                 cnz,cdpth,a_prtemp(i,:),a_prsaln(i,:),a_it(i,:),a_is(i,:))

    enddo
    nptot=ncount


    call read_sclim(cdate,iflag)
    if(iflag==0) then
      call sclim_mpts(cdate,a_prlon,a_prlat,a_ctemp,a_csaln,a_ctstd,a_csstd,ncount,cnz) 
    else
      write(*,*) ".... missing static clim files ...." 
    endif


!    do i=1,ncount
!         write(*,*) "************"
!         do k=1,cnz
 !        write(*,'(I5,7F12.2,2I5)') i,cdpth(k),a_prtemp(i,k),a_ctemp(i,k),a_ctstd(i,k),a_prsaln(i,k),a_csaln(i,k),a_csstd(i,k),a_it(i,k),a_is(i,k)
 !        enddo

 !   enddo



    !call armor_build_lookup(dclim_data_location,cdate)

    call init_armor(dclim_data_location,cdate,iflag)
    if(iflag==0) then
       call get_dclim_profiles(cdate,ncount,a_prlon,a_prlat,cnz,cdpth,a_dctemp,a_dcsaln)
    endif
!        do i=1,ncount
!         write(*,*) "************"
!         do k=1,cnz
!         write(*,'(I5,7F12.2,2I5)') i,cdpth(k),a_prtemp(i,k),a_dctemp(i,k),a_ctstd(i,k),a_prsaln(i,k),a_dcsaln(i,k),a_csstd(i,k),a_it(i,k),a_is(i,k)
!         enddo

!    enddo





    endif ! ncount >0


    end subroutine process_profiles



   subroutine pr2stdz(npin,pdpth,ptemp,psaln,npout,sdpth,itemp,isaln,itf,isf)

   implicit none


   integer(i4)        :: npin
   integer(i4)        :: npout
   real(r4)           :: pdpth(npin)
   real(r4)           :: ptemp(npin)
   real(r4)           :: psaln(npin)
   real(r4)           :: sdpth(npout)
   real(r4)           :: itemp(npout)
   real(r4)           :: isaln(npout)
   integer(i4)           :: itf(npout)
   integer(i4)           :: isf(npout)

   real(r4)           :: vdpth(npin)
   real(r4)           :: vvar(npin)
   real(r4)           :: mxdepth,mndepth
   integer(i4)        :: nvalid_s,nvalid_t,pvalid
   integer(i4)        :: nout
   integer(i4)        :: idpth(npout)

   integer(i4)        :: i,j,k
   integer(i4)        :: kmn_t,kmx_t,kmn_s,kmx_s


   itemp=rsfill
   isaln=rsfill

   nvalid_t=0

   where(pdpth<0.0) pdpth=0.0
   do i=1,npin
       if(ptemp(i)>gt_min .and. ptemp(i)<gt_max) then
        nvalid_t=nvalid_t+1
        vdpth(nvalid_t)=pdpth(i)
        vvar(nvalid_t)=ptemp(i)
       endif
   enddo

   if(nvalid_t>nmin) then
         mxdepth=maxval(vdpth(1:nvalid_t))
         mndepth=minval(vdpth(1:nvalid_t))
         kmx_t=int(ridx(sdpth,mxdepth))
         kmn_t=int(ridx(sdpth,mndepth))+1
         nout=(kmx_t-kmn_t)+1
         
         call linint(nvalid_t,vdpth,vvar,nout,sdpth(kmn_t:kmx_t),itemp(kmn_t:kmx_t))

         ! fix top of profile
         do i=1,kmn_t
             itemp(i)=vvar(1)
             itf(i)=2
         enddo

         do i=kmn_t+1,kmx_t
         itf(i)=1
         enddo

    endif

    nvalid_s=0
    vdpth=0.0
    vvar=0.0

     do i=1,npin
      if(psaln(i) >gs_min .and. psaln(i)<gs_max) then
        nvalid_s=nvalid_s+1
        vdpth(nvalid_s)=pdpth(i)
        vvar(nvalid_s)=psaln(i)
      endif
     enddo

     if(nvalid_s>nmin) then
        mxdepth=maxval(vdpth(1:nvalid_s))
        mndepth=minval(vdpth(1:nvalid_s))
        kmx_s=int(ridx(sdpth,mxdepth))
        kmn_s=int(ridx(sdpth,mndepth))
        nout=(kmx_s-kmn_s)+1
        call linint(nvalid_s,vdpth,vvar,nout,sdpth(kmn_s:kmx_s),isaln(kmn_s:kmx_s))

        do i=1,kmn_s
         isaln(i)=vvar(1)
         isf(i)=2
        enddo
        do i=kmn_s+1,kmx_s
         isf(i)=1
         enddo
     endif

end subroutine

subroutine setflg(nz,cdpth,t,s,it,is)
      implicit none

      integer(i4)  :: nz
      real(r4)     :: cdpth(nz)
      real(r4)     :: t(nz)
      real(r4)     :: s(nz)
      integer      :: i,j,k,nin,nout
      real(r4)     :: fact
      integer      :: kmn_t,kmx_t
      integer      :: kmn_s,kmx_s

      integer(i4)  :: is(nz)
      integer(i4)  :: it(nz)

      is=1
      it=1
      do j=1,nz
       if(t(j)<=rsfill) then
          it(j)=0
       endif
     enddo

      do j=1,nz
       if(s(j)<=rsfill) then
          is(j)=0
       endif
     enddo
   
      

end subroutine
      
 subroutine pxtnd(nz,cdpth,t,s,ct,cs,it,is)
      implicit none

      integer(i4)  :: nz
      real(r4)     :: cdpth(nz)
      real(r4)     :: t(nz)
      real(r4)     :: s(nz)
      real(r4)     :: ct(nz)
      real(r4)     :: cs(nz)
      real(r4)     :: ict(nz)
      real(r4)     :: pt(nz)
      real(r4)     :: tfact(nz),sfact(nz)
      integer      :: i,j,k,nin,nout
      real(r4)     :: fact
      integer      :: kmn_t,kmx_t
      integer      :: kmn_s,kmx_s

      integer(i4)  :: is(nz)
      integer(i4)  :: it(nz)

      tfact=0.
      sfact=0.

! convert clim pot temp to temp

      do j=1,nz
         !ict(j)=theta(cs(j),ct(j),0.0,cdpth(j))
      enddo

!       ict=ct
! merge clim temperature profile with profile data

kmx_t=nz

!do j=1,nz
!if(ict(j)==ict(j-1)) then
!        kmx_t=j
!        exit
!endif
!enddo

kmn_t=1

do j=1,nz
       if(t(j)<ckval) then
          kmx_t=max(1,j-1)
          exit
       endif
enddo

do k=kmx_t,nz
           fact=exp(-1.0*((cdpth(k)-cdpth(kmx_t))/scl_bot))
!           fact=1.0-((cdpth(k)-cdpth(kmx_t))/(cdpth(nz)-cdpth(kmx_t)))
           t(k)=ict(k)+(t(kmx_t)-ict(kmx_t))*fact
           tfact(k)=fact
           it(k)=2
enddo

! if it is a xbt, then derive salinity from climatology using the 
! displacement factor derived for temperature else use the displacement
! factor for salnity

if(all(s<ckval)) then

        do k=1,nz
           !fact=1.0-((cdpth(k)-cdpth(kmx_s))/(cdpth(nz)-cdpth(kmx_s)))
           !fact=exp(-1.0*((cdpth(k)-cdpth(kmx_s))/vscl))
           s(k)=cs(k)!+(s(kmx_s)-cs(kmx_s))*fact
           !sfact(k)=fact
           is(k)=2
        enddo

else

! merge clim salinity profile with profile data

kmx_s=nz
kmn_s=1
do j=1,nz
       if(s(j)<ckval) then
          kmx_s=max(1,j-1)
          exit
       endif
enddo

do k=kmx_s,nz
           !fact=1.0-((cdpth(k)-cdpth(kmx_s))/(cdpth(nz)-cdpth(kmx_s)))
           fact=exp(-1.0*((cdpth(k)-cdpth(kmx_s))/scl_bot))
           s(k)=cs(k)+(s(kmx_s)-cs(kmx_s))*fact
           sfact(k)=fact
           is(k)=2
enddo


endif

do k=1,nz
!t(k)=theta(s(k),t(k),cdpth(k),0.0)
t(k)=max(gt_min,min(t(k),gt_max))
s(k)=max(gs_min,min(s(k),gs_max))
!Write(*,'(I5,3x,7F12.2)') k,cdpth(k),t(k),ct(k),s(k),cs(k),tfact(k),sfact(k)
enddo
end subroutine

subroutine isvalid_profile(n,depth,t,s,nmin,dmin,lvalid)

implicit none

integer(i4)   :: n
integer(i4)   :: lvalid
integer(i4)   :: nmin

real(r4)      :: dmin
real(r4)      :: depth(n)
real(r4)      :: t(n)
real(r4)      :: s(n)

integer(i4)   :: valid_t(n),nt_valid
integer(i4)   :: valid_s(n),ns_valid
real(r4)      :: vtdepth(n),vsdepth(n)

integer       :: i



lvalid=0
valid_t=0
valid_s=0
vtdepth=10000 ! maxdepth
vsdepth=10000 ! max depth

do i=1,n
if(t(i) >ckval) then
        valid_t(i)=valid_t(i)+1
        vtdepth(i)=depth(i)
endif

if(s(i) >ckval) then
        valid_s(i)=valid_s(i)+1
        vsdepth(i)=depth(i)
endif
enddo

if( sum(valid_s) < nmin) s=rsfill



if(sum(valid_t)>=nmin .and. minval(vtdepth)<dmin) then
        lvalid=1
endif
!if(lvalid==0) then
! print *, sum(valid_s),sum(valid_t),minval(vtdepth),minval(vsdepth),lvalid
!endif
end subroutine







end module     
